/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package uwiki;

import java.sql.*;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.log4j.Logger;

import ddbc.commons.util.Jdbc;
import ddbc.commons.util.Strings;


/**
 * Class for data access functionality.
 * It's caller's responsability to provide and manage the JDBC connection.
 * @schemaVersion 41:0
 * @author Javier Aranda
 */
/* WHATIF (DAO instantiation) We could want to make the DAO a component and
 *	use different implementations for different scenarios.
 * For the sake of simplicity we assume only one type of access (JDBC),
 *  and thus define the methods as static.
 */
public class WData {
	/** Logger for internal use in the class. */
	private static final Logger LOGGER  = Logger.getLogger(WData.class);
	private static final SimpleDateFormat DF_TMS_MEDIAWIKI = new SimpleDateFormat("yyyyMMddHHmmss");

	/**
	 * Query used by the pool to validate connections.
	 */
	public static final String VALIDATION_QUERY = "SELECT COUNT(TITLE) FROM CUR";;

	/**
	 * Obtains the list of current page versions.
	 * @param conn
	 * @param target
	 * @throws SQLException
	 */
	public static void listCurrent(Connection conn, Collection<? super DataCur> target) throws SQLException {
		selectCurWhere(conn, target, " ORDER BY c.title ASC");
	}

	/**
	 * Obtains the current version for a page, given the page's title.
	 * @param conn
	 * @param title
	 * @return
	 * @throws SQLException
	 */
	public static DataCur getSingleCurByTitle(Connection conn, String title) throws SQLException {
		try {
			List<? super DataCur> tmpList = new ArrayList<Object>(1);
			selectCurWhere(conn, tmpList, " WHERE c.title=?", title);
			DataCur retCur = null;
			if (tmpList.size() > 0) {
				retCur = (DataCur)tmpList.get(0);
			}
			
			return retCur;
		} catch(SQLException sqle) {
			throw sqle; // Propagate
//		} catch(Exception e) {
//			LOGGER.error("unexpected error in getSingleCurByTitle", e);
//			throw new UnknownError("unexpected error in getSingleCurByTitle", e);
		}
	}

	/**
	 * Queries the current version data given the clauses (filtering, sorting, etc)
	 * @param conn
	 * @param target List where the data ({@link DataCur}) is added.
	 * @param where SQL clauses for the query (where, order by ...)
	 * @param parameters correlative values for the parameters specified in the clauses.
	 * @throws SQLException
	 */
	private static void selectCurWhere(Connection conn, Collection<? super DataCur> target,
			String where, Object... parameters) throws SQLException {
		// TODO Repasa
		PreparedStatement ps = conn.prepareStatement("SELECT c.cid, c.title, c.user, c.minor_edit, " +
				"c.text, c.timestamp, c.is_new, c.is_redirect FROM cur c " + where);
		Jdbc.fillParams(ps, parameters);
		ResultSet rs = ps.executeQuery();
		while(rs.next()) {
			DataCur cur = new DataCur();
			cur.setCid(rs.getInt("cid"));
			cur.setTitle(rs.getString("title"));
			cur.setUser(rs.getString("user"));
			cur.setMinorEdit(rs.getBoolean("minor_edit"));
			cur.setText(Strings.fromUtf8(rs.getBytes("text")));
			cur.setTimestamp(parseMWTms(rs.getString("timestamp")));
			cur.setNew(rs.getBoolean("is_new"));
			cur.setRedirect(rs.getBoolean("is_redirect"));
			
			target.add(cur);
		}
		rs.close(); ps.close();
	}

	/**
	 * Stores a new record of new current version data ( {@link DataCur} ).
	 * @param conn
	 * @param cur {@link DataCur} with the current version of page inserted.
	 * @throws SQLException
	 */
	public static void insertIntoCur(Connection conn, DataCur cur) throws SQLException {
		// NOTE: latest is "not null", but we don't new it's value until rid is calculated.
		PreparedStatement ps = conn.prepareStatement("INSERT INTO cur (title, user, minor_edit, text," +
				" timestamp, is_new, is_redirect) VALUES(?,?,?,?,?,?,?)");
		Jdbc.fillParams(ps, cur.getTitle(), cur.getUser(), cur.isMinorEdit(), Strings.toUtf8(cur.getText()),
				formatMWTms(cur.getTimestamp()), cur.isNew(), cur.isRedirect());
		ps.executeUpdate();
		ps.close();
		
	}

	/**
	 * Obtains the list of old versions of all pages.
	 * @param conn
	 * @param target Collection where the version information ( {@link DataOld} ) is added.
	 * @throws SQLException
	 */
	public static void listOld(Connection conn, Collection<? super DataOld> target) throws SQLException {
		selectOldWhere(conn, target, "");
	}
	
	/**
	 * Queries the current version data given the clauses (filtering, sorting, etc)
	 * @param conn
	 * @param target List where the data ({@link DataCur}) is added.
	 * @param where SQL clauses for the query (where, order by ...)
	 * @param parameters correlative values for the parameters specified in the clauses.
	 * @throws SQLException
	 */
	public static void listOldWhereTitle(Connection conn, String title, Collection<? super DataOld> target) throws SQLException {
		selectOldWhere(conn, target, " WHERE o.title=?", title);
	}
	
	/**
	 * Queries the old version data given the clauses (filtering, sorting, etc)
	 * @param conn
	 * @param target List where the data ({@link DataOld}) is added.
	 * @param where SQL clauses for the query (where, order by ...)
	 * @param parameters correlative values for the parameters specified in the clauses.
	 * @throws SQLException
	 */
	private static void selectOldWhere(Connection conn, Collection<? super DataOld> target,
			String where, Object... parameters) throws SQLException {
		PreparedStatement ps = conn.prepareStatement("SELECT o.oid, o.title, o.user, o.minor_edit, " +
				"o.text, o.timestamp from old o " + where);
		Jdbc.fillParams(ps, parameters);
		ResultSet rs = ps.executeQuery();
		while(rs.next()) {
			target.add(extractOldFromRS(rs));
		}
		rs.close(); ps.close();
	}

	/**
	 * Processes the ResultSet, extracting old version data.
	 * @param rs
	 * @param target Collection where the data ( {@link DataOld} ) is added.
	 * @throws SQLException
	 */
	private static DataOld extractOldFromRS(ResultSet rs) throws SQLException {
		DataOld old = new DataOld();
		old.setOid(rs.getInt("oid"));
		old.setTitle(rs.getString("title"));
		old.setUser(rs.getString("user"));
		old.setMinorEdit(rs.getBoolean("minor_edit"));
		old.setText(Strings.fromUtf8(rs.getBytes("text")));
		old.setTimestamp(rs.getTimestamp("timestamp"));

		return old;
	}

	/**
	 * Moves information from a current version into an old version.
	 * Used when a newer version is going to be inserted.
	 * @param conn
	 * @param title Title of the affected page.
	 * @throws SQLException
	 */
	public static void moveCurIntoOld(Connection conn, String title) throws SQLException {
		// Handle rollback in case the caller doesn't
		// CONSIDER Do so when performing > 1 updates in the same method.
		// CONSIDER Maybe we should assert in updating methods that autocommit is false.
		// We use an internal savepoint just in case the caller had some update pending commit (it's up to her to commit o rollback).
		boolean prevAutocommit = conn.getAutoCommit();
		if (prevAutocommit == true) { conn.setAutoCommit(false); }
		Savepoint sp = conn.setSavepoint();
		// A convenient way to know whether the update failed, instead of using many catch blocks.
		boolean success = false;
		try {
			// Copy data into old
			PreparedStatement ps = conn.prepareStatement("INSERT INTO old" +
					"(oid, title, user, minor_edit, text, timestamp) " +
					"SELECT cid, title, user, minor_edit, text, timestamp FROM cur WHERE title=?");
			Jdbc.fillParams(ps, title);
			ps.executeUpdate();
			ps.close();

			// Delete data from cur
			ps = conn.prepareStatement("DELETE FROM cur WHERE title=?");
			Jdbc.fillParams(ps, title);
			ps.executeUpdate();
			ps.close();
			
			success = true;
		} finally {
			if (!success) { conn.rollback(sp); }
			if (prevAutocommit == true) { conn.setAutoCommit(prevAutocommit); }
		}
	}

	private static java.util.Date parseMWTms(String s) {
		if (s == null) {
			return null;
		} else {
			try {
				return DF_TMS_MEDIAWIKI.parse(s);
			} catch(ParseException e) {
				LOGGER.warn("Read invalid date " + s);
				return null;
			}
		}
	}
	
	private static String formatMWTms(java.util.Date d) {
		if (d == null) {
			return null;
		} else {
			return DF_TMS_MEDIAWIKI.format(d);
		}
	}
}
