/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package uwiki;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.dbcp.ConnectionFactory;
import org.apache.log4j.Logger;

import ddbc.commons.jdbc.JdbcUtils;
import ddbc.commons.util.FlyWeight;
import ddbc.commons.util.Strings;


/**
 * Wiki Operations: Contains business logic and attached functionality.
 * @author Javier Aranda
 */
// WHATIF... (many instances) ¿could have many different wikis on the same JVM?
//	For the sake of simplicity we assume ony one wiki, so methods are static and
//	don't need to fetch instance by id.
public final class WOps {
	/** Logger for internal use in the class. */
	private static final Logger LOGGER = Logger.getLogger(WOps.class);
	
	/** Application configuration properties. */
	private static Properties config = null;
	/** SimpleDateFormat pattern used to format pages timestamps. */
	private static String patternTimestamp = "dd/MM/yyyy HH:mm";
	
	/**
	 * Set the application configuration.
	 * Invoked during de initialization.
	 * Performs database connection initilization (serves as early check).
	 * A simple config file should be available in WEB-INF/uwiki2.config.
	 * @param c Properties object with the configuration.
	 */
	static void setConfig(Properties c) {
		config = c;
		
		// ## Init Pool and check connection.
		try {
//			ConnectionFactory l_cf = JpConnectionFactoryBuilder.createFromProperties(c, "connectionFactory.", false);
			ConnectionFactory l_cf = JdbcUtils.createDriverConnectionFactoryFromProperties(c, "connectionFactory.", false);
			JdbcUtils.bindConnectionFactoryAsDriverManagerPool(l_cf, "uwiki");
	
			Connection conn = borrowConnection();
			//WData.listCurrent(conn, new ArrayList()); // Assume pool validates.
			returnConnection(conn);
		} catch(Exception e) {
			LOGGER.fatal("Error on jdbc pool initialization", e);
		}
		
		if (Strings.isEmpty(config.getProperty("format.timestamp"))) {
			patternTimestamp = config.getProperty("format.timestamp");
		}
	}

	/**
	 * Borrows a database connection.
	 * The connection should be returned with {@link #returnConnection(Connection)}
	 * @return
	 * @throws SQLException
	 */
	private static Connection borrowConnection() throws SQLException {
		return DriverManager.getConnection("jdbc:apache:commons:dbcp:uwiki");
	}

	/**
	 * Returns a borrowed database connection.
	 * The connection was obtained with {@link #borrowConnection()}
	 * @param conn
	 * @throws SQLException
	 */
	private static void returnConnection(Connection conn) throws SQLException {
		conn.close();
	}

	/**
	 * Formats a page timestamp.
	 * The format pattern is specified in the application configuration.
	 * @param date timestamp to format. can't be null.
	 * @return
	 */
	public static String formatTimestamp(Date date) {
		java.text.SimpleDateFormat df = FlyWeight.getDateFormat(patternTimestamp);
		String retFormat = df.format(date);
		return retFormat;
	}
	
	/**
	 * Comodity method for encoding and doing a send-redirect within the application
	 * @param request
	 * @param response
	 * @param pageName Name of the page (must begin with "/" from the context root).
	 * @throws IOException
	 */
	public static void sendRedirect(HttpServletRequest request, HttpServletResponse response, String pageName) throws IOException {
		response.sendRedirect(response.encodeRedirectURL(request.getContextPath() + pageName));
	}
	
	/**
	 * User Operation for the list of current page versions.
	 * @param pageList List where the {@link DataCur} instances are added.
	 * @throws SQLException
	 */
	public static void listCurrent(List<? super DataCur> pageList) throws SQLException {
		Connection conn = borrowConnection();
		conn.setAutoCommit(true);
		try {
			WData.listCurrent(conn, pageList);
		} finally {
			returnConnection(conn);
		}
	}
	
	/**
	 * User Operation for obtaining a page current version information, given it's title.
	 * @param title Requested page's title.
	 * @return {@link DataCur} instance with the page information.
	 * @throws SQLException
	 * @throws IllegalArgumentException The title didn't match with any available page.
	 */
	public static DataCur getPage(String title) throws SQLException {
		Connection conn = borrowConnection();
		conn.setAutoCommit(true);
		try {
			DataCur retCur = WData.getSingleCurByTitle(conn, title);
			if (retCur == null) {
				throw new IllegalArgumentException("Title doesn't seem to match any current page");
			}
			return retCur;
		} finally {
			returnConnection(conn);
		}
	}

	/**
	 * User operation to submit a page edition/version.
	 * @param title Title of the page.
	 * @param text Content of the page.
	 * @param user Author for the page (modification).
	 * @param isNew is_new flag. May be true even if it isn't the first version.
	 * @param redirect is_redirect flag
	 * @param minor minor_edit flag
	 * @throws Exception
	 */
	// WHATIF Consider the posibility of checking for race conditions
	//	preempting check -> Warn editor if someone is already editing.
	//	submit check -> Should ask for the predecesor id. It should be the latest.
	public static void cmdEdit(String title, String text, String user, boolean isNew, 
			boolean minor) throws Exception {
		// Check parameters.
		if (Strings.isEmpty(title)) {
			throw new Exception("Must specify a title");
		}
		if (Strings.isEmpty(text) || text.trim().equals("")) {
			throw new Exception("Text can't be blank");
		}

		boolean redirect = text.startsWith("#REDIRECT");

		Connection conn = borrowConnection();
		conn.setAutoCommit(false);
		try {
			// Move current into old. Doesn't do anything if submitting the first version.
			WData.moveCurIntoOld(conn, title);
			
			DataCur cur = new DataCur();
			cur.setTitle(title);
			cur.setText(text);
			cur.setUser(user);
			cur.setTimestamp(new Date());
			cur.setNew(isNew);
			cur.setMinorEdit(minor);
			cur.setRedirect(redirect);
			WData.insertIntoCur(conn, cur);
			
			conn.commit();
		} finally {
			returnConnection(conn);
		}
	}

	/**
	 * User operation for obtaining the version history (either of all pages, or of a given page).
	 * @param title Title of the requested page. null or empty to request all pages.
	 * @param target List where the versions (either DataCur or DataOld) are added.
	 * @throws SQLException
	 */
	public static void listHistory(String title, Collection<Object> target) throws SQLException {

		ArrayList<Object> pages = new ArrayList<Object>();
		Connection conn = borrowConnection();
		conn.setAutoCommit(true);
		try {
			if (Strings.isEmpty(title)) {
				WData.listCurrent(conn, pages);
				WData.listOld(conn, pages);
			} else {
				DataCur cur = WData.getSingleCurByTitle(conn, title);
				if (cur != null) { pages.add(cur); }
				WData.listOldWhereTitle(conn, title, pages);
			}
		} finally {
			returnConnection(conn);
		}
		Collections.sort(pages, ComparatorPagesByTimestamp.instance());
		target.addAll(pages);
	}
	
	// private constructor to prevent instantiation
	private WOps() {}
}
