/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.commons.digest;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.Serializable;
import java.security.MessageDigest;
import java.util.Arrays;

import org.apache.commons.codec.binary.Hex;
import org.apache.commons.codec.binary.StringUtils;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.io.output.NullOutputStream;

/**
 * Data class for the digest information.
 * This object is intended to be printed and/or compared in replacement of
 * other data which can be too big or might be preferred to keep secret.
 * The comparison is based on the cryptographic hash of the content. It might
 *   not be the same as comparing the actual contents byte by byte, but is
 *   is most improbable that two different contents happen to have the same digest,
 *   even so it's unpracticable to forge a different content (even through crytoanalysis)
 *   which has the same digest.
 * 
 * Instances can be easily created with {@link #createDigestData(byte[])} and siblings.
 * @author F.Javier Aranda (fjarandag@gmail.com)
 */
public class DigestData implements Serializable {
	private static final long serialVersionUID = 1L;

	public static final int HEADBYTES_SIZE_DEFAULT = 16;
	
	/**
	 * Tells if the bytes look like text or like binary data.
	 * @param bytes
	 * @param length
	 * @return
	 */
	public static boolean isLikeText(byte[] bytes, int length) {
		// Interpret as text if 75%+ of the 16 leading bytes are displayable ASCII [32,127]
		int l_computedBytes = Math.min(length, 16);
		int l_asciiCount = 0;
		for (int i = 0; i<l_computedBytes;i++) {
			byte b = bytes[i];
			if (b >= 32 && b <= 127) l_asciiCount++;
		}
		return (((double)l_asciiCount) / l_computedBytes) >= .75;
	}

	/**
	 * Creates a digest from some bulk bytes.
	 * @param bytes
	 * @param digestGen
	 * @return
	 */
	public static DigestData createDigestData(byte[] bytes, MessageDigest digestGen) {
		ByteArrayInputStream bis = new ByteArrayInputStream(bytes);
		return createDigestData(bis, digestGen);
	}

	/**
	 * Creates a digest from an InputStream (suitable for big contents).
	 * @param stream
	 * @param digestGen
	 * @return
	 */
	public static DigestData createDigestData(InputStream stream, MessageDigest digestGen) {
		try {
			DigestInputStream dis = new DigestInputStream(stream, digestGen);
			NullOutputStream nos = new NullOutputStream();
			IOUtils.copy(dis, nos);
			dis.close();
			DigestData digestData = dis.getDigest();
			return digestData;
		} catch (Exception e) {
			// Not expected
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Convenient method for creating a {@link DigestData}.
	 * @param bytes
	 * @return
	 */
	public static DigestData createDigestData(byte[] bytes) {
		byte[] l_hash = DigestUtils.sha256(bytes);
		byte[] l_headBytes = Arrays.copyOf(bytes, Math.min(bytes.length, HEADBYTES_SIZE_DEFAULT));
		return new DigestData("SHA256", l_hash, bytes.length, l_headBytes);
	}

	/**
	 * Convenient method for creating a {@link DigestData}.
	 * @param s
	 * @return
	 */
	public static DigestData createDigestData(CharSequence s) {
		return createDigestData(StringUtils.getBytesUtf8(s.toString()));
	}

	
	/** Length of the digested content. */
	private long i_length;
	/** Calculated content hash. Will be 16 bytes long for md5. */
	private byte[] i_hash;
	/** Hash algorith used. MD5 algorithm is good enough unless you need high security. */
	private String i_hashAlgorithm;
	/** Start of the content. Only for preview. Can be null or whatever size, but intended to be 16 bytes long. */
	private byte[] i_headBytes;

	/**
	 * Creates a Digest data object.
	 * @param algorithm Algorithm used to compute the hash/digest
	 * @param hashBytes 
	 * @param dataLength
	 * @param headBytes Leading bytes of the data. Used as a sample. Intended to be up to 16 bytes.
	 */
	public DigestData(String algorithm, byte[] hashBytes, long dataLength, byte[] headBytes) {
		super();
		i_hashAlgorithm = algorithm;
		i_hash = hashBytes;
		i_length = dataLength;
		i_headBytes = headBytes;
	}
	
	/**
	 * @return the hashAlgorithm
	 */
	public String getHashAlgorithm() {
		return i_hashAlgorithm;
	}

	/**
	 * @return the length
	 */
	public long getLength() {
		return i_length;
	}
	/**
	 * @return the hash
	 */
	public byte[] getHash() {
		return i_hash;
	}
	/**
	 * @return the start
	 */
	public byte[] getStart() {
		return i_headBytes;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		// hex(hash):length(%6d):head
		String l_headText = "";
		if (i_headBytes != null) {
			// Truncate length if needed
			byte[] l_shownBytes = i_headBytes;
			if (i_headBytes.length > i_length) {
				l_shownBytes = Arrays.copyOf(i_headBytes, (int)i_length);
			}
			
			l_headText = isLikeText(i_headBytes, (int)i_length) ?
					StringUtils.newStringUtf8(l_shownBytes) :
					"[" + Hex.encodeHexString(l_shownBytes) + "]";
		}
		return String.format("%s:%6d:%s", Hex.encodeHexString(i_hash), i_length, l_headText);
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.hashCode(i_hash);
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		DigestData other = (DigestData) obj;
		if (i_length != other.i_length)
			return false;
		if (!Arrays.equals(i_hash, other.i_hash))
			return false;
		if (!i_hashAlgorithm.equalsIgnoreCase(other.i_hashAlgorithm))
			return false;
		return true;
	}

}
