/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.commons.digest;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.MessageDigest;

/**
 * Filters an {@link InputStream} in order to obtain a {@link DigestData}
 * There is {@link java.security.DigestInputStream}, but we need some extra info.
 * @author F.Javier Aranda (fjarandag@gmail.com)
 */
public class DigestInputStream extends FilterInputStream {
	private int i_headLength;
	private long i_length = 0L;
	private MessageDigest i_digest;
	private byte[] i_head;
	private byte[] i_generatedHash = null;

	/**
	 * @param wrappedInput
	 */
	public DigestInputStream(InputStream wrappedInput, MessageDigest messageDigest) {
		this(wrappedInput, messageDigest, DigestData.HEADBYTES_SIZE_DEFAULT);
		
		i_digest = messageDigest;
	}

	/**
	 * @param wrappedInput
	 */
	public DigestInputStream(InputStream wrappedInput, MessageDigest messageDigest, int headLength) {
		super(wrappedInput);
		
		assert wrappedInput != null;
		assert messageDigest != null;
		//assert headLength >= 0; // Redundant as we create array
		
		i_headLength = headLength;
		i_head = new byte[headLength];
		i_digest = messageDigest;
	}
	

	/* (non-Javadoc)
	 * @see java.io.FilterInputStream#close()
	 */
	@Override
	public void close() throws IOException {
		super.close();
		i_generatedHash = i_digest.digest();
	}

	/* (non-Javadoc)
	 * @see java.io.FilterInputStream#read(byte[], int, int)
	 */
	@Override
	public int read(byte[] a_b, int a_off, int a_len) throws IOException {
		final int l_readCount = super.read(a_b, a_off, a_len);
		if (l_readCount > 0) {
			long l_bytesToFillHead = i_headLength - i_length; // long type makes sense for contents > 2GiB
			if (l_bytesToFillHead > 0) {
				// Add bytes to head
				int l_headBytesCount = Math.min(l_readCount, (int)l_bytesToFillHead);
				System.arraycopy(a_b, (int)a_off, i_head, (int)i_length, l_headBytesCount);
			}
			i_digest.update(a_b, a_off, l_readCount);
			i_length += l_readCount;
		}		
		return l_readCount;
	}

	/* (non-Javadoc)
	 * @see java.io.FilterInputStream#reset()
	 */
	@Override
	public synchronized void reset() throws IOException {
		throw new IOException("Not Supported");
	}

	/* (non-Javadoc)
	 * @see java.io.FilterInputStream#skip(long)
	 */
	@Override
	public long skip(long a_n) throws IOException {
		throw new IOException("Not Supported");
	}

	/* (non-Javadoc)
	 * @see java.io.FilterInputStream#markSupported()
	 */
	@Override
	public boolean markSupported() {
		return false;
	}


	/**
	 * Builds a DigestData with the information compiled from the input.
	 * @return
	 * @throws IllegalStateException
	 */
	public DigestData getDigest() throws IllegalStateException{
		if (i_generatedHash == null) {
			throw new IllegalStateException("Should close stream before requesting digest");
		}
		return new DigestData(i_digest.getAlgorithm(), i_generatedHash, i_length, i_head);
	}
}
