/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.commons.jdbc;

import java.io.IOException;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.dbcp.*;
import org.apache.commons.pool.ObjectPool;
import org.apache.commons.pool.impl.GenericObjectPool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.SAXException;

import ddbc.commons.util.FCollections;
import ddbc.commons.util.FlyWeight;

/**
 * Utility class for JDBC Operations.
 * @author F.Javier Aranda (fjarandag@gmail.com)
 */
public class JdbcUtils {
	/** JDBC Driver classname for the actual connection. */
	public static final String PROPERTY_ACTUALDRIVER = "driver";
	/** JDBC URL for the actual connection */
	public static final String PROPERTY_ACTUALURL = "url";
	/** Username for the actual connection. */
	public static final String PROPERTY_ACTUALUSER = "user";
	/** Password for the actual connection. */
	public static final String PROPERTY_ACTUALPASSWORD = "password";

	private static final Logger LOGGER = LoggerFactory.getLogger(JdbcUtils.class);
	
	private static HashMap<String,PoolableConnectionFactory> s_poolsByAlias = new HashMap<String,PoolableConnectionFactory>();
	public static void bindConnectionFactoryAsDriverManagerPool(ConnectionFactory factory, String alias) throws ClassNotFoundException, SQLException {
		ObjectPool connectionPool = new GenericObjectPool(null);
		PoolableConnectionFactory poolableConnectionFactory = new PoolableConnectionFactory(
				factory,connectionPool,null,"SELECT 1=1 FROM DUAL",false,true);
		s_poolsByAlias.put(alias, poolableConnectionFactory);

		Class.forName("org.apache.commons.dbcp.PoolingDriver");
		PoolingDriver driver = (PoolingDriver) DriverManager.getDriver("jdbc:apache:commons:dbcp:");

		driver.registerPool(alias, connectionPool);
		
		LOGGER.info("Pool registered into " + alias);
	}
	
	public static PoolableConnectionFactory getPoolByAlias(String alias) {
		return s_poolsByAlias.get(alias);
	}
	
	public static DriverConnectionFactory createDriverConnectionFactoryFromProperties(Properties propConfig,
			String prefix, boolean verify) throws IOException, ClassNotFoundException,
			SQLException, InstantiationException, IllegalAccessException {
		Properties l_subProperties = FCollections.subPropertiesPrefix(propConfig, prefix);
		String l_jdbcDriver = (String)l_subProperties.remove(PROPERTY_ACTUALDRIVER);
		String l_jdbcUrl = (String)l_subProperties.remove(PROPERTY_ACTUALURL);
		// User and password are kept 
		//String l_jdbcUser = (String)l_subProperties.remove(PROPERTY_ACTUALUSER);
		//String l_jdbcPwd = (String)l_subProperties.remove(PROPERTY_ACTUALPASSWORD);
		
		LOGGER.info("Creating factory for url={}, driver={}", l_jdbcUrl, l_jdbcDriver);

		// TT_LOW Consider using the DriverManager instances as done in DriverImporter (DDBC)
		Class<? extends Driver> l_class = (Class<? extends Driver>)Class.forName(l_jdbcDriver);
		Driver l_driver = FlyWeight.getDefault(l_class);
		DriverConnectionFactory l_connectionFactory = new DriverConnectionFactory(l_driver, l_jdbcUrl, l_subProperties);
		
		if (verify) {
			l_connectionFactory.createConnection().close();
		}
		return l_connectionFactory;
	}

	public static void closePool(String alias) {
		try {
			((PoolingDriver)DriverManager.getDriver("jdbc:apache:commons:dbcp:")).closePool(alias);
			LOGGER.info("Pool closed/deregistered from " + alias);
		} catch (SQLException e) {
			LOGGER.error("Failed to destroy pool " + alias, e);
		}
	}

	public static String normalizeString(String strSource) {
		StringBuilder target = new StringBuilder(strSource.length());
		// Normalize parts excluding char literals
		Pattern l_patternCharLiterals = FlyWeight.getPattern("'[^']*'('[^']*')*");
		Matcher l_matcherChars = l_patternCharLiterals.matcher(strSource);
		int l_convertStart = 0;
		boolean l_foundChars;
		do {
			l_foundChars = l_matcherChars.find();
			int l_convertEnd = l_foundChars ? l_matcherChars.start() : strSource.length();
			String l_ConvertNonChars = strSource.substring(l_convertStart, l_convertEnd);
			target.append(normalizeNonChars(l_ConvertNonChars));
			if (l_foundChars) {
				l_convertStart = l_matcherChars.regionEnd(); // Next non-chars start
				target.append(strSource.substring(l_convertEnd, l_convertStart));
			}
		} while (l_foundChars);
		return target.toString();
	}

	/**
	 * @param strSource
	 * @return
	 */
	private static String normalizeNonChars(String strSource) {
		// Easy part...
		strSource = strSource.trim().toLowerCase();
		// Now convert any possible blank space group into a single space
		// LOW_PRIORITY What if spaces and text are part of a literal string in an expression?
		Matcher l_matcher = FlyWeight.getPattern("\\s+").matcher(strSource);
		strSource = l_matcher.replaceAll(" ");
		// Finally supress spaces before and after non-word or number tokens
		// ( _a * 9 3 ) -> (_a*9 3)
		// i.e. supress spaces before and after non-number-or-identifier tokens.
		l_matcher = FlyWeight.getPattern("\\s?([^_a-zA-Z0-9])\\s?").matcher(strSource);
		strSource = l_matcher.replaceAll("$1");
		return strSource;
	}
}
