/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.commons.util;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Properties;

import javax.servlet.ServletConfig;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.xmlio.in.SimpleImportHandler;
import org.apache.commons.xmlio.in.SimpleImporter;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class Configs {
	/**
	 * Look for a main configuration property.
	 * The property might be specified as a main argument, as a system property, or as an environment variable.
	 * @param propertyName Name of the searched property
	 * @param mainArgs Main method arguments, in case the property might be there
	 * @param defaultValue Value to return in case the property is not found.
	 * @return Property found, or default value otherwise.
	 */
	public static String getStandAloneProperty(String propertyName, String[] mainArgs, String defaultValue) {
		// Try to obtain the property as a main argument
		// Either name=value or "name" "value"
		if (mainArgs != null) {
			for (int i = 0; i < mainArgs.length; i++) {
				if (mainArgs[i].startsWith(propertyName)) {
					String paramRemaining = mainArgs[i].substring(propertyName.length());
					if (Strings.isEmpty(paramRemaining)) {
						if (i + 1 < mainArgs.length) { // Has next parameter
							return mainArgs[i + 1];
						}
					} else if (paramRemaining.startsWith("=")) {
						return paramRemaining.substring(1);
					}
				}
			}
		}
		
		// Not found as parameter. Try as system property.
		String l_property = System.getProperty(propertyName);
		if (!Strings.isEmpty(l_property)) {
			return l_property;
		}
		// Not found as System property. Try as environment variable
		l_property = System.getenv(propertyName);
		if (!Strings.isEmpty(l_property)) {
			return l_property;
		}
		// Not found as Env. Return default Value.
		return defaultValue;
	}
	
	public static String getWebAppParameter(String propertyName, ServletConfig sconfig, String defaultValue) {
		String l_property = null;
		if (sconfig != null) {
			l_property = sconfig.getInitParameter(propertyName);
			if (!Strings.isEmpty(l_property)) {
				return l_property;
			}
		}
		// Not found as Init parameter. Try as system property.
		l_property = System.getProperty(propertyName);
		if (!Strings.isEmpty(l_property)) {
			return l_property;
		}
		// Not found as System property. Try as environment variable
		l_property = System.getenv(propertyName);
		if (!Strings.isEmpty(l_property)) {
			return l_property;
		}
		// Not found as Env. Return default Value.
		return defaultValue;
	}
	
	public static Properties loadSimplePropertiesFromLocation(String strLocation) throws IOException {
		InputStream is = getInputStreamFromLocation(strLocation, Configs.class);
		try {
			Properties properties = new Properties();
			properties.load(is);
			return properties;
		} finally {
			is.close();
		}
	}
	
	public static void parseXmlIoFromLocation(String strLocation, SimpleImportHandler handler) throws IOException, ParserConfigurationException, SAXException {
		InputStream is = getInputStreamFromLocation(strLocation, Configs.class);
		InputSource l_source = new InputSource(is);
		SimpleImporter l_importer = new SimpleImporter();
		l_importer.addSimpleImportHandler(handler);
		l_importer.parse(l_source);
	}
	
	public static InputStream getInputStreamFromLocation(String strLocation, Class refLocation) throws IOException {
		if (strLocation.startsWith("rsrc:")) {
			return refLocation.getResourceAsStream(strLocation.substring(5));
		} else {
			// Try as a URL (file or whatever)
			URL l_url = new URL(strLocation);
			return l_url.openStream();
		}
	}
}
