/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.commons.util;

import java.lang.reflect.Array;
import java.util.*;
import java.util.Map.Entry;

public class FCollections {
	public static <T1 extends Map> T1 putAll(T1 target, Object ... elements) {
		int l_numElements = elements.length / 2;
		for (int i = 0, j=0; i < l_numElements; i++) {
			assert j == 2 * i;
			target.put(elements[j++], elements[j++]);
		}
		return target;
	}

	public static HashMap newHashMap(Object... elements) {
		return putAll(new HashMap(), elements);
	}

	public static <T1>void setAtListFillingGaps(List<T1> list, int insertIdx, T1 insertValue, T1 fill) {
		int l_size = list.size();
		if (l_size > insertIdx) {
			list.set(insertIdx, insertValue);
		} else {
			//list.ensureCapacity(insertIdx);
			for (;l_size < insertIdx - 1; l_size++) { list.add(fill); }
			list.add(insertValue);
		}
	}

	public static <T1> T1[] appendToArray(T1[] srcArray, T1 ... appended ) {
		int l_srcLen = srcArray.length;
		int l_appendedLen = appended.length;
		T1[] l_newArray = Arrays.copyOf(srcArray, l_srcLen + l_appendedLen);
		System.arraycopy(appended, 0, l_newArray, l_srcLen, l_appendedLen);
		return l_newArray;
	}

	public static <T1> T1[] concatArray(Class<T1> objClass, Object ... elements ) {
		int totalLen = 0;
		int numOfElements = elements.length;
		boolean[] isArray = new boolean[numOfElements];
		for (int i = 0; i < numOfElements; i++) {
			Object curElem = elements[i];
			Class<?> l_elemClass = curElem.getClass();
			if (l_elemClass.isArray() && objClass.isAssignableFrom(l_elemClass.getComponentType()) ) {
				isArray[i] = true;
				totalLen += Array.getLength(curElem);
			} else {
				isArray[i] = false;
				totalLen++;
			}
		}
		T1[] l_result = (T1[])Array.newInstance(objClass, totalLen);
		int curOffset = 0;
		for (int i = 0; i < numOfElements; i++) {
			if (isArray[i]) {
				T1[] curElem = (T1[])elements[i];
				System.arraycopy(curElem, 0, l_result, curOffset, curElem.length);
				curOffset += curElem.length;
			} else {
				T1 curElem = (T1)elements[i];
				l_result[curOffset++] = curElem;
			}
		}
		return l_result;
	}

	/**
	 * Obtains some nested properties (those begining with a prefix) from some parent properties.
	 * The leading prefix is trimmed out of the keys of the obtained properties.
	 * @param parentProps The parent Properties Object.
	 * @param prefix Leading sequence which identifies the properties to be obtained.
	 * @return New Properties object with the obtained child properties.
	 */
	public static Properties subPropertiesPrefix(Properties parentProps, String prefix) {
		Set<Map.Entry<Object,Object>> l_propEntries = parentProps.entrySet();
		Properties l_result = new Properties();
		int l_prefixLength = prefix.length();
		for (Entry<Object, Object> l_entry : l_propEntries) {
			String l_key = (String)l_entry.getKey();
			if (l_key.startsWith(prefix)) {
				// Key is trimmed, incluiding an optional leading period.
				l_key = l_key.substring(l_prefixLength);
				if (l_key.charAt(0) == '.') { l_key = l_key.substring(1); }
				l_result.setProperty(l_key, (String)l_entry.getValue());
			}
		}
		return l_result;
	}
	
	/**
	 * Inserts a value in a Map in a multimap manner (Commons Collections).
	 * The values are inserted in a list accessed by the keys. The values are not replaced.
	 * @param <K> Type of the Map key
	 * @param <V> Type of the actual value (The "physical" map will have a List<V>)
	 * @param target Map into which the value is inserted
	 * @param a_key Insertion key.
	 * @param a_value Value added into the multi-map.
	 */
	public static <K,V> void multiMapPut(Map<? super K,? super List<V>> target, K a_key, V a_value) {
		List<V> l_valList = (List<V>)target.get(a_key);
		if (l_valList == null) {
			l_valList = new ArrayList<V>();
			target.put(a_key, l_valList);
		}
		l_valList.add(a_value);
	}
}
