/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.commons.util;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.helpers.AttributesImpl;

public class FactoryUtils {

	public static enum ConstructorTypeImporter {
		NONE,
		FACTORY_METHOD,
		PREFERRED_CONSTRUCTOR,
		DEFAULT_CONSTRUCTOR
	}

	public static class FactoryImporter {
		private ConstructorTypeImporter i_constructorType = ConstructorTypeImporter.NONE;
		private Object i_constructorMethod;
		public ConstructorTypeImporter getConstructorType() { return i_constructorType; }
		public Method getFactoryMethod() { return (Method)i_constructorMethod; }
		public Constructor getConstructor() { return (Constructor)i_constructorMethod; }
		public void setFactoryMethod(Method a_factoryMethod) {
			if (a_factoryMethod != null){
				i_constructorType = ConstructorTypeImporter.FACTORY_METHOD;
				i_constructorMethod = a_factoryMethod;
			}
		}
		public void setPreferredConstructor(Constructor a_constructor) {
			if (a_constructor != null) {
				i_constructorType = ConstructorTypeImporter.PREFERRED_CONSTRUCTOR;
				i_constructorMethod = a_constructor;
			}
		}
		public void setDefaultConstructor(Constructor a_constructor) {
			if (a_constructor != null) {
				i_constructorType = ConstructorTypeImporter.DEFAULT_CONSTRUCTOR;
				i_constructorMethod = a_constructor;
			}
		}
		
		public <TM> TM create(AttributesImpl a_attributes, String elementData) throws Exception {
			Map<String,String> l_mapAttributes = XmlUtils.attributesToMap(a_attributes);
			switch(i_constructorType) {
			case FACTORY_METHOD: return (TM)getFactoryMethod().invoke(null, l_mapAttributes, elementData);
			case PREFERRED_CONSTRUCTOR: return (TM)getConstructor().newInstance(l_mapAttributes, elementData);
			case DEFAULT_CONSTRUCTOR: return (TM)getConstructor().newInstance();
			default:
				throw new IllegalStateException("FactoryImporter with constructor NONE");
			}
		}
	}

	private static Logger s_logger = LoggerFactory.getLogger(FactoryUtils.class);

	/**
	 * Gets the factory method (if possible).
	 * @param a_class Class where the factory method is defined
	 * @param methodName Method name (usually "getInstance")
	 * @return 
	 */
	public static Method findFactoryMethod(Class<?> a_class, String methodName) {
		try {
			Method i_method = a_class.getMethod(methodName, Map.class, String.class);
			// Check if the method is suitable (must be static, defined in the same class (not an ancestor),
			//    invokable and return something).
			if (	Modifier.isStatic(i_method.getModifiers()) &&
					i_method.getDeclaringClass().equals(a_class) &&
					//i_method.isAccessible() &&
					! i_method.getReturnType().equals(void.class)) {
				return i_method;
			} else {
				s_logger.debug("Method {}#{} exists, but isn't suitable", a_class.getName(), methodName);
				return null;
			}
		} catch(Exception e) {
			s_logger.debug(e.toString());
			return null;
		}
	}

	public static <T> Constructor<T> findConstructor(Class<T> constructed, Class<?> ... parameterTypes) {
			try {
				Constructor<T> l_constructor = constructed.getConstructor(parameterTypes);
	//			if (l_constructor.isAccessible()) {
					return l_constructor;
	//			} else {
	//				return null;
	//			}
			} catch(Exception e) {
				s_logger.debug(e.toString());
				return null;
			}
		}

	public static FactoryImporter createFactoryData(String hostClassName,
			String customMethodName, String defaultMethodName) throws NoSuchMethodException {
		FactoryImporter factoryData = new FactoryImporter();
		Class l_hostClass;
		String l_errorMessage = String.format("No suitable constructor for class %s#%s", hostClassName, customMethodName);
		try {
			l_hostClass = Class.forName(hostClassName);
			if (!Strings.isEmpty(customMethodName)) {
				factoryData.setFactoryMethod(findFactoryMethod(l_hostClass, customMethodName));
			} else {
				factoryData.setFactoryMethod(findFactoryMethod(l_hostClass, "getInstance"));
				if (factoryData.getConstructorType() == ConstructorTypeImporter.NONE) {
					factoryData.setPreferredConstructor(findConstructor(l_hostClass, Map.class, String.class));
					if (factoryData.getConstructorType() == ConstructorTypeImporter.NONE) {
						factoryData.setDefaultConstructor(findConstructor(l_hostClass));
					}
				}
			}
		} catch (ClassNotFoundException e) {
			l_errorMessage = e.getMessage();
		}
		if (factoryData.getConstructorType() == ConstructorTypeImporter.NONE) {
			s_logger.error(l_errorMessage);
			throw new NoSuchMethodException(l_errorMessage);
		} else {
			return factoryData;
		}
	}

}
