/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.commons.util;

import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.WeakHashMap;
import java.util.regex.Pattern;

import org.apache.commons.collections.Transformer;

/**
 * Utility to implement flyweight design patten.
 * Given a Value, returns canonical instance.
 * Given a key(spec) and a Transformer, returns reusable instance.
 * @author Javier Aranda
 *
 */
public class FlyWeight {
	private static WeakHashMap<Object, Object> canonicalMap = new WeakHashMap<Object, Object>();
	
	/**
	 * Obtains the "preferred"/canonical instance for a value.
	 * @param o the Object value instance
	 * @return The preferred object instance.
	 */
	public static <T extends Object> T getPreferred(T o) {
		if (o == null) return null;
		T canonical = (T)canonicalMap.get(o);
		if (canonical == null) {
			if (o instanceof String) {
				canonical = (T)((String)o).intern();
			// Could do the same with other primitive wrappers.
			} else if (o instanceof Integer) {
				canonical = (T)Integer.valueOf(((Integer)o).intValue());
			} else {
				canonical = o;
			}
			canonicalMap.put(canonical, canonical);
		}
		return canonical;
	}
	
// NOTE: Disabled code: Would preferrable to use Commons Collections LazyMap
// http://commons.apache.org/collections/api-release/org/apache/commons/collections/map/LazyMap.html
//	private static WeakHashMap<Object, Object> singletonMap = new WeakHashMap<Object, Object>();
//	/**
//	 * Get a singleton object given its key.
//	 * @param key 
//	 * @param builder Transformer capable of building the singleton instance if it didn't exist.
//	 * @return
//	 */
//	public static Object getSingleton(Object key, Transformer builder) {
//		Object singleton = singletonMap.get(key);
//		if (singleton == null) {
//			singleton = builder.transform(key);
//			singletonMap.put(getPreferred(key), singleton);
//		}
//		return singleton;
//	}
	
// WHATIF: Creating a weakreference for the key and mapping it has some overhead
// maybe we should make a benchmark to test if it's really worth it.
// WHATIF: We could use a SortedMap so we can borrow a bigger map if requested size is not available.
	/** Map with available StringBuffer for reuse. */
	private static WeakHashMap<Integer, StringBuffer> bufferMap = new WeakHashMap<Integer, StringBuffer>();

	/**
	 * Obtains a usable {@link StringBuffer} for the given size.
	 * If not available, a new StringBuffer is created.
	 * The caller is expected to return the instance with {@link #returnStringBuffer(StringBuffer)}.
	 * Notice the StringBuffer can grow and be stored with a bigger size.
	 * @param size The initial size of the requested StringBuffer.
	 * @return
	 */
	public static StringBuffer borrowStringBuffer(int size) {
		StringBuffer sb = bufferMap.remove(size);
		if (sb == null) {
			sb = new StringBuffer(size);
			bufferMap.put(Integer.valueOf(size), sb);
		}
		return sb;
	}
	
	/**
	 * Stores a StringBuffer for its reuse.
	 * StringBuffers borrowed ( {@link #borrowStringBuffer(int)} ) are expected to be returned.
	 * @param sb Returned StringBuffer.
	 */
	public static void returnStringBuffer(StringBuffer sb) {
		int size = sb.capacity();
		sb.setLength(0);
		bufferMap.put(Integer.valueOf(size), sb);
	}
	
	/** Map with available RegEx Patterns. Use them as singleton. */
	private static WeakHashMap<String, Pattern> patternMap = new WeakHashMap<String, Pattern>();
	
	/**
	 * Obtains a RegEx {@link Pattern}. Creates one if not available.
	 * @param spec RegEx specification.
	 * @return
	 */
	public static Pattern getPattern(String spec) {
		Pattern pattern = patternMap.get(spec);
		if (pattern == null) {
			pattern = Pattern.compile(spec);
			patternMap.put(spec, pattern);
		}
		return pattern;
	}

	/** Map with available instances of SimpleDateFormat.
	 * SimpleDateFormat is not properly inmutable (properties locale and dateFormatSymbols),
	 * but as it is not expected to be modified, we use them as reentrant. */
	private static WeakHashMap<String, SimpleDateFormat> dateFormatMap = new WeakHashMap<String, SimpleDateFormat>();
	/**
	 * Obtains a {@link SimpleDateFormat} for the given pattern.
	 * Reuses previously created if possible, creates new if needed.
	 * @param pattern
	 * @return
	 */
	public static SimpleDateFormat getDateFormat(String pattern) {
		SimpleDateFormat retDF = dateFormatMap.get(pattern);
		if (retDF == null) {
			retDF = new SimpleDateFormat(pattern);
			dateFormatMap.put(pattern, retDF);
		}
		return retDF;
	}

	/** Map with available instances of SimpleDateFormat.
	 * SimpleDateFormat is not properly inmutable (properties locale and dateFormatSymbols),
	 * but as it is not expected to be modified, we use them as reentrant. */
	private static WeakHashMap<String, DecimalFormat> decimalFormatMap = new WeakHashMap<String, DecimalFormat>();
	/**
	 * Obtains a {@link DecimalFormat} for the given pattern.
	 * Reuses previously created if possible, creates new if needed.
	 * @param pattern
	 * @return
	 */
	public static DecimalFormat getDecimalFormat(String pattern) {
		DecimalFormat retDF = decimalFormatMap.get(pattern);
		if (retDF == null) {
			retDF = new DecimalFormat(pattern);
			decimalFormatMap.put(pattern, retDF);
		}
		return retDF;
	}

	
	private static WeakHashMap<Class<?>, Object> defaultInstanceMap = new WeakHashMap<Class<?>, Object>();
	public static <T> T borrowDefault(Class<T> a_class) throws InstantiationException, IllegalAccessException {
		T retObj = (T)defaultInstanceMap.remove(a_class);
		if (retObj == null) {
			retObj = a_class.newInstance();
		}
		return retObj;
	}
	public static void returnDefault(Object o) {
		defaultInstanceMap.put(o.getClass(), o);
	}
	public static <TM> TM getDefault(Class<TM> a_class) throws InstantiationException, IllegalAccessException {
		TM retObj = (TM)defaultInstanceMap.get(a_class);
		if (retObj == null) {
			retObj = a_class.newInstance();
			defaultInstanceMap.put(a_class, retObj);
		}
		return retObj;
	}

	/**
	 * Obtains an usable collection.
	 * reuses if available, creates new if needed.
	 * The caller is expected to invoke {@link #returnCollection(Collection)}.
	 * @param <T> Class of the requested collection.
	 * @param a_class Class of the requested collection.
	 * @return
	 * @throws InstantiationException Failure invoking {@link Class#newInstance()}
	 * @throws IllegalAccessException Failure invoking {@link Class#newInstance()}
	 */
// WHATIF we only need arraylist. It's interesting to consider size.
// To have hashMap (which is not a collection) is interesting.
// It could be better to have a pool with many instances, reclamable by GC.
	public static <T extends Collection<?>> T borrowCollection(Class<T> a_class) {
		T retObj = (T)defaultInstanceMap.remove(a_class);
		if (retObj == null) {
			try {
				retObj = a_class.newInstance();
			} catch (Exception e) {
				// Simply return null
			}
		}
		return retObj;
	}

	/**
	 * Returns a (borrowed) collection, allowing its reuse.
	 * The collection is cleared to prevent interference with next user.
	 * @param c
	 */
	public static void returnCollection(Collection<?> c) {
		c.clear();
		defaultInstanceMap.put(c.getClass(), c);
	}


}
