/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.commons.util;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.ref.SoftReference;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Pattern;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Utility class related to HTML documents.
 * @author Javier Aranda
 */
public class Html {
	private static final Logger LOGGER = LoggerFactory.getLogger(Html.class);
	// Data stored in references to allow unloading if JVM considers so.
	/** Reference to map SGML entity alias -> Unicode character code. */
	private static SoftReference<Map<String, Integer>> refMapAlias2Unicode;
	/** Reference to map Unicode character code -> SGML entity alias. */
	private static SoftReference<Map<Integer, String>> refMapUnicode2Alias;
	
	/**
	 * Get the SGML entity alias for a given unicode char.
	 * @param unicode Unicode char
	 * @return SGML entity alias, null if no match is found.
	 */
	public static String getEntityAliasUnicode(int unicode) {
		if (refMapUnicode2Alias == null || refMapUnicode2Alias.get() == null) {
			loadEntityData();
		}
		return refMapUnicode2Alias.get().get(unicode);
	}

	/**
	 * Gets the unicode char code for a given SGML alias
	 * @param entityAlias
	 * @return Integer containing the char, or null if the alias has no match.
	 */
	public static Integer getUnicodeEntityAlias(String entityAlias) {
		if (refMapAlias2Unicode == null || refMapAlias2Unicode.get() == null) {
			loadEntityData();
		}
		return refMapAlias2Unicode.get().get(entityAlias);
	}
	
	/**
	 * Loads the SGML - Unicode Mapping from htmlEntities.txt
	 */
	private static void loadEntityData() {
		try {
			// Maps under construction.
			Map<String, Integer> mapAlias2Unicode = new HashMap<String, Integer>(400);
			Map<Integer, String> mapUnicode2Alias = new HashMap<Integer, String>(400);

			InputStream ris = Html.class.getResourceAsStream("htmlEntities.txt");
			BufferedReader br = new BufferedReader(new InputStreamReader(ris, "ISO-8859-1"));
			try {
				String line;
				// For each line in the file
				while ((line = br.readLine()) != null) {
					try {
						line = line.trim();
						// If the line is not void or comment
						if (! (line.equals("") || line.startsWith("#")) ) {
							// Read the fields [0]: alias, [1]: unicode
							String[] lineFields = line.split(":");
							String alias = lineFields[0].trim();
							Integer unicode = Integer.valueOf(lineFields[1].trim());
							// Record the mapping.
							mapAlias2Unicode.put(alias.intern(), unicode);
							mapUnicode2Alias.put(unicode, alias.intern());
						}
					} catch(Exception e) {
						// En caso de que falle una línea no queremos que falle el resto.
						LOGGER.warn(
								"Couldn't load entity: " + line, e);
					}
				}
			} finally {
				ris.close();
			}
			
			// Store the maps as references.
			refMapAlias2Unicode = new SoftReference<Map<String,Integer>>(mapAlias2Unicode);
			refMapUnicode2Alias = new SoftReference<Map<Integer,String>>(mapUnicode2Alias);
		} catch(Exception e) {
			LOGGER.error(
					"Couldn't load entities", e);
		}
	}
	
	/**
	 * Convert escaped text (with SGML escape sequences) into unescaped text.
	 * @param html source escaped text. Should not be null.
	 * @return
	 */
	public static String unescapeEntities(String html) {
		// Search if there is any escape sequence.
		int idxOfAmp = html.indexOf('&');
		int idxOfSC = -1; // Index of SemiCollon. Also position for starting copy.
		// if there is no escape sequence, the return is really easy.
		if (idxOfAmp < 0) return html;
		// The unescaped text can't be bigger.
		// NOTE the requested size is rounded up to improve reusability.
		StringBuffer sb = FlyWeight.borrowStringBuffer(((html.length() / 16) + 1) * 16);
		// Pattern for number codes in escape sequences (e.g. #255 in &#255;)
		Pattern patternNumberEntity = FlyWeight.getPattern("#\\d{1,5}");
		// While there are escape sequences remaining
		while(idxOfAmp >= 0) {
			// Add text since the last escape sequence (or since the beginning)
			sb.append(html.substring(idxOfSC + 1, idxOfAmp));
			idxOfSC = html.indexOf(';', idxOfAmp + 1); // End of the escape sequence
			if (idxOfSC < 0) { // If the end was not found.
				String message = String.format("unescapeEntities: ';' not found in \"%s\" after %d", html, idxOfAmp);
				LOGGER.error(message);
				sb.append(html.substring(idxOfAmp));
				idxOfSC = html.length() - 1;
			} else { // Try to convert the escape sequence.
				String entityDef = html.substring(idxOfAmp + 1, idxOfSC);
				if (patternNumberEntity.matcher(entityDef).matches()) { // The content is numeric
					// Translate numeric code (discard heading #) and append it.
					int unicode = Integer.parseInt(entityDef.substring(1), 10);
					sb.append((char)unicode);
				} else { // Try to match as an alias
					Integer unicode = getUnicodeEntityAlias(entityDef);
					if (unicode != null) { // Known alias. Append it.
						sb.append((char)unicode.intValue());
					} else { // Unknown alias. Try to ignore. Could be an uncoupled ampersand.
						LOGGER.warn(String.format("Could not resolve entity: &%s;", entityDef));
						sb.append('&'); // This start didn't bring any result
						// Fix to continue the process after & (maybe the found semicollon was matched with another ampersand).
						idxOfSC = idxOfAmp;
					}
				}
			}
			// Search next escape sequence.
			idxOfAmp = html.indexOf('&', idxOfSC + 1);
		}
		// Append the text since the last escape sequence
		sb.append(html, idxOfSC, html.length());
		
		String ret = sb.toString();
		FlyWeight.returnStringBuffer(sb);
		return ret;
	}
	
	/**
	 * Convert unescaped text translating special characters into SGML escape codes.
	 * Characters < 32 (excluding whitespaces (9(Tab),10(CR),13(LF), etc) are converted into numeric escape sequences.
	 * Characters 32 <= c < 128 are normal characters (except '<','>','&' and '"', which are translated into named entities).
	 * Characters >= 128 are converted into SGML entities (named if possible, numeric otherwise).
	 * @param text
	 * @return
	 */
	public static String escapeEntities(String text) {
		int textLen = text.length();
		StringBuffer sb = FlyWeight.borrowStringBuffer((int)(textLen * 1.1) + 20);
		for (int i = 0; i < textLen; i++) {
			char c = text.charAt(i);
			if (c < 32) {
				if (Character.isWhitespace(c)) {
					sb.append(c);
				} else {
					sb.append('&' + Integer.toString(c) + ';');
				}
			} else if (c < 128) {
				switch(c) {
				case '<': sb.append("&lt;"); break;
				case '>': sb.append("&gt;"); break;
				case '"': sb.append("&quot;"); break;
				case '&': sb.append("&amp;"); break;
				default: sb.append((char)c);
				}
			} else { // c >= 128
				String entity = getEntityAliasUnicode(c);
				sb.append('&');
				if (entity != null) { // &someName;
					sb.append(entity);
				} else { // &#555;
					sb.append('#');
					sb.append((int)c);
				}
				sb.append(';');
			}
		}
		String ret = sb.toString();
		FlyWeight.returnStringBuffer(sb);
		return ret;
	}
}
