/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.commons.util;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Utility class for string management.
 * @author Javier Aranda
 */
public class Strings {
	
	private static final Logger LOGGER = LoggerFactory.getLogger(Strings.class);

	/**
	 * Shortcut method for testing whether a String has no content.
	 * We consider so if the string is null or String#isEmpty() returns true.
	 * @param s Checked string
	 * @return
	 */
	public static boolean isEmpty(String s) {
		return s == null || "".equals(s);
	}
	
	public static boolean isEmptyOrWhiteSpace(String s){
		return s == null || s.trim().equals("");
	}

	public static String defaultIfEmptyOrWhiteSpace(String s, String a_default) {
		return s == null || s.trim().equals("") ? a_default : s;
	}
	
	// lowercase literals for true of false for english and spanish.
	private static Set<String> falseNames = new HashSet<String>(Arrays.asList(new String[]{"false", "no", "n", "0"}));
	private static Set<String> trueNames = new HashSet<String>(Arrays.asList(new String[]{"true", "t", "yes", "y", "si", "s", "1"}));

	/**
	 * Parses a String into a boolean in a more flexible way.
	 * Is case-insensitive, and allows different variants for true and false ("true","yes","y","1").
	 * If the string can not be matched, it is considered false by default.
	 * @param s Checked String
	 * @return Boolean value interpreted from the String.
	 */
	public static boolean parseBoolean(String s) {
		return parseBoolean(s, false);
	}

	/**
	 * Parses a String into a boolean in a more flexible way.
	 * Is case-insensitive, and allows different variants for true and false ("true","yes","y","1").
	 * @param s Checked String
	 * @param defaultValue Value interpreted if no match is found.
	 * @return Boolean value interpreted from the String.
	 */
	private static boolean parseBoolean(String s, boolean defaultValue) {
		boolean retValue = defaultValue;
		// WHATIF We can check only the true set or the false set depending on the default value.
		if (!isEmpty(s)) {
			s = s.toLowerCase();
			if (trueNames.contains(s)) {
				retValue = true;
			} else if (falseNames.contains(s)) {
				retValue = false;
			}
		}
		return retValue;
	}
	
	public static byte[] toUtf8(String s) {
		try {
			return s.getBytes("UTF-8");
		} catch(UnsupportedEncodingException cheeseBoilerPlateException) {
			LOGGER.error("You should not ever read this message", cheeseBoilerPlateException);
			// Return something nasty in order to avoid the program from accidentaly finishing the operation
			return null;
		}
	}
	
	public static String fromUtf8(byte[] utf8) {
		try {
			return new String(utf8, "UTF-8");
		} catch(UnsupportedEncodingException cheeseBoilerPlateException) {
			LOGGER.error("You should not ever read this message", cheeseBoilerPlateException);
			// Return something nasty in order to avoid the program from accidentaly finishing the operation
			return null;
		}
	}
	
	public static String toString(Object o) {
		StringBuilder l_builder = new StringBuilder();
		try {
			append(l_builder, o);
		} catch (IOException e) {
			// Unexpected
			throw new Error("Unfeasible exception", e);
		}
		return l_builder.toString();
	}
	
	public static <T extends Appendable> T append(T target, Object o) throws IOException {
		if (o == null) {
			target.append("null");
		} else {
			if (o.getClass().isArray()) {
				int l_arrayLen = Array.getLength(o);
				String l_sep = "";
				target.append('[');
				for (int i = 0; i < l_arrayLen; i++) {
					target.append(l_sep);
					Object l_item = Array.get(o, i);
					append(target, l_item);
					l_sep = ",";
				}
				target.append(']');
			} else if (o instanceof Collection) {
				Collection l_col = (Collection)o;
				String l_sep = "";
				target.append('[');
				Iterator iterCol = l_col.iterator();
				while (iterCol.hasNext()) {
					target.append(l_sep);
					append(target, iterCol.next());
					l_sep = ",";
				}
				target.append(']');
			} else if (o instanceof Map) {
				Map l_map = (Map)o;
				List l_sortedKeys = new ArrayList(l_map.keySet());
				Collections.sort(l_sortedKeys);
				int l_longitud = l_sortedKeys.size();

				String l_sep = "";
				target.append('{');
				for (int i = 0; i < l_longitud;i++) {
					target.append(l_sep);
					Object l_key = l_sortedKeys.get(i);
					Object l_value = l_map.get(l_key);

					append(target, l_key);
					target.append('=');
					append(target, l_value);

					l_sep = ",";
				}
				target.append('}');
			} else {
				// Not a Recursive type
				target.append(String.valueOf(o));
			}
		}
		return target;
	}
	
	/**
	 * Finds and replaces in a String/CharSequence the references to
	 *    variables in the shape of ${varName} by its value.
	 * @param inStr Input String/CharSequence with the variable references
	 * @param properties Map containing the variable name-value tuples
	 * @return The string result of the replacement.
	 */
	public static String replaceStrVariables(CharSequence inStr, Map<String,String> properties) {
		Pattern l_substPattern = FlyWeight.getPattern("\\$\\{([^\\}]+)\\}"); // ${varName}
		Matcher l_matcher = l_substPattern.matcher(inStr);
		StringBuilder l_builder = null;
		int l_fromIndex = 0;
		while (l_matcher.find()) {
			if (l_builder == null) {l_builder = new StringBuilder(2*inStr.length());}
			l_builder.append(inStr.subSequence(l_fromIndex, l_matcher.start(0)));
			l_fromIndex = l_matcher.end(0);

			String l_varName = l_matcher.group(1);
			String l_varValue = properties.containsKey(l_varName) ?
					properties.get(l_varName) :
					l_matcher.group(0);
			l_builder.append(l_varValue);
		}
		
		if (l_builder == null) {
			return inStr.toString();
		} else {
			l_builder.append(inStr.subSequence(l_fromIndex, inStr.length()));
			return l_builder.toString();
		}
	}
}
