/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.tool.sr;

import java.text.DecimalFormat;
import java.util.*;

import ddbc.cpn2.CpUtils;
import ddbc.commons.util.FCollections;

public class DataSequenceClass implements Comparable<DataSequenceClass> {
	private static int s_idCounter = 0;
	private static DecimalFormat DF_ID = new DecimalFormat("'SR-seqC'0000");
	// TODO Check if we really need an exclusion list for values.
	// A sufficient varied test should provide cases which exclude undue dependencies.
	private static final Set<String> SKIPPED_VALUES = createSkippedValues();
	
	private static Set<String> createSkippedValues() {
		HashSet<String> l_set = new HashSet<String>();
		Collections.addAll(l_set, "true", "false", "0", "1", "", " ");
		return l_set;
	}

	private String i_srId;
	private ArrayList<DataStatementClass> i_statements = new ArrayList<DataStatementClass>();
	private ArrayList<String> i_idsInitialStatements = new ArrayList<String>();
	private HashSet<List<String>> i_dependencyGroups = new HashSet<List<String>>();
	private HashMap<String,List<String>> i_dependencyGroupByItem = new HashMap<String, List<String>>();

	public DataSequenceClass(List<DataStatementClass> statements) {
		super();
		i_srId = DF_ID.format(s_idCounter++);
		i_statements.addAll(statements);
	}

	/**
	 * @return the srId
	 */
	public String getSrId() {
		return i_srId;
	}
	
	public boolean matchesStatements(List<DataStatementClass> statements) {
		return i_statements.equals(statements);
	}
	
	public void addSequenceInstance(DataSequenceInstance sqInstance) {
		ArrayList<DataStatementInstance> l_stIs = sqInstance.getStatements(new ArrayList<DataStatementInstance>());
		boolean l_fistInstance = i_idsInitialStatements.isEmpty();
		i_idsInitialStatements.add(l_stIs.get(0).getStmtId());
		Map<String,List<String>> l_keysByValue = computeKeysByValue(l_stIs);
		if (l_fistInstance) {
			// Load the dependencies as they come
			for (List<String> l_refGroup : l_keysByValue.values()) {
				if (l_refGroup.size() > 1) { // 2+ columns with same value.
					i_dependencyGroups.add(l_refGroup);
					for (String l_item : l_refGroup) {
						i_dependencyGroupByItem.put(l_item, l_refGroup);
					}
				}
			}
		} else {
			// Intersect the dependency groups
			updateDependencyGroups(l_keysByValue);
		}
	}
	
	private void updateDependencyGroups(Map<String, List<String>> payloadKeysByValue) {
		Set<List<String>> l_foundStoredGroups = new HashSet<List<String>>();
		for (List<String> l_payloadGroup : payloadKeysByValue.values()) {
			// Get the stored groups related to the items of the payload group
			l_foundStoredGroups.clear();
			for (String l_payloadItem : l_payloadGroup) {
				List<String> l_storedGroup = i_dependencyGroupByItem.get(l_payloadItem);
				// Most times will be just the same group again
				if (l_storedGroup != null) { l_foundStoredGroups.add(l_storedGroup); }
			}
			// Hopefully, most times there will be only one group, being the same as the payload
			for (List<String> l_storedGroup : l_foundStoredGroups) {
				if (!l_payloadGroup.equals(l_storedGroup)) { // Need changes in the sets
					// Remove the group
					i_dependencyGroups.remove(l_storedGroup);
					for (String l_key : l_storedGroup) { i_dependencyGroupByItem.remove(l_key); }
				}
				// Given working groups A and B, candidates for insertion will be A & B, A - B and B - A.
				// Given A=payloadGroup, we exclude A - B, as the items not in B, if relevant, will be stored in other B's
				List<List<String>> l_candidates = new ArrayList<List<String>>();
				// A & B
				ArrayList<String> l_newGroup = new ArrayList<String>(l_storedGroup);
				l_newGroup.retainAll(l_payloadGroup);
				l_candidates.add(l_newGroup);
				// B - A
				l_newGroup = new ArrayList<String>(l_storedGroup);
				l_newGroup.removeAll(l_payloadGroup);
				l_candidates.add(l_newGroup);
				for (List<String> l_candidate : l_candidates) {
					if (l_candidate.size() > 1) { // Insert only if big enough
						i_dependencyGroups.add(l_candidate);
						for (String l_item : l_candidate) {
							i_dependencyGroupByItem.put(l_item, l_candidate);
						}
					}
				}
			}
			
		}
	}

	private Map<String, List<String>> computeKeysByValue(ArrayList<DataStatementInstance> stIs) {
		HashMap<String, List<String>> l_result = new HashMap<String, List<String>>();
		// Register mapping for each statement
		for (int idxStmt = 0; idxStmt < stIs.size(); idxStmt++) {
			String l_prefixStmt = "s" + (idxStmt + 1);
			DataStatementInstance l_curStmt = stIs.get(idxStmt);
			// Register mapping for each jdbc parameter in the statement
			int l_paramCount = l_curStmt.getSetParametersCount();
			ArrayList<Object> l_parameters = l_curStmt.getSetParameters(new ArrayList<Object>(l_paramCount * 2));
			for (int i = 0; i < l_paramCount; i++) {
				String l_key = l_prefixStmt + "p" + (i + 1); // s1p1
				String l_value = CpUtils.toString(
						((Object[])l_parameters.get(i*2 + 1))[0]);
				registerReverseMapping(l_result, l_key, l_value);
			}

			// Register mapping for each Resultset column in the statement
			ArrayList<Object> l_firstRsRow = l_curStmt.getFirstRsRow(new ArrayList<Object>());
			int l_rsColCount; 
			if (l_firstRsRow != null) {
				// TODO Not yet executed ?
				l_rsColCount = l_firstRsRow.size() / 2;
				for (int i = 0; i < l_rsColCount; i++) {
					String l_key = l_prefixStmt + "r1c"
							+ l_firstRsRow.get(i * 2); // s1r1cRSKEY
					String l_value = CpUtils.toString(l_firstRsRow
							.get(i * 2 + 1));
					registerReverseMapping(l_result, l_key, l_value);
				}
			}

			// Register mapping for each Generated keys column in the statement
			ArrayList<Object> l_firstGkRow = l_curStmt.getFirstGkRow(new ArrayList<Object>());
			if (l_firstGkRow != null) {
				// TODO Not yet executed ?
				int l_gkColCount = l_firstGkRow.size() / 2;
				for (int i = 0; i < l_gkColCount; i++) {
					String l_key = l_prefixStmt + "k1c"
							+ l_firstGkRow.get(i * 2); // s1k1cRSKEY
					String l_value = CpUtils.toString(l_firstGkRow
							.get(i * 2 + 1));
					registerReverseMapping(l_result, l_key, l_value);
				}
			}
		}
		return l_result;
	}

	/**
	 * @param mmap
	 * @param l_key
	 * @param l_value
	 */
	private void registerReverseMapping(HashMap<String, List<String>> mmap,
			String l_key, String l_value) {
//		if (l_value != null && ! SKIPPED_VALUES.contains(l_value)) {
			// The key and value are inserted switched as this is a reverse-mapping.
		// Note: We are allowing null key in the map.
			FCollections.multiMapPut(mmap, l_value, l_key);
//		}
	}

	public <TList extends List<? super DataStatementClass>> TList getStatements(TList target) {
		target.addAll(i_statements);
		return target;
	}

	public <TList extends List<? super String>> TList getIdsInitialStatements (TList target) {
		target.addAll(i_idsInitialStatements);
		return target;
	}
	
	public <TList extends List<List<String>>> TList getDependencyGroups (TList target) {
		for (List<String> l_group : i_dependencyGroups) {
			target.add(l_group);
		}
		return target;
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(DataSequenceClass o) {
		return this.i_srId.compareTo(o.i_srId);
	}

	
}
