/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.tool.sr;

import java.io.IOException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.*;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ddbc.cpn2.JdbcMethod;
import ddbc.cpn2.impl.CpPreparedStatement;
import ddbc.cpn2.impl.CpResultSet;
import ddbc.cpn2.log.LogEvent;
import ddbc.cpn2.log.LogListener;
import ddbc.commons.jdbc.JdbcUtils;
import ddbc.commons.util.Convert;
import ddbc.commons.util.FCollections;

public class GenSrListener implements LogListener {
	private static Logger LOGGER = LoggerFactory.getLogger(GenSrListener.class);

	private HashMap<String,DataStatementClass> i_stmtClassFromSql = new HashMap<String, DataStatementClass>();
	private HashMap<DataStatementClass, ArrayList<DataSequenceClass>> i_seqsByFirstStmt =
		new HashMap<DataStatementClass, ArrayList<DataSequenceClass>>();
	
	// TT_LOW Consider that a plain statement id can have multiple sql statements attached.
	private HashMap<String, DataConnection> i_connCtxById = new HashMap<String, DataConnection>();
	private HashMap<String, DataStatementInstance> i_stmtInstanceById = new HashMap<String, DataStatementInstance>();
	private HashMap<String, DataStatementInstance> i_stmtInstanceByRsId = new HashMap<String, DataStatementInstance>();
	private HashMap<String, DataStatementInstance> i_stmtInstanceByGkId = new HashMap<String, DataStatementInstance>();

	// TT_LOW Consider provide some way to dispose the resultsets, statements attached to
	//   a connection when this is disposed.

	public GenSrListener() {
		super();
	}

	
	@Override
	public void flush() throws IOException {
		// Nothing to do
	}

	@Override
	public void process(LogEvent event) {
		switch (event.getType()) {
		case METHOD_SUCCESS:
			JdbcMethod l_method = event.getMethod();
			switch (l_method) {
			// Methods that alter transaction
			case CONN_setAutoCommitB:
			case CONN_commit:
			case CONN_rollback:
			case CONN_setSavepoint:
			case CONN_setSavepointS:
			case CONN_rollbackSv:
			case CONN_releaseSavepointSv:
			case CONN_close:
				handleTxEvent(event);
				break;
			// Methods that define/execute SQL
			case CONN_prepareStatementS:
			case CONN_prepareStatementSI:
			case CONN_prepareStatementSIa:
			case CONN_prepareStatementSII:
			case CONN_prepareStatementSIII:
			case CONN_prepareStatementSSa:
			case STMT_executeS:
			case STMT_executeSI:
			case STMT_executeSIa:
			case STMT_executeSSa:
			case STMT_executeQueryS:
			case STMT_executeUpdateS:
			case STMT_executeUpdateSI:
			case STMT_executeUpdateSIa:
			case STMT_executeUpdateSSa:
			case PRST_execute:
			case PRST_executeQuery:
			case PRST_executeUpdate:
			case STMT_getResultSet:
			case STMT_getGeneratedKeys:
				handleSqlEvent(event);
				break;
			// Methods that handle parameters
			// TT_LOW Handle cases (lobs, streams) where values are deferred
			case PRST_setArrayIAr:
			case PRST_setAsciiStreamIIs: case PRST_setAsciiStreamIIsI: case PRST_setAsciiStreamIIsL:
			case PRST_setBigDecimalIBd:
			case PRST_setBinaryStreamIIs: case PRST_setBinaryStreamIIsI: case PRST_setBinaryStreamIIsL:
			case PRST_setBlobIBl:case PRST_setBlobIIs:case PRST_setBlobIIsL:
			case PRST_setBooleanIB:
			case PRST_setByteIBy:case PRST_setBytesIBya:
			case PRST_setCharacterStreamIRd:case PRST_setCharacterStreamIRdI:case PRST_setCharacterStreamIRdL:
			case PRST_setClobICl:case PRST_setClobIRd:case PRST_setClobIRdL:
			case PRST_setDateIDa:case PRST_setDateIDaCa:
			case PRST_setDoubleID:
			case PRST_setFloatIF:
			case PRST_setIntII:
			case PRST_setLongIL:
			case PRST_setNCharacterStreamIRd:case PRST_setNCharacterStreamIRdL:
			case PRST_setNClobINc:case PRST_setNClobIRd:case PRST_setNClobIRdL:
			case PRST_setNStringIS:
			case PRST_setNullII:case PRST_setNullIIS:
			case PRST_setObjectIO:case PRST_setObjectIOI:case PRST_setObjectIOII:
			case PRST_setRefIRf:
			case PRST_setRowIdIRi:
			case PRST_setShortISh:
			case PRST_setSQLXMLISx:
			case PRST_setStringIS:
			case PRST_setTimeITi:case PRST_setTimeITiCa:
			case PRST_setTimestampITm:case PRST_setTimestampITmCa:
			case PRST_setUnicodeStreamIIsI:
			case PRST_setURLIUr:
				handlePsSetEvent(event);
				break;

			// Methods that handle results.
			// TT_LOW Handle cases with deferred values involved
			// TT_LOW Handle use of resultset cursors
			case RSST_getArrayI:		case RSST_getArrayS:
			case RSST_getAsciiStreamI:	case RSST_getAsciiStreamS:
			case RSST_getBigDecimalI:	case RSST_getBigDecimalII:
			case RSST_getBigDecimalS:	case RSST_getBigDecimalSI:
			case RSST_getBinaryStreamI:	case RSST_getBinaryStreamS:
			case RSST_getBlobI:			case RSST_getBlobS:
			case RSST_getBooleanI:		case RSST_getBooleanS:
			case RSST_getByteI:			case RSST_getByteS:
			case RSST_getBytesI:		case RSST_getBytesS:
			case RSST_getCharacterStreamI:case RSST_getCharacterStreamS:
			case RSST_getClobI:			case RSST_getClobS:
			case RSST_getDateI:			case RSST_getDateS:
			case RSST_getDateICa:		case RSST_getDateSCa:
			case RSST_getDoubleI:		case RSST_getDoubleS:
			case RSST_getFloatI:		case RSST_getFloatS:
			case RSST_getIntI:			case RSST_getIntS:
			case RSST_getLongI:			case RSST_getLongS:
			case RSST_getNCharacterStreamI:case RSST_getNCharacterStreamS:
			case RSST_getNClobI:		case RSST_getNClobS:
			case RSST_getNStringI:		case RSST_getNStringS:
			case RSST_getObjectI:		case RSST_getObjectS:
			case RSST_getObjectIMp:		case RSST_getObjectSMp:
			case RSST_getRefI:			case RSST_getRefS:
			case RSST_getRowIdI:		case RSST_getRowIdS:
			case RSST_getShortI:		case RSST_getShortS:
			case RSST_getSQLXMLI:		case RSST_getSQLXMLS:
			case RSST_getStringI:		case RSST_getStringS:
			case RSST_getTimeI:			case RSST_getTimeS:
			case RSST_getTimeICa:		case RSST_getTimeSCa:
			case RSST_getTimestampI:	case RSST_getTimestampS:
			case RSST_getTimestampICa:	case RSST_getTimestampSCa:
			case RSST_getUnicodeStreamI:case RSST_getUnicodeStreamS:
			case RSST_getURLI:			case RSST_getURLS:
			case RSST_next:
				handleRsEvent(event);
				break;
			case STMT_close:
				// TODO Clear links to statement (not alive)
				break;
			case RSST_close:
				// TODO Clear links to statement (not alive)
				break;
			}
			break;
		case CUSTOM:
			String l_tag = event.getCustomTag();
			if (l_tag.equals("CONN_new")) {
				handleConnNew(event);
			}
			// TT_LOW Else handle not-yet-implemented-wrapper-stream's deferred values.
			break;
		}
	}

	private void handleTxEvent(LogEvent event) {
		// TODO Auto-generated method stub
		JdbcMethod l_method = event.getMethod();
		DataConnection l_ctx = i_connCtxById.get(event.getConnectionId());
		switch(l_method) {
		case CONN_setAutoCommitB:
			l_ctx.setAutoCommit(Convert.toBoolean(event.getParams()[0]));
			// Inner commit invocation
			commitConnection(l_ctx);
			break;
		case CONN_commit:
			// Inner commit invocation
			commitConnection(l_ctx);
			break;
		case CONN_rollback:
			// Discard current connection information
			disposeConnectionData(l_ctx);
			break;
		case CONN_setSavepoint:
		case CONN_setSavepointS:
		case CONN_rollbackSv:
		case CONN_releaseSavepointSv:
			LOGGER.warn("savepoints not supported");
			break;
		case CONN_close:
			// Discard current connection information (Close == implicit rollback).
			disposeConnectionData(l_ctx);
			break;
		}
	}

	private void handleSqlEvent(LogEvent event) {
		JdbcMethod l_method = event.getMethod();
		Object l_eventResult = event.getResult();
		// Create / fetch statement instance.
		DataStatementInstance l_stmt;
		switch(l_method) {
		case CONN_prepareStatementS:
		case CONN_prepareStatementSI:
		case CONN_prepareStatementSIa:
		case CONN_prepareStatementSII:
		case CONN_prepareStatementSIII:
		case CONN_prepareStatementSSa:
		case STMT_executeS:
		case STMT_executeSI:
		case STMT_executeSIa:
		case STMT_executeSSa:
		case STMT_executeQueryS:
		case STMT_executeUpdateS:
		case STMT_executeUpdateSI:
		case STMT_executeUpdateSIa:
		case STMT_executeUpdateSSa:
			// Create statement instance
			String l_sql = (String)event.getParams()[0];
			l_sql = JdbcUtils.normalizeString(l_sql);
			String l_stmtId;
			if (l_eventResult.toString().startsWith("CpPreparedStatement:")){
				l_stmtId = l_eventResult.toString().substring("CpPreparedStatement:".length());
			} else {
				l_stmtId = event.getSrcId();
			}
			l_stmt = startStatement(event.getConnectionId(), l_stmtId, l_sql, event);

			break;
		default:
			// Statement retrieved
			l_stmt = i_stmtInstanceById.get(event.getSrcId());
		}
		
		// Execute statement (if applies)
		switch(l_method) {
		case STMT_executeS:
		case STMT_executeSI:
		case STMT_executeSIa:
		case STMT_executeSSa:
		case STMT_executeQueryS:
		case STMT_executeUpdateS:
		case STMT_executeUpdateSI:
		case STMT_executeUpdateSIa:
		case STMT_executeUpdateSSa:
		case PRST_execute:
		case PRST_executeQuery:
		case PRST_executeUpdate:
			executeStatement(l_stmt, event);
			break;
		}

		// Do result handling (if applies)
		switch(l_method) {
		case STMT_executeQueryS:
		case PRST_executeQuery:
		case STMT_getResultSet:
			// Returns ResultSet
			String l_rsId = l_eventResult.toString().substring("CpResultSet:".length());
			i_stmtInstanceByRsId.put(l_rsId, l_stmt);
			break;
		case PRST_executeUpdate:
		case STMT_executeUpdateS:
		case STMT_executeUpdateSI:
		case STMT_executeUpdateSIa:
		case STMT_executeUpdateSSa:
		case STMT_getUpdateCount:
			// TODO Mark as an update
			break;
		case STMT_executeS:
		case STMT_executeSI:
		case STMT_executeSIa:
		case STMT_executeSSa:
		case PRST_execute:
			// TODO Check if some handling should be provided
			break;
		case STMT_getGeneratedKeys:
			// Similar to resultset.
			l_rsId = l_eventResult.toString().substring("CpResultSet:".length());
			i_stmtInstanceByGkId .put(l_rsId, l_stmt);
			break;
		}
		
	}

	private void executeStatement(DataStatementInstance stmt, LogEvent event) {
		// # Start tx if applies
		DataConnection l_ctx = i_connCtxById.get(event.getConnectionId());
		DataSequenceInstance l_sq = l_ctx.getCurrentSequence();
		// if no current sequence, the tx will be started regardless the autocommit.
		if (l_sq == null) {
			l_sq = new DataSequenceInstance(l_ctx);
			l_ctx.setCurrentSequence(l_sq);
		}
		
		// # Add to sequence instance
		l_sq.addStatement(stmt);
		
		// # Commit if applies
		if (l_ctx.isAutoCommit()) {
			commitConnection(l_ctx);
			i_stmtInstanceById.remove(stmt.getStmtId());
		}
	}


	private void commitConnection(DataConnection connData) {
		DataSequenceInstance l_sq = connData.getCurrentSequence();
		if (l_sq != null) {
			// # Build statement class list
			ArrayList<DataStatementInstance> l_stIs = l_sq.getStatements(new ArrayList<DataStatementInstance>());
			ArrayList<DataStatementClass> l_stCs = new ArrayList<DataStatementClass>(l_stIs.size());
			for (DataStatementInstance l_stI : l_stIs) {
				String l_normSql = l_stI.getNormSql();
				// fetch/Generate statement class.
				DataStatementClass l_stC = i_stmtClassFromSql.get(l_normSql);
				boolean l_isUpdate = ! l_normSql.startsWith("select");
				if (l_stC == null) {
					l_stC = new DataStatementClass(l_normSql, l_isUpdate);
					i_stmtClassFromSql.put(l_normSql, l_stC);
				}
				l_stC.addInstance(l_stI);
				l_stCs.add(l_stC);
			}
	
			// # Fetch/Create sequence class
			DataStatementClass l_firstStC = l_stCs.get(0);
			ArrayList<DataSequenceClass> l_sqCCandidates = i_seqsByFirstStmt.get(l_firstStC);
			if (l_sqCCandidates == null) {
				l_sqCCandidates = new ArrayList<DataSequenceClass>();
				i_seqsByFirstStmt.put(l_firstStC, l_sqCCandidates);
			}
			DataSequenceClass l_sqC = null;
			for (DataSequenceClass l_candidate : l_sqCCandidates) {
				if (l_candidate.matchesStatements(l_stCs)) {
					l_sqC = l_candidate;
					break;
				}
			}
			// If not found create and register
			if (l_sqC == null) {
				l_sqC = new DataSequenceClass(l_stCs);
				l_sqCCandidates.add(l_sqC);
			}
			
			// # Feed the sequence class
			l_sqC.addSequenceInstance(l_sq);
		}
		// # Get rid of disposable data
		disposeConnectionData(connData);
	}
	
	private void rollbackConnection(DataConnection connData) {
		// # Get rid of disposable data
		disposeConnectionData(connData);
	}



	private void disposeConnectionData(DataConnection connData) {
		// TODO Auto-generated method stub
		ArrayList<DataStatementInstance> l_statements = connData.
				getLiveStatements(new ArrayList<DataStatementInstance>());
		ArrayList<String> l_resultSets = new ArrayList<String>();
		for (DataStatementInstance l_stmt : l_statements) {
			l_resultSets.clear();
			l_stmt.getLiveResultSets(l_resultSets);
			for (String l_idRs : l_resultSets) {
				i_stmtInstanceByRsId.remove(l_idRs);
				i_stmtInstanceByGkId.remove(l_idRs);
			}
		}
		connData.setCurrentSequence(null);
	}


	private void handlePsSetEvent(LogEvent event) {
		DataStatementInstance l_stmt = i_stmtInstanceById.get(event.getSrcId());
		Object[] l_setParameters = event.getParams();
		l_stmt.onSetParameter(event.getMethod(),
				Convert.toInteger(l_setParameters[0]),
				Arrays.copyOfRange(l_setParameters, 1, l_setParameters.length));
	}

	private void handleRsEvent(LogEvent event) {
		DataStatementInstance l_stmt = i_stmtInstanceByRsId.get(event.getSrcId());
		boolean l_fromRs = true;
		if (l_stmt == null) {
			l_stmt = i_stmtInstanceByGkId.get(event.getSrcId());
			l_fromRs = false;
		}
		if (l_stmt != null) {
			JdbcMethod l_method = event.getMethod();
			if (l_method == JdbcMethod.RSST_next) {
				if (l_fromRs) {
					l_stmt.onResultSetNext();
				} else {
					l_stmt.onGkNext();
				}
			} else {
				Object[] l_parameters = event.getParams();
				// TT_LOW Check rare get Methods with 2+ parameters.
				if (l_fromRs) {
					l_stmt.onResultSetGet(l_method, l_parameters[0], event.getResult());
				} else {
					l_stmt.onGkGet(l_method, l_parameters[0], event.getResult());
				}
			}
		}
	}

	private void handleConnNew(LogEvent event) {
		String l_connId = event.getSrcId();
		DataConnection l_context = new DataConnection();
		i_connCtxById.put(l_connId, l_context);
		
	}

	private DataStatementInstance startStatement(String connId, String stmtId, String sql, LogEvent event) {
		DataStatementInstance l_stmt = new DataStatementInstance(stmtId, sql, event.getStackTrace());
		i_stmtInstanceById.put(stmtId, l_stmt);
		DataConnection l_connData = i_connCtxById.get(connId);
		l_connData.addLiveStatement(l_stmt);
		return l_stmt;
	}
	

	public <TList extends List<? super DataStatementClass>> TList getStatementClasses(TList target) {
		target.addAll(i_stmtClassFromSql.values());
		return target;
	}
	
	public <TList extends List<? super DataSequenceClass>> TList getSequenceClasses(TList target) {
		for (List<DataSequenceClass> l_seqList : i_seqsByFirstStmt.values()) {
			target.addAll(l_seqList);
		}
		return target;
	}
	
}
