/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.tool.sr;

import java.io.*;
import java.util.*;

import org.apache.commons.codec.binary.StringUtils;
import org.apache.commons.xmlio.in.SimpleImporter;
import org.apache.commons.xmlio.out.XMLWriter;
import org.apache.log4j.PropertyConfigurator;
import org.xml.sax.InputSource;

import ddbc.tool.sr.SrDataSequence;
import ddbc.tool.sr.SrDataStatement;

import ddbc.cpn2.log.XmlWriterListener;
import ddbc.importers.LogEventImporter;

/**
 * CLI App for generating the DDBC Statement Report from the activity log.
 * Syntax<pre>java ddbc.tool.sr.MainGenSr &lr;inputFile&gt; &lr;outputFile&gt;</pre> 
 * @author F.Javier Aranda (fjarandag@gmail.com)
 */
public class MainGenSr {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			PropertyConfigurator.configure("log4j.properties");
			
			String l_strInput = args[0];
			String l_strOutput = args[1];
			FileInputStream l_fis = new FileInputStream(l_strInput);
			FileOutputStream l_fos = new FileOutputStream(l_strOutput);

			// complete the input file, which is missing the header and the footer.
			byte[] l_bytesHeader = StringUtils.getBytesUtf8(
					"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<log>\n");
			byte[] l_bytesFooter = StringUtils.getBytesUtf8("</log>\n");
			List<InputStream> l_inputs = Arrays.asList(
					new ByteArrayInputStream(l_bytesHeader),
					l_fis,
					new ByteArrayInputStream(l_bytesFooter));
			SequenceInputStream l_isSeq = new SequenceInputStream(
					Collections.enumeration(l_inputs));

			// The report data is computed by a LogEventListener, which is
			//  fed from the xml importer.
			GenSrListener l_listener = new GenSrListener();
			HashMap<String,String> l_attributes = new HashMap<String, String>();
			LogEventImporter l_importHandler = new LogEventImporter("/log", l_attributes, l_listener);

			// Consume input (computing intermediate)
			InputSource l_source = new InputSource(l_isSeq);
			SimpleImporter l_importer = new SimpleImporter();
			l_importer.addSimpleImportHandler(l_importHandler);
			l_importer.parse(l_source);
			
			writeStatementReport(l_listener, l_fos);

			l_fis.close();
			l_fos.close();
		} catch (Throwable t) {
			System.err.println("In MainGenSr.main()");
			t.printStackTrace(System.err);
			System.exit(1);
		}
	}

	private static void writeStatementReport(GenSrListener listener, OutputStream a_output)
			throws IOException {
//		int nextRefStatement = 1;
//		HashMap<SrDataStatement, Integer> l_stmtRefsByStmt = new HashMap<SrDataStatement, Integer>();
//		int nextRefSequence = 1;

		XMLWriter l_writer = new XMLWriter(new OutputStreamWriter(a_output, "UTF-8"));
		l_writer.writeXMLDeclaration();
//		l_writer.writeStartTag(XMLWriter.createStartTag("statementReport", new String[][]{
//				{"xmlns", "http://www.example.org/jdbc-proxy/statements-report"},
//				{"xmlns:xsi","http://www.w3.org/2001/XMLSchema-instance"},
//				{"xsi:schemaLocation","http://www.example.org/jdbc-proxy/statements-report statements-report.xsd"}
//		}));
		l_writer.writeStartTag("<statementReport>");

		l_writer.writeStartTag("<statements>");
		// TT_LOW Check if the data in the datastatement is pertinent
		ArrayList<DataStatementClass> l_statements = new ArrayList<DataStatementClass>();
		listener.getStatementClasses(l_statements);
		Collections.sort(l_statements);
		
		for (DataStatementClass l_statement : l_statements) {
//			int l_stmtRef = nextRefStatement++;
//			l_stmtRefsByStmt.put(l_statement, (Integer)l_stmtRef);
			l_writer.writeStartTag(XMLWriter.createStartTag(
					"statement", new String[][]{
							{"stmtId", l_statement.getSrId()}}
					));
			l_writer.writeElementWithCData("<normSql>",
					l_statement.getNormSql(), "</normSql>");
			ArrayList<StackTraceElement[]> l_framesList =
					l_statement.getFramesList(new ArrayList<StackTraceElement[]>());
			for (StackTraceElement[] l_location : l_framesList) {
				l_writer.writeStartTag("<location>");
				XmlWriterListener.writeFrames(l_writer, l_location);
				l_writer.writeEndTag("</location>");
			}			
			l_writer.writeEndTag("</statement>");
		}
		l_writer.writeEndTag("</statements>");

		l_writer.writeStartTag("<sequences>");
		ArrayList<DataSequenceClass> l_sequences =
				listener.getSequenceClasses(new ArrayList<DataSequenceClass>());
		Collections.sort(l_sequences);
		for (DataSequenceClass l_sequence : l_sequences) {
			l_writer.writeStartTag(XMLWriter.createStartTag(
					"sequence", new String[][]{
							{"seqId", l_sequence.getSrId()}}
					));

			l_writer.writeStartTag("<stmtRefs>");
			ArrayList<DataStatementClass> l_seqStatements =
					l_sequence.getStatements(new ArrayList<DataStatementClass>());
			for (DataStatementClass l_seqStmt : l_seqStatements) {
				l_writer.writeEmptyElement(XMLWriter.createEmptyTag("stmtRef",
						"idRef", l_seqStmt.getSrId()));
			}
			l_writer.writeEndTag("</stmtRefs>");
			
			l_writer.writeStartTag("<dataDependencies>");
			ArrayList<List<String>> l_dependGroups = l_sequence.getDependencyGroups(new ArrayList<List<String>>());
			for (List<String> l_group : l_dependGroups) {
				l_writer.writeStartTag("<group>", XMLWriter.NO_NEWLINE);
				// TT_LOW String utility function
				String l_sep = "";
				for (String l_item : l_group) {
					l_writer.write(l_sep);
					l_writer.write(l_item);
					l_sep = ",";
				}
				l_writer.writeEndTag("</group>", XMLWriter.NEWLINE);
			}
			
			l_writer.writeEndTag("</dataDependencies>");
			
			l_writer.writeEndTag("</sequence>");
		}
		l_writer.writeEndTag("</sequences>");
		
		l_writer.writeEndTag("</statementReport>");
		l_writer.flush();
	}


}
