/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc;

import java.sql.*;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.regex.Pattern;

import org.apache.commons.collections.Transformer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ddbc.importers.DriverImporter;

import ddbc.commons.util.Configs;

/**
 * Driver for decorating JDBC.
 * @author F.Javier Aranda (fjarandag@gmail.com)
 */
public class DDriver implements Driver {
	private static Logger LOGGER = LoggerFactory.getLogger(DDriver.class);
	private static boolean s_isInit = false;
	private static boolean s_overridesUrls = false;
	
	private static DDriver s_instance = new DDriver();
	public static DDriver getInstance() {
		return s_instance;
	}

	private static boolean s_registeredInDriverManager = false;
	static {
		if (!s_registeredInDriverManager) {
			try {
				DriverManager.registerDriver(getInstance());
			} catch (SQLException e) {
				LOGGER.error("registering driver", e);
			}
			s_registeredInDriverManager = true;
		}
	}

	public static String trimUrl(String url) {
		if (url.startsWith("ddbc:")) {
			return url.substring(5);
		} else {
			return url;
		}
	}
	
	/**
	 * Mapping from registered URL's to ConnectionFactories.
	 * The Factory is an Common Collections Transformer that turns connecting properties into the collection.
	 */
	private static Map<String, DDriverConnectionFactory> s_connFactoriesByUrl;
// DILEMMA: Use a match by pattern?
//  - Might make things more complex.
//  - Not really needed, as it would be unusual that two urls have the same schema permanently.
//	private static List<Pattern> s_urlPatterns;
//	private static List<Transformer> s_connMetaFactories;
	
	private static void ensureInit() {
		if (!s_isInit) {
			String driverConfigLocation = Configs.getStandAloneProperty("DDBC_CONFIG", null, "file:ddbc.xml");
			forceConfig(driverConfigLocation);
		}
	}
	private static void forceConfig(String cfgLocation) {
		try {
			LOGGER.info("Loading configuration from location: " + cfgLocation);
			Properties l_properties = System.getProperties();
			String l_configDirLoc = cfgLocation;
			if (l_configDirLoc.lastIndexOf('\\') > 0) {
				l_configDirLoc = l_configDirLoc.substring(0, l_configDirLoc.lastIndexOf('\\'));
			} else if (l_configDirLoc.lastIndexOf('/') > 0) {
				l_configDirLoc = l_configDirLoc.substring(0, l_configDirLoc.lastIndexOf('/'));
			} else if (l_configDirLoc.lastIndexOf(':') > 0) {
				l_configDirLoc = l_configDirLoc.substring(0, l_configDirLoc.lastIndexOf(':'));
			} else {
				l_configDirLoc = "";
			}
			
			DriverImporter importer = new DriverImporter();
			l_properties.put("ddbc_config.dirLoc", l_configDirLoc);
			importer.setImportProperties((Map)l_properties);
			Configs.parseXmlIoFromLocation(cfgLocation, importer);
			
			s_connFactoriesByUrl = importer.getConnFactoriesByUrl();
			s_overridesUrls = importer.getOverridesUrls();
			s_isInit = true;
		} catch (Exception e) {
			LOGGER.error("Could not load configuration from location: " + cfgLocation, e);
			
			// TODO Apply a safe configuration.
			// For example: Try to load the most common Drivers, accept none.
		}
	}

	/* (non-Javadoc)
	 * @see java.sql.Driver#acceptsURL(java.lang.String)
	 */
	public boolean acceptsURL(String url) throws SQLException {
		return fetchConnectionFactory(url) != null;
	}

	/* (non-Javadoc)
	 * @see java.sql.Driver#connect(java.lang.String, java.util.Properties)
	 */
	public Connection connect(String url, Properties info)
			throws SQLException {
		DDriverConnectionFactory connFactory = fetchConnectionFactory(url);
		if (connFactory == null) {
			throw new SQLException("No mapping for URL: " + url);
		} else {
			return (Connection)connFactory.createConnection(url, info);
		}
	}
	
	private DDriverConnectionFactory fetchConnectionFactory(String url) {
		ensureInit();
		String correctedUrl;
		if (url.startsWith("ddbc:") ) {
			correctedUrl = url.substring(5);
		} else if (s_overridesUrls) {
			correctedUrl = url;
		} else {
			// Not starting with prefix & not overriding.
			return null;
		}
		return s_connFactoriesByUrl.get(correctedUrl);
	}

	/* (non-Javadoc)
	 * @see java.sql.Driver#getMajorVersion()
	 */
	public int getMajorVersion() {
		return 0;
	}

	/* (non-Javadoc)
	 * @see java.sql.Driver#getMinorVersion()
	 */
	public int getMinorVersion() {
		return 0;
	}

	/* (non-Javadoc)
	 * @see java.sql.Driver#getPropertyInfo(java.lang.String, java.util.Properties)
	 */
	public DriverPropertyInfo[] getPropertyInfo(String a_url, Properties a_info)
			throws SQLException {
		// Not supported (at the moment)
		return new DriverPropertyInfo[]{};
	}

	/* (non-Javadoc)
	 * @see java.sql.Driver#jdbcCompliant()
	 */
	public boolean jdbcCompliant() {
		// We are not compliant unless we take the effort to pass some tests.
		return false;
	}

}
