/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.cpn2;

import java.sql.*;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.atomic.AtomicInteger;

import ddbc.DDriverConnectionFactory;
import ddbc.cpn2.impl.*;
import ddbc.cpn2.log.ConnectionLogger;
import ddbc.cpn2.log.CpLogger;
import ddbc.cpn2.st.HandlerMapper;
import ddbc.impl.DWrapperFactory;
import ddbc.commons.util.FlyWeight;

/**
 * Chain Proxy N2 Factory implementation for use from DDriver.
 * See sample configuration file <b>ddbc.xml</b> for options and details (corresponds to "handler" element).
 * @author F.Javier Aranda (fjarandag@gmail.com)
 *
 */
public class CpFactory extends DWrapperFactory {
	private CpLogger i_cpLogger;
	private HandlerMapper i_mapper;

	public static CpFactory createDumbInstance() {
		CpFactory l_instance = new CpFactory();
		// Attach a logger that does nothing.
		l_instance.i_cpLogger = new CpLogger();
		l_instance.i_mapper = HandlerMapper.dumbInstance();
		return l_instance;
	}
	
	public CpFactory() {
		super();
	}

	

	/**
	 * @return the cpLogger
	 */
	public CpLogger getLogger() {
		return i_cpLogger;
	}

	/**
	 * @param a_logger the cpLogger to set
	 */
	public void setLogger(CpLogger a_logger) {
		i_cpLogger = a_logger;
	}
	
	/**
	 * @return the mapper
	 */
	public HandlerMapper getMapper() {
		return i_mapper;
	}

	/**
	 * @param mapper the mapper to set
	 */
	public void setMapper(HandlerMapper mapper) {
		i_mapper = mapper;
	}



	private static Map<String,AtomicInteger> s_counters = new HashMap<String, AtomicInteger>();
	protected static String getNewId(Class<?> interfaceClass) {
		String l_idPrefix = JdbcMethod.prefixByReflectedClass.get(interfaceClass);
		if (l_idPrefix == null) { l_idPrefix = "Xxxx"; }
		AtomicInteger l_intGen = s_counters.get(l_idPrefix);
		if (l_intGen == null) {
			l_intGen = new AtomicInteger(0);
			s_counters.put(l_idPrefix, l_intGen);
		}
		int l_counter = l_intGen.getAndIncrement();
		DecimalFormat l_intFormat = FlyWeight.getDecimalFormat("0000");
		return l_idPrefix + l_intFormat.format(l_counter);
	}
	
	/**
	 * Obtains a wrapper connection which delegates on the given one.
	 * @param wrapped Given actual connection.
	 * @return Wrapper connection.
	 */
	public CpConnection wrapConnection(Connection wrapped) throws SQLException {
		// When: CpConnection, etc. needs setted.
		String l_newId = getNewId(Connection.class);
		ConnectionLogger l_newLogger = new ConnectionLogger(i_cpLogger, l_newId);
		CpConnection l_wrapper = new CpConnection(wrapped, this, l_newLogger, i_mapper, l_newId);
		return l_wrapper;
	}
	
	public DDriverConnectionFactory wrapConnectionFactory(final DDriverConnectionFactory factory) {
		return new DDriverConnectionFactory() {
			@Override
			public Connection createConnection(String url, Properties connProperties) throws SQLException {
				Connection l_conn = factory.createConnection(url, connProperties);
				l_conn = CpFactory.this.wrapConnection(l_conn);
				return l_conn;
			}
		};
	}

	/**
	 * Obtains a wrapper Statement which delegates on the given one.
	 * @param wrapped Given actual statement
	 * @return Wrapper statement
	 */
	public CpStatement wrapStatement(Statement wrapped, Connection conn) throws SQLException {
		CpConnection l_cpConn = (CpConnection)conn;
		CpStatement l_stmt = new CpStatement(l_cpConn, wrapped, getNewId(Statement.class));
		return l_stmt;
	}
	
	/**
	 * Obtains a wrapper PreparedStatement which delegates on the given one.
	 * @param conn Associated wrapper connection.
	 * @param ps Given actual prepared statement
	 * @return  Wrapper prepared statement
	 */
	public CpPreparedStatement wrapPreparedStatement(PreparedStatement wrapped,
			Connection conn) throws SQLException {
		CpConnection l_cpConn = (CpConnection)conn;
		CpPreparedStatement l_ps = new CpPreparedStatement(l_cpConn, wrapped,
				getNewId(PreparedStatement.class));
		return l_ps;
	}
	
	/**
	 * Obtains a wrapper CallableStatement which delegates on the given one.
	 * @param wrapped Given actual callable statement
	 * @param conn Associated wrapper connection.
	 * @return  Wrapper callable statement
	 */
	public CpCallableStatement wrapCallableStatement(CallableStatement wrapped, Connection conn) throws SQLException {
		CpConnection l_cpConn = (CpConnection)conn;
		CpCallableStatement l_cs = new CpCallableStatement(l_cpConn, wrapped,
				getNewId(CallableStatement.class));
		return l_cs;
	}
	
	/**
	 * Obtains a wrapper Blob which delegates on the given one.
	 * @param wrapped Given actual blob object.
	 * @param conn Associated wrapper connection.
	 * @return Wrapper blob object.
	 */
	public CpBlob wrapBlob(Blob wrapped, Connection conn) throws SQLException {
		CpConnection l_cpConn = (CpConnection)conn;
		return new CpBlob(l_cpConn, wrapped, getNewId(Blob.class));
	}

	/**
	 * Obtains a wrapper Clob which delegates on the given one.
	 * @param conn Associated wrapper connection.
	 * @param blob Given actual clob object.
	 * @return Wrapper clob object.
	 */
	public CpClob wrapClob(Clob wrapped, Connection conn) throws SQLException {
		CpConnection l_cpConn = (CpConnection)conn;
		return new CpClob(l_cpConn, wrapped, getNewId(Clob.class));
	}

	/**
	 * Obtains a wrapper NClob which delegates on the given one.
	 * @param conn Associated wrapper connection.
	 * @param blob Given actual nclob object.
	 * @return Wrapper nclob object.
	 */
	public CpNClob wrapNClob(NClob wrapped, Connection conn) throws SQLException {
		CpConnection l_cpConn = (CpConnection)conn;
		return new CpNClob(l_cpConn, wrapped, getNewId(NClob.class));
	}

	public CpResultSet wrapResultSet(ResultSet wrapped, Statement stmt) throws SQLException {
		CpStatement l_cpStmt = (CpStatement)stmt;
		return new CpResultSet(l_cpStmt, wrapped, getNewId(ResultSet.class));
	}

	public CpResultSet wrapResultSet(ResultSet wrapped, Connection conn) throws SQLException {
		CpConnection l_cpConn = (CpConnection)conn;
		return new CpResultSet(l_cpConn, wrapped, getNewId(ResultSet.class));
	}

	public CpArray wrapArray(Array wrapped, ResultSet rs) {
		CpResultSet l_cpRs = (CpResultSet)rs;
		return new CpArray(l_cpRs, wrapped, getNewId(Array.class));
	}

	public CpRef wrapRef(Ref wrapped, ResultSet rs) throws SQLException {
		CpResultSet l_cpRs = (CpResultSet)rs;
		return new CpRef(l_cpRs.cpConnection(), wrapped, getNewId(Ref.class));
	}
	/** @deprecated Use {@link #wrapRef(Ref, Connection)} */
	public CpRef wrapRef(Ref wrapped, Connection conn) throws SQLException {
		CpConnection l_cpConn = (CpConnection)conn;
		return new CpRef(l_cpConn, wrapped, getNewId(Ref.class));
	}
	
	public CpStruct wrapStruct(Struct wrapped, ResultSet rs) throws SQLException {
		CpResultSet l_cpRs = (CpResultSet)rs;
		return new CpStruct(l_cpRs, wrapped, getNewId(Struct.class));
	}

	public CpStruct wrapStruct(Struct wrapped, Connection conn) throws SQLException {
		CpConnection l_cpConn = (CpConnection)conn;
		return new CpStruct(l_cpConn, wrapped, getNewId(Struct.class));
	}

	public CpSavepoint wrapSavepoint(Savepoint wrapped, Connection conn) throws SQLException {
		CpConnection l_cpConn = (CpConnection)conn;
		return new CpSavepoint(l_cpConn, wrapped, getNewId(Savepoint.class));
	}

	public CpDatabaseMetaData wrapDatabaseMetaData(DatabaseMetaData wrapped, Connection conn) throws SQLException {
		CpConnection l_cpConn = (CpConnection)conn;
		return new CpDatabaseMetaData(l_cpConn, wrapped, getNewId(DatabaseMetaData.class));
	}
	public CpParameterMetaData wrapParameterMetaData(ParameterMetaData wrapped, PreparedStatement ps) throws SQLException {
		CpPreparedStatement l_cpPs = (CpPreparedStatement)ps;
		return new CpParameterMetaData(l_cpPs, wrapped, getNewId(ParameterMetaData.class));
	}
	public CpResultSetMetaData  wrapResultSetMetaData(ResultSetMetaData wrapped, ResultSet rs) throws SQLException {
		CpResultSet l_cpRs = (CpResultSet)rs;
		return new CpResultSetMetaData(l_cpRs, wrapped, getNewId(ResultSetMetaData.class));
	}
	public CpResultSetMetaData  wrapResultSetMetaData(ResultSetMetaData wrapped, PreparedStatement ps) throws SQLException {
		CpPreparedStatement l_cpPs = (CpPreparedStatement)ps;
		return new CpResultSetMetaData(l_cpPs, wrapped, getNewId(ResultSetMetaData.class));
	}
}
