/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.cpn2.impl;

import java.io.*;
import java.sql.Clob;
import java.sql.SQLException;

import ddbc.cpn2.CpUtils;
import ddbc.cpn2.CpWrapper;
import ddbc.cpn2.JdbcMethod;
import ddbc.cpn2.log.ConnectionLogger;
import ddbc.impl.DConnection;

//TODO Interesaría un proxy del Writer/Stream para controlar qué valores se introducen.
/**
 * Base Proxy/Decorator class for a java.sql.Clob.
 * It's just a filter-delegate implementation, ready to override in a subclass.
 * @author F.Javier Aranda (fjarandag@gmail.com)
 */
public class CpClob implements Clob, CpWrapper {
	/** JpConnection used to create the Clob. */
	private CpConnection i_cpConnection;
	/** Wrapped Clob object. */
	private Clob i_wrapped;
	private ConnectionLogger i_logger;
	private String i_objId;

	// Default constructor, just in case a subclass needs it.
	protected CpClob() {
		super();
	}

	/**
	 * @param connection
	 * @param wrapped
	 */
	public CpClob(CpConnection connection, Clob wrapped, String objId) {
		super();
		i_cpConnection = connection;
		i_wrapped = wrapped;
		i_logger = connection.cpLogger();
		i_objId = objId;
	}

	/**
	 * Return the wrapped clob object.
	 */
	public Clob cpWrapped() {
		return i_wrapped;
	}

	public CpConnection cpConnection() {
		return (CpConnection)i_cpConnection;
	}
	
	public String cpObjectId() {
		return i_objId;
	}

	////////////////////////////////////////////////////////////////////////////
	///{AUTOGEN:FRAME_BEGIN}

	//{AUTOGEN:CLOB_free:BEGIN}
	public void free() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CLOB_free);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.free();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CLOB_free, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CLOB_free, null);
	}
	//{AUTOGEN:CLOB_free:END}
	//{AUTOGEN:CLOB_getAsciiStream:BEGIN}
	public InputStream getAsciiStream() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CLOB_getAsciiStream);
		InputStream l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getAsciiStream();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CLOB_getAsciiStream, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CLOB_getAsciiStream, l_result);
			return l_result;
	}
	//{AUTOGEN:CLOB_getAsciiStream:END}
	//{AUTOGEN:CLOB_getCharacterStream:BEGIN}
	public Reader getCharacterStream() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CLOB_getCharacterStream);
		Reader l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getCharacterStream();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CLOB_getCharacterStream, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CLOB_getCharacterStream, l_result);
			return l_result;
	}
	//{AUTOGEN:CLOB_getCharacterStream:END}
	//{AUTOGEN:CLOB_getCharacterStreamLL:BEGIN}
	public Reader getCharacterStream(long a_pos, long a_length) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CLOB_getCharacterStreamLL,a_pos, a_length);
		Reader l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getCharacterStream(a_pos, a_length);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CLOB_getCharacterStreamLL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CLOB_getCharacterStreamLL, l_result);
			return l_result;
	}
	//{AUTOGEN:CLOB_getCharacterStreamLL:END}
	//{AUTOGEN:CLOB_getSubStringLI:BEGIN}
	public String getSubString(long a_pos, int a_length) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CLOB_getSubStringLI,a_pos, a_length);
		String l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getSubString(a_pos, a_length);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CLOB_getSubStringLI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CLOB_getSubStringLI, l_result);
			return l_result;
	}
	//{AUTOGEN:CLOB_getSubStringLI:END}
	//{AUTOGEN:CLOB_length:BEGIN}
	public long length() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CLOB_length);
		long l_result = 0L;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.length();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CLOB_length, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CLOB_length, l_result);
			return l_result;
	}
	//{AUTOGEN:CLOB_length:END}
	//{AUTOGEN:CLOB_positionClL:BEGIN}
	public long position(Clob a_searchstr, long a_start) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CLOB_positionClL,a_searchstr, a_start);
		long l_result = 0L;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.position(a_searchstr, a_start);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CLOB_positionClL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CLOB_positionClL, l_result);
			return l_result;
	}
	//{AUTOGEN:CLOB_positionClL:END}
	//{AUTOGEN:CLOB_positionSL:BEGIN}
	public long position(String a_searchstr, long a_start) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CLOB_positionSL,a_searchstr, a_start);
		long l_result = 0L;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.position(a_searchstr, a_start);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CLOB_positionSL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CLOB_positionSL, l_result);
			return l_result;
	}
	//{AUTOGEN:CLOB_positionSL:END}
	//{AUTOGEN:CLOB_setAsciiStreamL:BEGIN}
	public OutputStream setAsciiStream(long a_pos) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CLOB_setAsciiStreamL,a_pos);
		OutputStream l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.setAsciiStream(a_pos);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CLOB_setAsciiStreamL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CLOB_setAsciiStreamL, l_result);
			return l_result;
	}
	//{AUTOGEN:CLOB_setAsciiStreamL:END}
	//{AUTOGEN:CLOB_setCharacterStreamL:BEGIN}
	public Writer setCharacterStream(long a_pos) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CLOB_setCharacterStreamL,a_pos);
		Writer l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.setCharacterStream(a_pos);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CLOB_setCharacterStreamL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CLOB_setCharacterStreamL, l_result);
			return l_result;
	}
	//{AUTOGEN:CLOB_setCharacterStreamL:END}
	//{AUTOGEN:CLOB_setStringLSII:BEGIN}
	public int setString(long a_pos, String a_str, int a_offset, int a_len) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CLOB_setStringLSII,a_pos, a_str, a_offset, a_len);
		int l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.setString(a_pos, a_str, a_offset, a_len);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CLOB_setStringLSII, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CLOB_setStringLSII, l_result);
			return l_result;
	}
	//{AUTOGEN:CLOB_setStringLSII:END}
	//{AUTOGEN:CLOB_setStringLS:BEGIN}
	public int setString(long a_pos, String a_str) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CLOB_setStringLS,a_pos, a_str);
		int l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.setString(a_pos, a_str);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CLOB_setStringLS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CLOB_setStringLS, l_result);
			return l_result;
	}
	//{AUTOGEN:CLOB_setStringLS:END}
	//{AUTOGEN:CLOB_truncateL:BEGIN}
	public void truncate(long a_len) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CLOB_truncateL,a_len);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.truncate(a_len);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CLOB_truncateL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CLOB_truncateL, null);
	}
	//{AUTOGEN:CLOB_truncateL:END}
	///{AUTOGEN:FRAME_END}
	////////////////////////////////////////////////////////////////////////////

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "CpClob:" + i_objId;
	}

}
