/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.cpn2.impl;

import java.sql.*;
import java.util.Map;
import java.util.Properties;

import ddbc.cpn2.*;
import ddbc.cpn2.log.ConnectionLogger;
import ddbc.cpn2.st.HandlerMapper;

/**
 * Base Proxy/Decorator for the Connection interface.
 * Just works as a filter for the wrapped connection (wrapping generated objects if applies).
 * It is expected to be extended to provide some functionality.
 * @author F.Javier Aranda (fjarandag@gmail.com)
 */
public class CpConnection implements Connection, CpWrapper {
	/** The wrapped Connection, on which to delegate. */
	private Connection i_wrapped;
	/** Factory used to create wrapper objects.
	 * By default has a factory for the base objects, which may not do
	 * anything interesting. */
	// TODO Check if a default instance is really needed (causes zombie instances of CpLogger).
	private CpFactory i_factory = CpFactory.createDumbInstance();

	private ConnectionLogger i_logger;
	
	private HandlerMapper i_mapper;
	
	private String i_objId;
	
	private ConnectionAdaptorContext i_adaptorContext;

	// Default constructor, Just in case a subclass doesn't fit the informed constructor
	protected CpConnection() {
		super();
	}

	public CpConnection(Connection wrapped, CpFactory factory,
			ConnectionLogger logger, HandlerMapper mapper, String connId) {
		super();
		i_wrapped = wrapped;
		i_factory = factory;
		i_logger = logger;
		i_mapper = mapper;
		i_objId = connId;
		i_adaptorContext = new ConnectionAdaptorContext();
		logger.logCustom(connId, "CONN_new");
	}

	///////////////////////////////////////////////////////
	// BOILERPLATE
	//


	///////////////////////////////////
	// FIELD ACCESORS
	//
	/**
	 * Returns the wrapped Connection object
	 */
	public Connection cpWrapped() {
		return i_wrapped;
	}
	

	/**
	 * Returns the wrapper factory
	 * @return JpWrapperFactory this instance uses to create wrapped objects for statements and lobs.
	 */
	public CpFactory cpFactory() {
		return i_factory;
	}

	public ConnectionLogger cpLogger() {
		return i_logger;
	}
	
	public String cpObjectId() {
		return i_objId;
	}
	
	public ConnectionAdaptorContext cpContext() {
		return i_adaptorContext;
	}
	
	public HandlerMapper cpMapper() {
		return i_mapper;
	}
	
	///////////////////////////////////
	// IMPLEMENT/DELEGATE (Statement)
	//
	
	
	////////////////////////////////////////////////////////////////////////////
	///{AUTOGEN:FRAME_BEGIN}

	//{AUTOGEN:CONN_clearWarnings:BEGIN}
	public void clearWarnings() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_clearWarnings);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.clearWarnings();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_clearWarnings, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_clearWarnings, null);
	}
	//{AUTOGEN:CONN_clearWarnings:END}
	//{AUTOGEN:CONN_close:BEGIN}
	public void close() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_close);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.close();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_close, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_close, null);
	}
	//{AUTOGEN:CONN_close:END}
	//{AUTOGEN:CONN_commit:BEGIN}
	public void commit() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_commit);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.commit();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_commit, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_commit, null);
	}
	//{AUTOGEN:CONN_commit:END}
	//{AUTOGEN:CONN_createBlob:BEGIN}
	public Blob createBlob() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_createBlob);
		Blob l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.createBlob();
			l_result = i_factory.wrapBlob(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_createBlob, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_createBlob, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_createBlob:END}
	//{AUTOGEN:CONN_createClob:BEGIN}
	public Clob createClob() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_createClob);
		Clob l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.createClob();
			l_result = i_factory.wrapClob(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_createClob, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_createClob, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_createClob:END}
	//{AUTOGEN:CONN_createNClob:BEGIN}
	public NClob createNClob() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_createNClob);
		NClob l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.createNClob();
			l_result = i_factory.wrapNClob(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_createNClob, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_createNClob, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_createNClob:END}
	//{AUTOGEN:CONN_createSQLXML:BEGIN}
	public SQLXML createSQLXML() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_createSQLXML);
		SQLXML l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.createSQLXML();
			l_result = i_factory.wrapSqlXml(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_createSQLXML, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_createSQLXML, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_createSQLXML:END}
	//{AUTOGEN:CONN_createStatement:BEGIN}
	public Statement createStatement() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_createStatement);
		Statement l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_mapper.createStatement(this, JdbcMethod.CONN_createStatement);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_createStatement, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_createStatement, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_createStatement:END}
	//{AUTOGEN:CONN_createStatementII:BEGIN}
	public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_createStatementII, resultSetType, resultSetConcurrency);
		Statement l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_mapper.createStatement(this, JdbcMethod.CONN_createStatementII, resultSetType, resultSetConcurrency);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_createStatementII, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_createStatementII, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_createStatementII:END}
	//{AUTOGEN:CONN_createStatementIII:BEGIN}
	public Statement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_createStatementIII,
				resultSetType, resultSetConcurrency, resultSetHoldability);
		Statement l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_mapper.createStatement(this, JdbcMethod.CONN_createStatementIII,
					resultSetType, resultSetConcurrency, resultSetHoldability);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_createStatementIII, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_createStatementIII, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_createStatementIII:END}
	//{AUTOGEN:CONN_getAutoCommit:BEGIN}
	public boolean getAutoCommit() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_getAutoCommit);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getAutoCommit();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_getAutoCommit, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_getAutoCommit, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_getAutoCommit:END}
	//{AUTOGEN:CONN_getCatalog:BEGIN}
	public String getCatalog() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_getCatalog);
		String l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getCatalog();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_getCatalog, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_getCatalog, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_getCatalog:END}
	//{AUTOGEN:CONN_getClientInfo:BEGIN}
	public Properties getClientInfo() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_getClientInfo);
		Properties l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getClientInfo();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_getClientInfo, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_getClientInfo, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_getClientInfo:END}
	//{AUTOGEN:CONN_getClientInfoS:BEGIN}
	public String getClientInfo(String name) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_getClientInfoS,name);
		String l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getClientInfo(name);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_getClientInfoS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_getClientInfoS, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_getClientInfoS:END}
	//{AUTOGEN:CONN_getHoldability:BEGIN}
	public int getHoldability() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_getHoldability);
		int l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getHoldability();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_getHoldability, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_getHoldability, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_getHoldability:END}
	//{AUTOGEN:CONN_getMetaData:BEGIN}
	public DatabaseMetaData getMetaData() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_getMetaData);
		DatabaseMetaData l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Custom Information will be needed to match the virtual schema.
			l_result = i_wrapped.getMetaData();
			l_result = i_factory.wrapDatabaseMetaData(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_getMetaData, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_getMetaData, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_getMetaData:END}
	//{AUTOGEN:CONN_getTransactionIsolation:BEGIN}
	public int getTransactionIsolation() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_getTransactionIsolation);
		int l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getTransactionIsolation();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_getTransactionIsolation, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_getTransactionIsolation, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_getTransactionIsolation:END}
	//{AUTOGEN:CONN_getTypeMap:BEGIN}
	public Map<String,Class<?>> getTypeMap() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_getTypeMap);
		Map<String,Class<?>> l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getTypeMap();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_getTypeMap, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_getTypeMap, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_getTypeMap:END}
	//{AUTOGEN:CONN_getWarnings:BEGIN}
	public SQLWarning getWarnings() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_getWarnings);
		SQLWarning l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getWarnings();
			// TT_LOW Wrap SQLWarning
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_getWarnings, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_getWarnings, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_getWarnings:END}
	//{AUTOGEN:CONN_isClosed:BEGIN}
	public boolean isClosed() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_isClosed);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.isClosed();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_isClosed, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_isClosed, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_isClosed:END}
	//{AUTOGEN:CONN_isReadOnly:BEGIN}
	public boolean isReadOnly() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_isReadOnly);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.isReadOnly();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_isReadOnly, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_isReadOnly, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_isReadOnly:END}
	//{AUTOGEN:CONN_isValidI:BEGIN}
	public boolean isValid(int timeout) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_isValidI,timeout);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.isValid(timeout);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_isValidI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_isValidI, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_isValidI:END}
	//{AUTOGEN:CONN_nativeSQLS:BEGIN}
	public String nativeSQL(String sql) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_nativeSQLS,sql);
		String l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW It's a different translation than ours,
			//   but should the sql be translated before into the actual schema ??
			l_result = i_wrapped.nativeSQL(sql);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_nativeSQLS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_nativeSQLS, l_result);
		return l_result;
	}
	//{AUTOGEN:CONN_nativeSQLS:END}
	//{AUTOGEN:CONN_prepareCallS:BEGIN}
	public CallableStatement prepareCall(String sql) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_prepareCallS,sql);
		CallableStatement l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW CallableStatements are not adapted by the moment.
			l_result = i_wrapped.prepareCall(sql);
			l_result = i_factory.wrapCallableStatement(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_prepareCallS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_prepareCallS, l_result);
		return l_result;
	}
	//{AUTOGEN:CONN_prepareCallS:END}
	//{AUTOGEN:CONN_prepareCallSII:BEGIN}
	public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_prepareCallSII,sql, resultSetType, resultSetConcurrency);
		CallableStatement l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW CallableStatements are not adapted by the moment.
			l_result = i_wrapped.prepareCall(sql, resultSetType, resultSetConcurrency);
			l_result = i_factory.wrapCallableStatement(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_prepareCallSII, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_prepareCallSII, l_result);
		return l_result;
	}
	//{AUTOGEN:CONN_prepareCallSII:END}
	//{AUTOGEN:CONN_prepareCallSIII:BEGIN}
	public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_prepareCallSIII,sql, resultSetType, resultSetConcurrency, resultSetHoldability);
		CallableStatement l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW CallableStatements are not adapted by the moment.
			l_result = i_wrapped.prepareCall(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
			l_result = i_factory.wrapCallableStatement(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_prepareCallSIII, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_prepareCallSIII, l_result);
		return l_result;
	}
	//{AUTOGEN:CONN_prepareCallSIII:END}
	//{AUTOGEN:CONN_prepareStatementS:BEGIN}
	public PreparedStatement prepareStatement(String sql) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_prepareStatementS,sql);
		PreparedStatement l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_mapper.prepareStatement(this, JdbcMethod.CONN_prepareStatementS, sql);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_prepareStatementS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_prepareStatementS, l_result);
		return l_result;
	}
	//{AUTOGEN:CONN_prepareStatementS:END}
	//{AUTOGEN:CONN_prepareStatementSI:BEGIN}
	public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_prepareStatementSI,sql, autoGeneratedKeys);
		PreparedStatement l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_mapper.prepareStatement(this, JdbcMethod.CONN_prepareStatementSI, sql, autoGeneratedKeys);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_prepareStatementSI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_prepareStatementSI, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_prepareStatementSI:END}
	//{AUTOGEN:CONN_prepareStatementSIa:BEGIN}
	public PreparedStatement prepareStatement(String sql, int[] columnIndexes) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_prepareStatementSIa,sql, columnIndexes);
		PreparedStatement l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_mapper.prepareStatement(this, JdbcMethod.CONN_prepareStatementSIa, sql, columnIndexes);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_prepareStatementSIa, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_prepareStatementSIa, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_prepareStatementSIa:END}
	//{AUTOGEN:CONN_prepareStatementSII:BEGIN}
	public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_prepareStatementSII,sql, resultSetType, resultSetConcurrency);
		PreparedStatement l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_mapper.prepareStatement(this, JdbcMethod.CONN_prepareStatementSII, sql, resultSetType, resultSetConcurrency);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_prepareStatementSII, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_prepareStatementSII, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_prepareStatementSII:END}
	//{AUTOGEN:CONN_prepareStatementSIII:BEGIN}
	public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_prepareStatementSIII,sql, resultSetType, resultSetConcurrency, resultSetHoldability);
		PreparedStatement l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_mapper.prepareStatement(this, JdbcMethod.CONN_prepareStatementSIII, sql, resultSetType, resultSetConcurrency, resultSetHoldability);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_prepareStatementSIII, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_prepareStatementSIII, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_prepareStatementSIII:END}
	//{AUTOGEN:CONN_prepareStatementSSa:BEGIN}
	public PreparedStatement prepareStatement(String sql, String[] columnNames) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_prepareStatementSSa,sql, columnNames);
		PreparedStatement l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_mapper.prepareStatement(this, JdbcMethod.CONN_prepareStatementSSa, sql, (Object)columnNames);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_prepareStatementSSa, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_prepareStatementSSa, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_prepareStatementSSa:END}
	//{AUTOGEN:CONN_releaseSavepointSv:BEGIN}
	public void releaseSavepoint(Savepoint savepoint) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_releaseSavepointSv,savepoint);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.releaseSavepoint(savepoint);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_releaseSavepointSv, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_releaseSavepointSv, null);
	}
	//{AUTOGEN:CONN_releaseSavepointSv:END}
	//{AUTOGEN:CONN_rollback:BEGIN}
	public void rollback() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_rollback);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.rollback();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_rollback, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_rollback, null);
	}
	//{AUTOGEN:CONN_rollback:END}
	//{AUTOGEN:CONN_rollbackSv:BEGIN}
	public void rollback(Savepoint savepoint) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_rollbackSv,savepoint);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.rollback(savepoint);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_rollbackSv, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_rollbackSv, null);
	}
	//{AUTOGEN:CONN_rollbackSv:END}
	//{AUTOGEN:CONN_setAutoCommitB:BEGIN}
	public void setAutoCommit(boolean autoCommit) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_setAutoCommitB,autoCommit);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.setAutoCommit(autoCommit);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_setAutoCommitB, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_setAutoCommitB, null);
	}
	//{AUTOGEN:CONN_setAutoCommitB:END}
	//{AUTOGEN:CONN_setCatalogS:BEGIN}
	public void setCatalog(String catalog) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_setCatalogS,catalog);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.setCatalog(catalog);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_setCatalogS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_setCatalogS, null);
	}
	//{AUTOGEN:CONN_setCatalogS:END}
	//{AUTOGEN:CONN_setClientInfoPr:BEGIN}
	public void setClientInfo(Properties properties) {
		i_logger.logStart(i_objId, JdbcMethod.CONN_setClientInfoPr,properties);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.setClientInfo(properties);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchUnchecked(i_logger, i_objId, JdbcMethod.CONN_setClientInfoPr, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_setClientInfoPr, null);
	}
	//{AUTOGEN:CONN_setClientInfoPr:END}
	//{AUTOGEN:CONN_setClientInfoSS:BEGIN}
	public void setClientInfo(String name, String value) {
		i_logger.logStart(i_objId, JdbcMethod.CONN_setClientInfoSS,name, value);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.setClientInfo(name, value);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchUnchecked(i_logger, i_objId, JdbcMethod.CONN_setClientInfoSS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_setClientInfoSS, null);
	}
	//{AUTOGEN:CONN_setClientInfoSS:END}
	//{AUTOGEN:CONN_setHoldabilityI:BEGIN}
	public void setHoldability(int holdability) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_setHoldabilityI,holdability);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.setHoldability(holdability);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_setHoldabilityI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_setHoldabilityI, null);
	}
	//{AUTOGEN:CONN_setHoldabilityI:END}
	//{AUTOGEN:CONN_setReadOnlyB:BEGIN}
	public void setReadOnly(boolean readOnly) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_setReadOnlyB,readOnly);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.setReadOnly(readOnly);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_setReadOnlyB, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_setReadOnlyB, null);
	}
	//{AUTOGEN:CONN_setReadOnlyB:END}
	//{AUTOGEN:CONN_setSavepoint:BEGIN}
	public Savepoint setSavepoint() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_setSavepoint);
		Savepoint l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.setSavepoint();
			l_result = cpFactory().wrapSavepoint(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_setSavepoint, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_setSavepoint, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_setSavepoint:END}
	//{AUTOGEN:CONN_setSavepointS:BEGIN}
	public Savepoint setSavepoint(String name) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_setSavepointS,name);
		Savepoint l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.setSavepoint(name);
			l_result = cpFactory().wrapSavepoint(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_setSavepointS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_setSavepointS, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_setSavepointS:END}
	//{AUTOGEN:CONN_setTransactionIsolationI:BEGIN}
	public void setTransactionIsolation(int level) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_setTransactionIsolationI,level);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.setTransactionIsolation(level);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_setTransactionIsolationI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_setTransactionIsolationI, null);
	}
	//{AUTOGEN:CONN_setTransactionIsolationI:END}
	//{AUTOGEN:CONN_setTypeMapMp:BEGIN}
	public void setTypeMap(Map<String, Class<?>> map) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_setTypeMapMp,map);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.setTypeMap(map);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_setTypeMapMp, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_setTypeMapMp, null);
	}
	//{AUTOGEN:CONN_setTypeMapMp:END}
	//{AUTOGEN:CONN_createArrayOfSOa:BEGIN}
	public Array createArrayOf(String typeName, Object[] elements) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_createArrayOfSOa,typeName, elements);
		Array l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.createArrayOf(typeName, elements);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_createArrayOfSOa, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_createArrayOfSOa, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_createArrayOfSOa:END}
	//{AUTOGEN:CONN_createStructSOa:BEGIN}
	public Struct createStruct(String typeName, Object[] attributes) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.CONN_createStructSOa,typeName, attributes);
		Struct l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.createStruct(typeName, attributes);
			l_result = cpFactory().wrapStruct(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.CONN_createStructSOa, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.CONN_createStructSOa, l_result);
			return l_result;
	}
	//{AUTOGEN:CONN_createStructSOa:END}
	///{AUTOGEN:FRAME_END}



	@Override
	public boolean isWrapperFor(Class<?> iface) throws SQLException {
		// TT_LOW Some day will have to find if this is what is expected...
		if (iface.isAssignableFrom(i_wrapped.getClass())) {
			return true;
		} else {
			return i_wrapped.isWrapperFor(iface);
		}
	}

	@Override
	public <T> T unwrap(Class<T> arg0) throws SQLException {
		// TT_LOW Some day will have to find if this is what is expected...
		if (arg0.isAssignableFrom(i_wrapped.getClass())) {
			return (T)i_wrapped;
		} else {
			return i_wrapped.unwrap(arg0);
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "CpConnection:" + i_objId;
	}


}