/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.cpn2.impl;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.*;
import java.util.Calendar;
import java.util.Map;

import ddbc.cpn2.*;
import ddbc.cpn2.log.ConnectionLogger;

/**
 * Base Decorator for ResultSet
 * @author F.Javier Aranda (fjarandag@gmail.com)
 *
 */
public class CpResultSet implements ResultSet, CpWrapper {
	private CpStatement i_cpStatement;
	private CpConnection i_cpConnection;
	private ResultSet i_wrapped;
	private ConnectionLogger i_logger;
	private String i_objId;
	private CpFactory i_factory; // shortcut to attached connection factory.

	// Default constructor, just in case a subclass needs it.
	protected CpResultSet() {
		super();
	}

	/**
	 * @param connection
	 * @param wrapped
	 */
	public CpResultSet(CpStatement stmt, ResultSet wrapped, String objId) {
		super();
		i_cpStatement = stmt;
		i_cpConnection = stmt.cpConnection();
		i_wrapped = wrapped;
		i_logger = stmt.cpConnection().cpLogger();
		i_objId = objId;
		i_factory = stmt.cpConnection().cpFactory();
	}

	public CpResultSet(CpConnection conn, ResultSet wrapped, String objId) {
		super();
		i_cpStatement = null;
		// TT_LOW Maybe the statement could be obtained ad-hoc. Would it be worth it? 
//		i_cpStatement = wrapped.getStatement();
		i_cpConnection = conn;
		i_wrapped = wrapped;
		i_logger = conn.cpLogger();
		i_objId = objId;
		i_factory = conn.cpFactory();
	}

	/**
	 * Return the wrapped clob object.
	 */
	public ResultSet cpWrapped() {
		return i_wrapped;
	}

	public CpStatement cpStatement() {
		return (CpStatement)i_cpStatement;
	}
	
	public CpConnection cpConnection() {
		return (CpConnection)i_cpConnection;
	}

	public String cpObjectId() {
		return i_objId;
	}

	////////////////////////////////////////////////////////////////////////////
	///{AUTOGEN:FRAME_BEGIN}
	//{AUTOGEN:RSST_absoluteI:BEGIN}
	public boolean absolute(int row) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_absoluteI,row);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.absolute(row);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_absoluteI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_absoluteI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_absoluteI:END}
	//{AUTOGEN:RSST_afterLast:BEGIN}
	public void afterLast() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_afterLast);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.afterLast();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_afterLast, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_afterLast, null);
	}
	//{AUTOGEN:RSST_afterLast:END}
	//{AUTOGEN:RSST_beforeFirst:BEGIN}
	public void beforeFirst() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_beforeFirst);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.beforeFirst();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_beforeFirst, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_beforeFirst, null);
	}
	//{AUTOGEN:RSST_beforeFirst:END}
	//{AUTOGEN:RSST_cancelRowUpdates:BEGIN}
	public void cancelRowUpdates() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_cancelRowUpdates);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.cancelRowUpdates();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_cancelRowUpdates, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_cancelRowUpdates, null);
	}
	//{AUTOGEN:RSST_cancelRowUpdates:END}
	//{AUTOGEN:RSST_clearWarnings:BEGIN}
	public void clearWarnings() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_clearWarnings);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.clearWarnings();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_clearWarnings, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_clearWarnings, null);
	}
	//{AUTOGEN:RSST_clearWarnings:END}
	//{AUTOGEN:RSST_close:BEGIN}
	public void close() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_close);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.close();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_close, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_close, null);
	}
	//{AUTOGEN:RSST_close:END}
	//{AUTOGEN:RSST_deleteRow:BEGIN}
	public void deleteRow() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_deleteRow);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.deleteRow();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_deleteRow, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_deleteRow, null);
	}
	//{AUTOGEN:RSST_deleteRow:END}
	//{AUTOGEN:RSST_findColumnS:BEGIN}
	public int findColumn(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_findColumnS,columnName);
		int l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.findColumn(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_findColumnS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_findColumnS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_findColumnS:END}
	//{AUTOGEN:RSST_first:BEGIN}
	public boolean first() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_first);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.first();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_first, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_first, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_first:END}
	//{AUTOGEN:RSST_getArrayI:BEGIN}
	public Array getArray(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getArrayI,columnIndex);
		Array l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getArray(columnIndex);
			l_result = i_factory.wrapArray(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getArrayI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getArrayI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getArrayI:END}
	//{AUTOGEN:RSST_getArrayS:BEGIN}
	public Array getArray(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getArrayS,columnName);
		Array l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getArray(columnName);
			l_result = i_factory.wrapArray(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getArrayS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getArrayS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getArrayS:END}
	//{AUTOGEN:RSST_getAsciiStreamI:BEGIN}
	public InputStream getAsciiStream(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getAsciiStreamI,columnIndex);
		InputStream l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getAsciiStream(columnIndex);
			// TODO Some wrapping to log the retrieved information
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getAsciiStreamI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getAsciiStreamI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getAsciiStreamI:END}
	//{AUTOGEN:RSST_getAsciiStreamS:BEGIN}
	public InputStream getAsciiStream(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getAsciiStreamS,columnName);
		InputStream l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getAsciiStream(columnName);
			// TODO Some wrapping to log the retrieved information
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getAsciiStreamS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getAsciiStreamS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getAsciiStreamS:END}
	//{AUTOGEN:RSST_getBigDecimalII:BEGIN}
	public BigDecimal getBigDecimal(int columnIndex, int scale) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getBigDecimalII,columnIndex, scale);
		BigDecimal l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getBigDecimal(columnIndex, scale);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getBigDecimalII, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getBigDecimalII, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getBigDecimalII:END}
	//{AUTOGEN:RSST_getBigDecimalI:BEGIN}
	public BigDecimal getBigDecimal(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getBigDecimalI,columnIndex);
		BigDecimal l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getBigDecimal(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getBigDecimalI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getBigDecimalI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getBigDecimalI:END}
	//{AUTOGEN:RSST_getBigDecimalSI:BEGIN}
	public BigDecimal getBigDecimal(String columnName, int scale) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getBigDecimalSI,columnName, scale);
		BigDecimal l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getBigDecimal(columnName, scale);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getBigDecimalSI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getBigDecimalSI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getBigDecimalSI:END}
	//{AUTOGEN:RSST_getBigDecimalS:BEGIN}
	public BigDecimal getBigDecimal(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getBigDecimalS,columnName);
		BigDecimal l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getBigDecimal(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getBigDecimalS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getBigDecimalS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getBigDecimalS:END}
	//{AUTOGEN:RSST_getBinaryStreamI:BEGIN}
	public InputStream getBinaryStream(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getBinaryStreamI,columnIndex);
		InputStream l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getBinaryStream(columnIndex);
			// TODO Some wrapping to log the retrieved information
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getBinaryStreamI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getBinaryStreamI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getBinaryStreamI:END}
	//{AUTOGEN:RSST_getBinaryStreamS:BEGIN}
	public InputStream getBinaryStream(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getBinaryStreamS,columnName);
		InputStream l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getBinaryStream(columnName);
			// TODO Some wrapping to log the retrieved information
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getBinaryStreamS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getBinaryStreamS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getBinaryStreamS:END}
	//{AUTOGEN:RSST_getBlobI:BEGIN}
	public Blob getBlob(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getBlobI,columnIndex);
		Blob l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getBlob(columnIndex);
			l_result = i_factory.wrapBlob(l_result, cpConnection());
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getBlobI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getBlobI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getBlobI:END}
	//{AUTOGEN:RSST_getBlobS:BEGIN}
	public Blob getBlob(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getBlobS,columnName);
		Blob l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getBlob(columnName);
			l_result = i_factory.wrapBlob(l_result, cpConnection());
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getBlobS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getBlobS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getBlobS:END}
	//{AUTOGEN:RSST_getBooleanI:BEGIN}
	public boolean getBoolean(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getBooleanI,columnIndex);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getBoolean(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getBooleanI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getBooleanI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getBooleanI:END}
	//{AUTOGEN:RSST_getBooleanS:BEGIN}
	public boolean getBoolean(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getBooleanS,columnName);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getBoolean(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getBooleanS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getBooleanS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getBooleanS:END}
	//{AUTOGEN:RSST_getByteI:BEGIN}
	public byte getByte(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getByteI,columnIndex);
		byte l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getByte(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getByteI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getByteI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getByteI:END}
	//{AUTOGEN:RSST_getByteS:BEGIN}
	public byte getByte(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getByteS,columnName);
		byte l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getByte(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getByteS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getByteS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getByteS:END}
	//{AUTOGEN:RSST_getBytesI:BEGIN}
	public byte[] getBytes(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getBytesI,columnIndex);
		byte[] l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getBytes(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getBytesI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getBytesI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getBytesI:END}
	//{AUTOGEN:RSST_getBytesS:BEGIN}
	public byte[] getBytes(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getBytesS,columnName);
		byte[] l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getBytes(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getBytesS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getBytesS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getBytesS:END}
	//{AUTOGEN:RSST_getCharacterStreamI:BEGIN}
	public Reader getCharacterStream(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getCharacterStreamI,columnIndex);
		Reader l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getCharacterStream(columnIndex);
			// TODO Some wrapping to log the retrieved information
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getCharacterStreamI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getCharacterStreamI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getCharacterStreamI:END}
	//{AUTOGEN:RSST_getCharacterStreamS:BEGIN}
	public Reader getCharacterStream(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getCharacterStreamS,columnName);
		Reader l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getCharacterStream(columnName);
			// TODO Some wrapping to log the retrieved information
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getCharacterStreamS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getCharacterStreamS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getCharacterStreamS:END}
	//{AUTOGEN:RSST_getClobI:BEGIN}
	public Clob getClob(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getClobI,columnIndex);
		Clob l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getClob(columnIndex);
			l_result = i_factory.wrapClob(l_result, cpConnection());
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getClobI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getClobI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getClobI:END}
	//{AUTOGEN:RSST_getClobS:BEGIN}
	public Clob getClob(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getClobS,columnName);
		Clob l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getClob(columnName);
			l_result = i_factory.wrapClob(l_result, cpConnection());
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getClobS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getClobS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getClobS:END}
	//{AUTOGEN:RSST_getConcurrency:BEGIN}
	public int getConcurrency() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getConcurrency);
		int l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getConcurrency();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getConcurrency, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getConcurrency, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getConcurrency:END}
	//{AUTOGEN:RSST_getCursorName:BEGIN}
	public String getCursorName() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getCursorName);
		String l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getCursorName();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getCursorName, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getCursorName, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getCursorName:END}
	//{AUTOGEN:RSST_getDateICa:BEGIN}
	public Date getDate(int columnIndex, Calendar cal) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getDateICa,columnIndex, cal);
		Date l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getDate(columnIndex, cal);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getDateICa, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getDateICa, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getDateICa:END}
	//{AUTOGEN:RSST_getDateI:BEGIN}
	public Date getDate(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getDateI,columnIndex);
		Date l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getDate(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getDateI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getDateI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getDateI:END}
	//{AUTOGEN:RSST_getDateSCa:BEGIN}
	public Date getDate(String columnName, Calendar cal) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getDateSCa,columnName, cal);
		Date l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getDate(columnName, cal);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getDateSCa, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getDateSCa, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getDateSCa:END}
	//{AUTOGEN:RSST_getDateS:BEGIN}
	public Date getDate(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getDateS,columnName);
		Date l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getDate(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getDateS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getDateS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getDateS:END}
	//{AUTOGEN:RSST_getDoubleI:BEGIN}
	public double getDouble(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getDoubleI,columnIndex);
		double l_result = 0.0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getDouble(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getDoubleI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getDoubleI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getDoubleI:END}
	//{AUTOGEN:RSST_getDoubleS:BEGIN}
	public double getDouble(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getDoubleS,columnName);
		double l_result = 0.0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getDouble(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getDoubleS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getDoubleS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getDoubleS:END}
	//{AUTOGEN:RSST_getFetchDirection:BEGIN}
	public int getFetchDirection() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getFetchDirection);
		int l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getFetchDirection();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getFetchDirection, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getFetchDirection, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getFetchDirection:END}
	//{AUTOGEN:RSST_getFetchSize:BEGIN}
	public int getFetchSize() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getFetchSize);
		int l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getFetchSize();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getFetchSize, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getFetchSize, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getFetchSize:END}
	//{AUTOGEN:RSST_getFloatI:BEGIN}
	public float getFloat(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getFloatI,columnIndex);
		float l_result = 0.0f;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getFloat(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getFloatI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getFloatI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getFloatI:END}
	//{AUTOGEN:RSST_getFloatS:BEGIN}
	public float getFloat(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getFloatS,columnName);
		float l_result = 0.0f;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getFloat(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getFloatS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getFloatS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getFloatS:END}
	//{AUTOGEN:RSST_getHoldability:BEGIN}
	public int getHoldability() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getHoldability);
		int l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getHoldability();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getHoldability, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getHoldability, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getHoldability:END}
	//{AUTOGEN:RSST_getIntI:BEGIN}
	public int getInt(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getIntI,columnIndex);
		int l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getInt(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getIntI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getIntI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getIntI:END}
	//{AUTOGEN:RSST_getIntS:BEGIN}
	public int getInt(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getIntS,columnName);
		int l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getInt(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getIntS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getIntS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getIntS:END}
	//{AUTOGEN:RSST_getLongI:BEGIN}
	public long getLong(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getLongI,columnIndex);
		long l_result = 0L;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getLong(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getLongI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getLongI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getLongI:END}
	//{AUTOGEN:RSST_getLongS:BEGIN}
	public long getLong(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getLongS,columnName);
		long l_result = 0L;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getLong(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getLongS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getLongS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getLongS:END}
	//{AUTOGEN:RSST_getMetaData:BEGIN}
	public ResultSetMetaData getMetaData() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getMetaData);
		ResultSetMetaData l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getMetaData();
			l_result = cpConnection().cpFactory().wrapResultSetMetaData(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getMetaData, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getMetaData, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getMetaData:END}
	//{AUTOGEN:RSST_getNCharacterStreamI:BEGIN}
	public Reader getNCharacterStream(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getNCharacterStreamI,columnIndex);
		Reader l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getNCharacterStream(columnIndex);
			// TODO Some wrapping to log the retrieved information
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getNCharacterStreamI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getNCharacterStreamI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getNCharacterStreamI:END}
	//{AUTOGEN:RSST_getNCharacterStreamS:BEGIN}
	public Reader getNCharacterStream(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getNCharacterStreamS,columnName);
		Reader l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getNCharacterStream(columnName);
			// TODO Some wrapping to log the retrieved information
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getNCharacterStreamS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getNCharacterStreamS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getNCharacterStreamS:END}
	//{AUTOGEN:RSST_getNClobI:BEGIN}
	public NClob getNClob(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getNClobI,columnIndex);
		NClob l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getNClob(columnIndex);
			l_result = i_factory.wrapNClob(l_result, cpConnection());
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getNClobI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getNClobI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getNClobI:END}
	//{AUTOGEN:RSST_getNClobS:BEGIN}
	public NClob getNClob(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getNClobS,columnName);
		NClob l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getNClob(columnName);
			l_result = i_factory.wrapNClob(l_result, cpConnection());
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getNClobS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getNClobS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getNClobS:END}
	//{AUTOGEN:RSST_getNStringI:BEGIN}
	public String getNString(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getNStringI,columnIndex);
		String l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getNString(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getNStringI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getNStringI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getNStringI:END}
	//{AUTOGEN:RSST_getNStringS:BEGIN}
	public String getNString(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getNStringS,columnName);
		String l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getNString(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getNStringS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getNStringS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getNStringS:END}
	//{AUTOGEN:RSST_getObjectIMp:BEGIN}
	public Object getObject(int columnIndex, Map<String, Class<?>> map) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getObjectIMp,columnIndex, map);
		Object l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getObject(columnIndex, map);
			// TT_LOW type sensitive wrapper?
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getObjectIMp, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getObjectIMp, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getObjectIMp:END}
	//{AUTOGEN:RSST_getObjectI:BEGIN}
	public Object getObject(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getObjectI,columnIndex);
		Object l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getObject(columnIndex);
			// TT_LOW type sensitive wrapper?
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getObjectI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getObjectI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getObjectI:END}
	//{AUTOGEN:RSST_getObjectSMp:BEGIN}
	public Object getObject(String columnName, Map<String, Class<?>> map) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getObjectSMp,columnName, map);
		Object l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getObject(columnName, map);
			// TT_LOW type sensitive wrapper?
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getObjectSMp, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getObjectSMp, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getObjectSMp:END}
	//{AUTOGEN:RSST_getObjectS:BEGIN}
	public Object getObject(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getObjectS,columnName);
		Object l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getObject(columnName);
			// TT_LOW type sensitive wrapper?
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getObjectS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getObjectS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getObjectS:END}
	//{AUTOGEN:RSST_getRefI:BEGIN}
	public Ref getRef(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getRefI,columnIndex);
		Ref l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getRef(columnIndex);
			l_result = i_factory.wrapRef(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getRefI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getRefI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getRefI:END}
	//{AUTOGEN:RSST_getRefS:BEGIN}
	public Ref getRef(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getRefS,columnName);
		Ref l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getRef(columnName);
			l_result = i_factory.wrapRef(l_result, this);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getRefS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getRefS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getRefS:END}
	//{AUTOGEN:RSST_getRow:BEGIN}
	public int getRow() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getRow);
		int l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getRow();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getRow, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getRow, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getRow:END}
	//{AUTOGEN:RSST_getRowIdI:BEGIN}
	public RowId getRowId(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getRowIdI,columnIndex);
		RowId l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getRowId(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getRowIdI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getRowIdI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getRowIdI:END}
	//{AUTOGEN:RSST_getRowIdS:BEGIN}
	public RowId getRowId(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getRowIdS,columnName);
		RowId l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getRowId(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getRowIdS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getRowIdS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getRowIdS:END}
	//{AUTOGEN:RSST_getShortI:BEGIN}
	public short getShort(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getShortI,columnIndex);
		short l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getShort(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getShortI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getShortI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getShortI:END}
	//{AUTOGEN:RSST_getShortS:BEGIN}
	public short getShort(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getShortS,columnName);
		short l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getShort(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getShortS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getShortS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getShortS:END}
	//{AUTOGEN:RSST_getSQLXMLI:BEGIN}
	public SQLXML getSQLXML(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getSQLXMLI,columnIndex);
		SQLXML l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getSQLXML(columnIndex);
			l_result = i_factory.wrapSqlXml(l_result, cpConnection());
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getSQLXMLI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getSQLXMLI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getSQLXMLI:END}
	//{AUTOGEN:RSST_getSQLXMLS:BEGIN}
	public SQLXML getSQLXML(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getSQLXMLS,columnName);
		SQLXML l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getSQLXML(columnName);
			l_result = i_factory.wrapSqlXml(l_result, cpConnection());
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getSQLXMLS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getSQLXMLS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getSQLXMLS:END}
	//{AUTOGEN:RSST_getStatement:BEGIN}
	public Statement getStatement() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getStatement);
		Statement l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			//l_result = i_wrapped.getStatement();
			// TT_LOW What if the resultset was returned by DatabaseMetaData?
			//   Will it return null? What if it is an statement querying on meta-data views? 
			l_result = i_cpStatement;
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getStatement, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getStatement, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getStatement:END}
	//{AUTOGEN:RSST_getStringI:BEGIN}
	public String getString(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getStringI,columnIndex);
		String l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getString(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getStringI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getStringI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getStringI:END}
	//{AUTOGEN:RSST_getStringS:BEGIN}
	public String getString(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getStringS,columnName);
		String l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getString(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getStringS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getStringS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getStringS:END}
	//{AUTOGEN:RSST_getTimeICa:BEGIN}
	public Time getTime(int columnIndex, Calendar cal) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getTimeICa,columnIndex, cal);
		Time l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getTime(columnIndex, cal);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getTimeICa, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getTimeICa, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getTimeICa:END}
	//{AUTOGEN:RSST_getTimeI:BEGIN}
	public Time getTime(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getTimeI,columnIndex);
		Time l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getTime(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getTimeI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getTimeI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getTimeI:END}
	//{AUTOGEN:RSST_getTimeSCa:BEGIN}
	public Time getTime(String columnName, Calendar cal) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getTimeSCa,columnName, cal);
		Time l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getTime(columnName, cal);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getTimeSCa, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getTimeSCa, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getTimeSCa:END}
	//{AUTOGEN:RSST_getTimeS:BEGIN}
	public Time getTime(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getTimeS,columnName);
		Time l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getTime(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getTimeS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getTimeS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getTimeS:END}
	//{AUTOGEN:RSST_getTimestampICa:BEGIN}
	public Timestamp getTimestamp(int columnIndex, Calendar cal) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getTimestampICa,columnIndex, cal);
		Timestamp l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getTimestamp(columnIndex, cal);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getTimestampICa, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getTimestampICa, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getTimestampICa:END}
	//{AUTOGEN:RSST_getTimestampI:BEGIN}
	public Timestamp getTimestamp(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getTimestampI,columnIndex);
		Timestamp l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getTimestamp(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getTimestampI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getTimestampI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getTimestampI:END}
	//{AUTOGEN:RSST_getTimestampSCa:BEGIN}
	public Timestamp getTimestamp(String columnName, Calendar cal) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getTimestampSCa,columnName, cal);
		Timestamp l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getTimestamp(columnName, cal);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getTimestampSCa, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getTimestampSCa, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getTimestampSCa:END}
	//{AUTOGEN:RSST_getTimestampS:BEGIN}
	public Timestamp getTimestamp(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getTimestampS,columnName);
		Timestamp l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getTimestamp(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getTimestampS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getTimestampS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getTimestampS:END}
	//{AUTOGEN:RSST_getType:BEGIN}
	public int getType() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getType);
		int l_result = 0;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getType();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getType, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getType, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getType:END}
	//{AUTOGEN:RSST_getUnicodeStreamI:BEGIN}
	public InputStream getUnicodeStream(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getUnicodeStreamI,columnIndex);
		InputStream l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getUnicodeStream(columnIndex);
			// TODO Some wrapping to log the retrieved information
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getUnicodeStreamI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getUnicodeStreamI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getUnicodeStreamI:END}
	//{AUTOGEN:RSST_getUnicodeStreamS:BEGIN}
	public InputStream getUnicodeStream(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getUnicodeStreamS,columnName);
		InputStream l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getUnicodeStream(columnName);
			// TODO Some wrapping to log the retrieved information
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getUnicodeStreamS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getUnicodeStreamS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getUnicodeStreamS:END}
	//{AUTOGEN:RSST_getURLI:BEGIN}
	public URL getURL(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getURLI,columnIndex);
		URL l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getURL(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getURLI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getURLI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getURLI:END}
	//{AUTOGEN:RSST_getURLS:BEGIN}
	public URL getURL(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getURLS,columnName);
		URL l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getURL(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getURLS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getURLS, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getURLS:END}
	//{AUTOGEN:RSST_getWarnings:BEGIN}
	public SQLWarning getWarnings() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_getWarnings);
		SQLWarning l_result = null;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.getWarnings();
			// TODO Wrap warnings
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_getWarnings, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_getWarnings, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_getWarnings:END}
	//{AUTOGEN:RSST_insertRow:BEGIN}
	public void insertRow() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_insertRow);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.insertRow();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_insertRow, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_insertRow, null);
	}
	//{AUTOGEN:RSST_insertRow:END}
	//{AUTOGEN:RSST_isAfterLast:BEGIN}
	public boolean isAfterLast() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_isAfterLast);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.isAfterLast();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_isAfterLast, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_isAfterLast, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_isAfterLast:END}
	//{AUTOGEN:RSST_isBeforeFirst:BEGIN}
	public boolean isBeforeFirst() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_isBeforeFirst);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.isBeforeFirst();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_isBeforeFirst, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_isBeforeFirst, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_isBeforeFirst:END}
	//{AUTOGEN:RSST_isClosed:BEGIN}
	public boolean isClosed() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_isClosed);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.isClosed();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_isClosed, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_isClosed, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_isClosed:END}
	//{AUTOGEN:RSST_isFirst:BEGIN}
	public boolean isFirst() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_isFirst);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.isFirst();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_isFirst, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_isFirst, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_isFirst:END}
	//{AUTOGEN:RSST_isLast:BEGIN}
	public boolean isLast() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_isLast);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.isLast();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_isLast, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_isLast, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_isLast:END}
	//{AUTOGEN:RSST_last:BEGIN}
	public boolean last() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_last);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.last();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_last, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_last, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_last:END}
	//{AUTOGEN:RSST_moveToCurrentRow:BEGIN}
	public void moveToCurrentRow() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_moveToCurrentRow);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.moveToCurrentRow();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_moveToCurrentRow, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_moveToCurrentRow, null);
	}
	//{AUTOGEN:RSST_moveToCurrentRow:END}
	//{AUTOGEN:RSST_moveToInsertRow:BEGIN}
	public void moveToInsertRow() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_moveToInsertRow);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.moveToInsertRow();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_moveToInsertRow, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_moveToInsertRow, null);
	}
	//{AUTOGEN:RSST_moveToInsertRow:END}
	//{AUTOGEN:RSST_next:BEGIN}
	public boolean next() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_next);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.next();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_next, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_next, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_next:END}
	//{AUTOGEN:RSST_previous:BEGIN}
	public boolean previous() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_previous);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.previous();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_previous, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_previous, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_previous:END}
	//{AUTOGEN:RSST_refreshRow:BEGIN}
	public void refreshRow() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_refreshRow);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.refreshRow();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_refreshRow, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_refreshRow, null);
	}
	//{AUTOGEN:RSST_refreshRow:END}
	//{AUTOGEN:RSST_relativeI:BEGIN}
	public boolean relative(int rows) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_relativeI,rows);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.relative(rows);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_relativeI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_relativeI, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_relativeI:END}
	//{AUTOGEN:RSST_rowDeleted:BEGIN}
	public boolean rowDeleted() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_rowDeleted);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.rowDeleted();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_rowDeleted, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_rowDeleted, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_rowDeleted:END}
	//{AUTOGEN:RSST_rowInserted:BEGIN}
	public boolean rowInserted() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_rowInserted);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.rowInserted();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_rowInserted, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_rowInserted, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_rowInserted:END}
	//{AUTOGEN:RSST_rowUpdated:BEGIN}
	public boolean rowUpdated() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_rowUpdated);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.rowUpdated();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_rowUpdated, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_rowUpdated, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_rowUpdated:END}
	//{AUTOGEN:RSST_setFetchDirectionI:BEGIN}
	public void setFetchDirection(int direction) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_setFetchDirectionI,direction);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.setFetchDirection(direction);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_setFetchDirectionI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_setFetchDirectionI, null);
	}
	//{AUTOGEN:RSST_setFetchDirectionI:END}
	//{AUTOGEN:RSST_setFetchSizeI:BEGIN}
	public void setFetchSize(int rows) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_setFetchSizeI,rows);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.setFetchSize(rows);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_setFetchSizeI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_setFetchSizeI, null);
	}
	//{AUTOGEN:RSST_setFetchSizeI:END}
	//{AUTOGEN:RSST_updateArrayIAr:BEGIN}
	public void updateArray(int columnIndex, Array x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateArrayIAr,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW Is it out wrapper?
			i_wrapped.updateArray(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateArrayIAr, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateArrayIAr, null);
	}
	//{AUTOGEN:RSST_updateArrayIAr:END}
	//{AUTOGEN:RSST_updateArraySAr:BEGIN}
	public void updateArray(String columnName, Array x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateArraySAr,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW Is it out wrapper?
			i_wrapped.updateArray(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateArraySAr, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateArraySAr, null);
	}
	//{AUTOGEN:RSST_updateArraySAr:END}
	//{AUTOGEN:RSST_updateAsciiStreamIIsI:BEGIN}
	public void updateAsciiStream(int columnIndex, InputStream x, int length) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateAsciiStreamIIsI,columnIndex, x, length);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateAsciiStream(columnIndex, x, length);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateAsciiStreamIIsI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateAsciiStreamIIsI, null);
	}
	//{AUTOGEN:RSST_updateAsciiStreamIIsI:END}
	//{AUTOGEN:RSST_updateAsciiStreamIIsL:BEGIN}
	public void updateAsciiStream(int arg0, InputStream arg1, long arg2) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateAsciiStreamIIsL,arg0, arg1, arg2);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateAsciiStream(arg0, arg1, arg2);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateAsciiStreamIIsL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateAsciiStreamIIsL, null);
	}
	//{AUTOGEN:RSST_updateAsciiStreamIIsL:END}
	//{AUTOGEN:RSST_updateAsciiStreamIIs:BEGIN}
	public void updateAsciiStream(int arg0, InputStream arg1) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateAsciiStreamIIs,arg0, arg1);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateAsciiStream(arg0, arg1);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateAsciiStreamIIs, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateAsciiStreamIIs, null);
	}
	//{AUTOGEN:RSST_updateAsciiStreamIIs:END}
	//{AUTOGEN:RSST_updateAsciiStreamSIsI:BEGIN}
	public void updateAsciiStream(String columnName, InputStream x, int length) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateAsciiStreamSIsI,columnName, x, length);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateAsciiStream(columnName, x, length);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateAsciiStreamSIsI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateAsciiStreamSIsI, null);
	}
	//{AUTOGEN:RSST_updateAsciiStreamSIsI:END}
	//{AUTOGEN:RSST_updateAsciiStreamSIsL:BEGIN}
	public void updateAsciiStream(String arg0, InputStream arg1, long arg2) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateAsciiStreamSIsL,arg0, arg1, arg2);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateAsciiStream(arg0, arg1, arg2);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateAsciiStreamSIsL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateAsciiStreamSIsL, null);
	}
	//{AUTOGEN:RSST_updateAsciiStreamSIsL:END}
	//{AUTOGEN:RSST_updateAsciiStreamSIs:BEGIN}
	public void updateAsciiStream(String arg0, InputStream arg1) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateAsciiStreamSIs,arg0, arg1);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateAsciiStream(arg0, arg1);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateAsciiStreamSIs, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateAsciiStreamSIs, null);
	}
	//{AUTOGEN:RSST_updateAsciiStreamSIs:END}
	//{AUTOGEN:RSST_updateBigDecimalIBd:BEGIN}
	public void updateBigDecimal(int columnIndex, BigDecimal x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBigDecimalIBd,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateBigDecimal(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBigDecimalIBd, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBigDecimalIBd, null);
	}
	//{AUTOGEN:RSST_updateBigDecimalIBd:END}
	//{AUTOGEN:RSST_updateBigDecimalSBd:BEGIN}
	public void updateBigDecimal(String columnName, BigDecimal x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBigDecimalSBd,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateBigDecimal(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBigDecimalSBd, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBigDecimalSBd, null);
	}
	//{AUTOGEN:RSST_updateBigDecimalSBd:END}
	//{AUTOGEN:RSST_updateBinaryStreamIIsI:BEGIN}
	public void updateBinaryStream(int columnIndex, InputStream x, int length) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBinaryStreamIIsI,columnIndex, x, length);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateBinaryStream(columnIndex, x, length);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBinaryStreamIIsI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBinaryStreamIIsI, null);
	}
	//{AUTOGEN:RSST_updateBinaryStreamIIsI:END}
	//{AUTOGEN:RSST_updateBinaryStreamIIsL:BEGIN}
	public void updateBinaryStream(int arg0, InputStream arg1, long arg2) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBinaryStreamIIsL,arg0, arg1, arg2);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateBinaryStream(arg0, arg1, arg2);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBinaryStreamIIsL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBinaryStreamIIsL, null);
	}
	//{AUTOGEN:RSST_updateBinaryStreamIIsL:END}
	//{AUTOGEN:RSST_updateBinaryStreamIIs:BEGIN}
	public void updateBinaryStream(int arg0, InputStream arg1) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBinaryStreamIIs,arg0, arg1);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateBinaryStream(arg0, arg1);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBinaryStreamIIs, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBinaryStreamIIs, null);
	}
	//{AUTOGEN:RSST_updateBinaryStreamIIs:END}
	//{AUTOGEN:RSST_updateBinaryStreamSIsI:BEGIN}
	public void updateBinaryStream(String columnName, InputStream x, int length) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBinaryStreamSIsI,columnName, x, length);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateBinaryStream(columnName, x, length);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBinaryStreamSIsI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBinaryStreamSIsI, null);
	}
	//{AUTOGEN:RSST_updateBinaryStreamSIsI:END}
	//{AUTOGEN:RSST_updateBinaryStreamSIsL:BEGIN}
	public void updateBinaryStream(String arg0, InputStream arg1, long arg2) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBinaryStreamSIsL,arg0, arg1, arg2);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateBinaryStream(arg0, arg1, arg2);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBinaryStreamSIsL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBinaryStreamSIsL, null);
	}
	//{AUTOGEN:RSST_updateBinaryStreamSIsL:END}
	//{AUTOGEN:RSST_updateBinaryStreamSIs:BEGIN}
	public void updateBinaryStream(String arg0, InputStream arg1) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBinaryStreamSIs,arg0, arg1);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateBinaryStream(arg0, arg1);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBinaryStreamSIs, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBinaryStreamSIs, null);
	}
	//{AUTOGEN:RSST_updateBinaryStreamSIs:END}
	//{AUTOGEN:RSST_updateBlobIBl:BEGIN}
	public void updateBlob(int columnIndex, Blob x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBlobIBl,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW Is it our wrapper?
			i_wrapped.updateBlob(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBlobIBl, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBlobIBl, null);
	}
	//{AUTOGEN:RSST_updateBlobIBl:END}
	//{AUTOGEN:RSST_updateBlobIIsL:BEGIN}
	public void updateBlob(int arg0, InputStream arg1, long arg2) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBlobIIsL,arg0, arg1, arg2);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW Is it our wrapper?
			i_wrapped.updateBlob(arg0, arg1, arg2);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBlobIIsL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBlobIIsL, null);
	}
	//{AUTOGEN:RSST_updateBlobIIsL:END}
	//{AUTOGEN:RSST_updateBlobIIs:BEGIN}
	public void updateBlob(int arg0, InputStream arg1) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBlobIIs,arg0, arg1);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateBlob(arg0, arg1);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBlobIIs, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBlobIIs, null);
	}
	//{AUTOGEN:RSST_updateBlobIIs:END}
	//{AUTOGEN:RSST_updateBlobSBl:BEGIN}
	public void updateBlob(String columnName, Blob x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBlobSBl,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW Is it our wrapper?
			i_wrapped.updateBlob(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBlobSBl, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBlobSBl, null);
	}
	//{AUTOGEN:RSST_updateBlobSBl:END}
	//{AUTOGEN:RSST_updateBlobSIsL:BEGIN}
	public void updateBlob(String arg0, InputStream arg1, long arg2) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBlobSIsL,arg0, arg1, arg2);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateBlob(arg0, arg1, arg2);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBlobSIsL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBlobSIsL, null);
	}
	//{AUTOGEN:RSST_updateBlobSIsL:END}
	//{AUTOGEN:RSST_updateBlobSIs:BEGIN}
	public void updateBlob(String arg0, InputStream arg1) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBlobSIs,arg0, arg1);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateBlob(arg0, arg1);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBlobSIs, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBlobSIs, null);
	}
	//{AUTOGEN:RSST_updateBlobSIs:END}
	//{AUTOGEN:RSST_updateBooleanIB:BEGIN}
	public void updateBoolean(int columnIndex, boolean x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBooleanIB,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateBoolean(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBooleanIB, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBooleanIB, null);
	}
	//{AUTOGEN:RSST_updateBooleanIB:END}
	//{AUTOGEN:RSST_updateBooleanSB:BEGIN}
	public void updateBoolean(String columnName, boolean x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBooleanSB,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateBoolean(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBooleanSB, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBooleanSB, null);
	}
	//{AUTOGEN:RSST_updateBooleanSB:END}
	//{AUTOGEN:RSST_updateByteIBy:BEGIN}
	public void updateByte(int columnIndex, byte x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateByteIBy,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateByte(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateByteIBy, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateByteIBy, null);
	}
	//{AUTOGEN:RSST_updateByteIBy:END}
	//{AUTOGEN:RSST_updateByteSBy:BEGIN}
	public void updateByte(String columnName, byte x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateByteSBy,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateByte(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateByteSBy, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateByteSBy, null);
	}
	//{AUTOGEN:RSST_updateByteSBy:END}
	//{AUTOGEN:RSST_updateBytesIBya:BEGIN}
	public void updateBytes(int columnIndex, byte[] x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBytesIBya,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateBytes(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBytesIBya, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBytesIBya, null);
	}
	//{AUTOGEN:RSST_updateBytesIBya:END}
	//{AUTOGEN:RSST_updateBytesSBya:BEGIN}
	public void updateBytes(String columnName, byte[] x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateBytesSBya,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateBytes(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateBytesSBya, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateBytesSBya, null);
	}
	//{AUTOGEN:RSST_updateBytesSBya:END}
	//{AUTOGEN:RSST_updateCharacterStreamIRdI:BEGIN}
	public void updateCharacterStream(int columnIndex, Reader x, int length) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateCharacterStreamIRdI,columnIndex, x, length);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateCharacterStream(columnIndex, x, length);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateCharacterStreamIRdI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateCharacterStreamIRdI, null);
	}
	//{AUTOGEN:RSST_updateCharacterStreamIRdI:END}
	//{AUTOGEN:RSST_updateCharacterStreamIRdL:BEGIN}
	public void updateCharacterStream(int arg0, Reader arg1, long arg2) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateCharacterStreamIRdL,arg0, arg1, arg2);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateCharacterStream(arg0, arg1, arg2);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateCharacterStreamIRdL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateCharacterStreamIRdL, null);
	}
	//{AUTOGEN:RSST_updateCharacterStreamIRdL:END}
	//{AUTOGEN:RSST_updateCharacterStreamIRd:BEGIN}
	public void updateCharacterStream(int arg0, Reader arg1) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateCharacterStreamIRd,arg0, arg1);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateCharacterStream(arg0, arg1);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateCharacterStreamIRd, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateCharacterStreamIRd, null);
	}
	//{AUTOGEN:RSST_updateCharacterStreamIRd:END}
	//{AUTOGEN:RSST_updateCharacterStreamSRdI:BEGIN}
	public void updateCharacterStream(String columnName, Reader reader, int length) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateCharacterStreamSRdI,columnName, reader, length);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateCharacterStream(columnName, reader, length);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateCharacterStreamSRdI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateCharacterStreamSRdI, null);
	}
	//{AUTOGEN:RSST_updateCharacterStreamSRdI:END}
	//{AUTOGEN:RSST_updateCharacterStreamSRdL:BEGIN}
	public void updateCharacterStream(String arg0, Reader arg1, long arg2) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateCharacterStreamSRdL,arg0, arg1, arg2);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateCharacterStream(arg0, arg1, arg2);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateCharacterStreamSRdL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateCharacterStreamSRdL, null);
	}
	//{AUTOGEN:RSST_updateCharacterStreamSRdL:END}
	//{AUTOGEN:RSST_updateCharacterStreamSRd:BEGIN}
	public void updateCharacterStream(String arg0, Reader arg1) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateCharacterStreamSRd,arg0, arg1);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateCharacterStream(arg0, arg1);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateCharacterStreamSRd, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateCharacterStreamSRd, null);
	}
	//{AUTOGEN:RSST_updateCharacterStreamSRd:END}
	//{AUTOGEN:RSST_updateClobICl:BEGIN}
	public void updateClob(int columnIndex, Clob x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateClobICl,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW Is it our wrapper?
			i_wrapped.updateClob(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateClobICl, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateClobICl, null);
	}
	//{AUTOGEN:RSST_updateClobICl:END}
	//{AUTOGEN:RSST_updateClobIRdL:BEGIN}
	public void updateClob(int arg0, Reader arg1, long arg2) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateClobIRdL,arg0, arg1, arg2);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateClob(arg0, arg1, arg2);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateClobIRdL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateClobIRdL, null);
	}
	//{AUTOGEN:RSST_updateClobIRdL:END}
	//{AUTOGEN:RSST_updateClobIRd:BEGIN}
	public void updateClob(int arg0, Reader arg1) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateClobIRd,arg0, arg1);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateClob(arg0, arg1);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateClobIRd, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateClobIRd, null);
	}
	//{AUTOGEN:RSST_updateClobIRd:END}
	//{AUTOGEN:RSST_updateClobSCl:BEGIN}
	public void updateClob(String columnName, Clob x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateClobSCl,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW Is it out wrapper?
			i_wrapped.updateClob(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateClobSCl, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateClobSCl, null);
	}
	//{AUTOGEN:RSST_updateClobSCl:END}
	//{AUTOGEN:RSST_updateClobSRdL:BEGIN}
	public void updateClob(String arg0, Reader arg1, long arg2) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateClobSRdL,arg0, arg1, arg2);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateClob(arg0, arg1, arg2);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateClobSRdL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateClobSRdL, null);
	}
	//{AUTOGEN:RSST_updateClobSRdL:END}
	//{AUTOGEN:RSST_updateClobSRd:BEGIN}
	public void updateClob(String arg0, Reader arg1) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateClobSRd,arg0, arg1);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateClob(arg0, arg1);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateClobSRd, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateClobSRd, null);
	}
	//{AUTOGEN:RSST_updateClobSRd:END}
	//{AUTOGEN:RSST_updateDateIDa:BEGIN}
	public void updateDate(int columnIndex, Date x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateDateIDa,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateDate(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateDateIDa, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateDateIDa, null);
	}
	//{AUTOGEN:RSST_updateDateIDa:END}
	//{AUTOGEN:RSST_updateDateSDa:BEGIN}
	public void updateDate(String columnName, Date x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateDateSDa,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateDate(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateDateSDa, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateDateSDa, null);
	}
	//{AUTOGEN:RSST_updateDateSDa:END}
	//{AUTOGEN:RSST_updateDoubleID:BEGIN}
	public void updateDouble(int columnIndex, double x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateDoubleID,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateDouble(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateDoubleID, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateDoubleID, null);
	}
	//{AUTOGEN:RSST_updateDoubleID:END}
	//{AUTOGEN:RSST_updateDoubleSD:BEGIN}
	public void updateDouble(String columnName, double x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateDoubleSD,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateDouble(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateDoubleSD, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateDoubleSD, null);
	}
	//{AUTOGEN:RSST_updateDoubleSD:END}
	//{AUTOGEN:RSST_updateFloatIF:BEGIN}
	public void updateFloat(int columnIndex, float x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateFloatIF,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateFloat(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateFloatIF, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateFloatIF, null);
	}
	//{AUTOGEN:RSST_updateFloatIF:END}
	//{AUTOGEN:RSST_updateFloatSF:BEGIN}
	public void updateFloat(String columnName, float x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateFloatSF,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateFloat(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateFloatSF, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateFloatSF, null);
	}
	//{AUTOGEN:RSST_updateFloatSF:END}
	//{AUTOGEN:RSST_updateIntII:BEGIN}
	public void updateInt(int columnIndex, int x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateIntII,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateInt(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateIntII, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateIntII, null);
	}
	//{AUTOGEN:RSST_updateIntII:END}
	//{AUTOGEN:RSST_updateIntSI:BEGIN}
	public void updateInt(String columnName, int x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateIntSI,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateInt(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateIntSI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateIntSI, null);
	}
	//{AUTOGEN:RSST_updateIntSI:END}
	//{AUTOGEN:RSST_updateLongIL:BEGIN}
	public void updateLong(int columnIndex, long x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateLongIL,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateLong(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateLongIL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateLongIL, null);
	}
	//{AUTOGEN:RSST_updateLongIL:END}
	//{AUTOGEN:RSST_updateLongSL:BEGIN}
	public void updateLong(String columnName, long x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateLongSL,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateLong(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateLongSL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateLongSL, null);
	}
	//{AUTOGEN:RSST_updateLongSL:END}
	//{AUTOGEN:RSST_updateNCharacterStreamIRdL:BEGIN}
	public void updateNCharacterStream(int arg0, Reader arg1, long arg2) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateNCharacterStreamIRdL,arg0, arg1, arg2);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateNCharacterStream(arg0, arg1, arg2);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateNCharacterStreamIRdL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateNCharacterStreamIRdL, null);
	}
	//{AUTOGEN:RSST_updateNCharacterStreamIRdL:END}
	//{AUTOGEN:RSST_updateNCharacterStreamIRd:BEGIN}
	public void updateNCharacterStream(int arg0, Reader arg1) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateNCharacterStreamIRd,arg0, arg1);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateNCharacterStream(arg0, arg1);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateNCharacterStreamIRd, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateNCharacterStreamIRd, null);
	}
	//{AUTOGEN:RSST_updateNCharacterStreamIRd:END}
	//{AUTOGEN:RSST_updateNCharacterStreamSRdL:BEGIN}
	public void updateNCharacterStream(String arg0, Reader arg1, long arg2) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateNCharacterStreamSRdL,arg0, arg1, arg2);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateNCharacterStream(arg0, arg1, arg2);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateNCharacterStreamSRdL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateNCharacterStreamSRdL, null);
	}
	//{AUTOGEN:RSST_updateNCharacterStreamSRdL:END}
	//{AUTOGEN:RSST_updateNCharacterStreamSRd:BEGIN}
	public void updateNCharacterStream(String arg0, Reader arg1) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateNCharacterStreamSRd,arg0, arg1);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateNCharacterStream(arg0, arg1);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateNCharacterStreamSRd, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateNCharacterStreamSRd, null);
	}
	//{AUTOGEN:RSST_updateNCharacterStreamSRd:END}
	//{AUTOGEN:RSST_updateNClobINc:BEGIN}
	public void updateNClob(int columnIndex, NClob clob) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateNClobINc,columnIndex, clob);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW Is it our wrapper?
			i_wrapped.updateNClob(columnIndex, clob);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateNClobINc, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateNClobINc, null);
	}
	//{AUTOGEN:RSST_updateNClobINc:END}
	//{AUTOGEN:RSST_updateNClobIRdL:BEGIN}
	public void updateNClob(int arg0, Reader arg1, long arg2) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateNClobIRdL,arg0, arg1, arg2);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateNClob(arg0, arg1, arg2);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateNClobIRdL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateNClobIRdL, null);
	}
	//{AUTOGEN:RSST_updateNClobIRdL:END}
	//{AUTOGEN:RSST_updateNClobIRd:BEGIN}
	public void updateNClob(int arg0, Reader arg1) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateNClobIRd,arg0, arg1);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateNClob(arg0, arg1);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateNClobIRd, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateNClobIRd, null);
	}
	//{AUTOGEN:RSST_updateNClobIRd:END}
	//{AUTOGEN:RSST_updateNClobSNc:BEGIN}
	public void updateNClob(String columnName, NClob clob) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateNClobSNc,columnName, clob);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW Is it out wrapper?
			i_wrapped.updateNClob(columnName, clob);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateNClobSNc, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateNClobSNc, null);
	}
	//{AUTOGEN:RSST_updateNClobSNc:END}
	//{AUTOGEN:RSST_updateNClobSRdL:BEGIN}
	public void updateNClob(String arg0, Reader arg1, long arg2) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateNClobSRdL,arg0, arg1, arg2);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateNClob(arg0, arg1, arg2);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateNClobSRdL, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateNClobSRdL, null);
	}
	//{AUTOGEN:RSST_updateNClobSRdL:END}
	//{AUTOGEN:RSST_updateNClobSRd:BEGIN}
	public void updateNClob(String arg0, Reader arg1) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateNClobSRd,arg0, arg1);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TODO Wrap streams
			i_wrapped.updateNClob(arg0, arg1);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateNClobSRd, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateNClobSRd, null);
	}
	//{AUTOGEN:RSST_updateNClobSRd:END}
	//{AUTOGEN:RSST_updateNStringIS:BEGIN}
	public void updateNString(int columnIndex, String string) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateNStringIS,columnIndex, string);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateNString(columnIndex, string);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateNStringIS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateNStringIS, null);
	}
	//{AUTOGEN:RSST_updateNStringIS:END}
	//{AUTOGEN:RSST_updateNStringSS:BEGIN}
	public void updateNString(String columnName, String string) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateNStringSS,columnName, string);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateNString(columnName, string);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateNStringSS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateNStringSS, null);
	}
	//{AUTOGEN:RSST_updateNStringSS:END}
	//{AUTOGEN:RSST_updateNullI:BEGIN}
	public void updateNull(int columnIndex) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateNullI,columnIndex);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateNull(columnIndex);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateNullI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateNullI, null);
	}
	//{AUTOGEN:RSST_updateNullI:END}
	//{AUTOGEN:RSST_updateNullS:BEGIN}
	public void updateNull(String columnName) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateNullS,columnName);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateNull(columnName);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateNullS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateNullS, null);
	}
	//{AUTOGEN:RSST_updateNullS:END}
	//{AUTOGEN:RSST_updateObjectIOI:BEGIN}
	public void updateObject(int columnIndex, Object x, int scaleOrLength) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateObjectIOI,columnIndex, x, scaleOrLength);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateObject(columnIndex, x, scaleOrLength);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateObjectIOI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateObjectIOI, null);
	}
	//{AUTOGEN:RSST_updateObjectIOI:END}
	//{AUTOGEN:RSST_updateObjectIO:BEGIN}
	public void updateObject(int columnIndex, Object x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateObjectIO,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateObject(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateObjectIO, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateObjectIO, null);
	}
	//{AUTOGEN:RSST_updateObjectIO:END}
	//{AUTOGEN:RSST_updateObjectSOI:BEGIN}
	public void updateObject(String columnName, Object x, int scaleOrLength) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateObjectSOI,columnName, x, scaleOrLength);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateObject(columnName, x, scaleOrLength);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateObjectSOI, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateObjectSOI, null);
	}
	//{AUTOGEN:RSST_updateObjectSOI:END}
	//{AUTOGEN:RSST_updateObjectSO:BEGIN}
	public void updateObject(String columnName, Object x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateObjectSO,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateObject(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateObjectSO, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateObjectSO, null);
	}
	//{AUTOGEN:RSST_updateObjectSO:END}
	//{AUTOGEN:RSST_updateRefIRf:BEGIN}
	public void updateRef(int columnIndex, Ref x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateRefIRf,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW Is it our wrapper?
			i_wrapped.updateRef(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateRefIRf, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateRefIRf, null);
	}
	//{AUTOGEN:RSST_updateRefIRf:END}
	//{AUTOGEN:RSST_updateRefSRf:BEGIN}
	public void updateRef(String columnName, Ref x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateRefSRf,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW Is it our wrapper?
			i_wrapped.updateRef(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateRefSRf, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateRefSRf, null);
	}
	//{AUTOGEN:RSST_updateRefSRf:END}
	//{AUTOGEN:RSST_updateRow:BEGIN}
	public void updateRow() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateRow);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateRow();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateRow, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateRow, null);
	}
	//{AUTOGEN:RSST_updateRow:END}
	//{AUTOGEN:RSST_updateRowIdIRi:BEGIN}
	public void updateRowId(int columnIndex, RowId x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateRowIdIRi,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateRowId(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateRowIdIRi, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateRowIdIRi, null);
	}
	//{AUTOGEN:RSST_updateRowIdIRi:END}
	//{AUTOGEN:RSST_updateRowIdSRi:BEGIN}
	public void updateRowId(String columnName, RowId x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateRowIdSRi,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateRowId(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateRowIdSRi, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateRowIdSRi, null);
	}
	//{AUTOGEN:RSST_updateRowIdSRi:END}
	//{AUTOGEN:RSST_updateShortISh:BEGIN}
	public void updateShort(int columnIndex, short x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateShortISh,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateShort(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateShortISh, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateShortISh, null);
	}
	//{AUTOGEN:RSST_updateShortISh:END}
	//{AUTOGEN:RSST_updateShortSSh:BEGIN}
	public void updateShort(String columnName, short x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateShortSSh,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateShort(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateShortSSh, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateShortSSh, null);
	}
	//{AUTOGEN:RSST_updateShortSSh:END}
	//{AUTOGEN:RSST_updateSQLXMLISx:BEGIN}
	public void updateSQLXML(int columnIndex, SQLXML xmlObject) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateSQLXMLISx,columnIndex, xmlObject);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW Is it our wrapper?
			i_wrapped.updateSQLXML(columnIndex, xmlObject);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateSQLXMLISx, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateSQLXMLISx, null);
	}
	//{AUTOGEN:RSST_updateSQLXMLISx:END}
	//{AUTOGEN:RSST_updateSQLXMLSSx:BEGIN}
	public void updateSQLXML(String columnName, SQLXML xmlObject) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateSQLXMLSSx,columnName, xmlObject);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			// TT_LOW Is it our wrapper?
			i_wrapped.updateSQLXML(columnName, xmlObject);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateSQLXMLSSx, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateSQLXMLSSx, null);
	}
	//{AUTOGEN:RSST_updateSQLXMLSSx:END}
	//{AUTOGEN:RSST_updateStringIS:BEGIN}
	public void updateString(int columnIndex, String x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateStringIS,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateString(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateStringIS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateStringIS, null);
	}
	//{AUTOGEN:RSST_updateStringIS:END}
	//{AUTOGEN:RSST_updateStringSS:BEGIN}
	public void updateString(String columnName, String x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateStringSS,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateString(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateStringSS, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateStringSS, null);
	}
	//{AUTOGEN:RSST_updateStringSS:END}
	//{AUTOGEN:RSST_updateTimeITi:BEGIN}
	public void updateTime(int columnIndex, Time x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateTimeITi,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateTime(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateTimeITi, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateTimeITi, null);
	}
	//{AUTOGEN:RSST_updateTimeITi:END}
	//{AUTOGEN:RSST_updateTimeSTi:BEGIN}
	public void updateTime(String columnName, Time x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateTimeSTi,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateTime(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateTimeSTi, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateTimeSTi, null);
	}
	//{AUTOGEN:RSST_updateTimeSTi:END}
	//{AUTOGEN:RSST_updateTimestampITm:BEGIN}
	public void updateTimestamp(int columnIndex, Timestamp x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateTimestampITm,columnIndex, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateTimestamp(columnIndex, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateTimestampITm, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateTimestampITm, null);
	}
	//{AUTOGEN:RSST_updateTimestampITm:END}
	//{AUTOGEN:RSST_updateTimestampSTm:BEGIN}
	public void updateTimestamp(String columnName, Timestamp x) throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_updateTimestampSTm,columnName, x);
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			i_wrapped.updateTimestamp(columnName, x);
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_updateTimestampSTm, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_updateTimestampSTm, null);
	}
	//{AUTOGEN:RSST_updateTimestampSTm:END}
	//{AUTOGEN:RSST_wasNull:BEGIN}
	public boolean wasNull() throws SQLException {
		i_logger.logStart(i_objId, JdbcMethod.RSST_wasNull);
		boolean l_result = false;
		try {
			//{AUTOGEN:CUSTOM_BEGIN}
			l_result = i_wrapped.wasNull();
			//{AUTOGEN:CUSTOM_END}
		} catch (Throwable t) {
			CpUtils.handleCatchSqle(i_logger, i_objId, JdbcMethod.RSST_wasNull, t);
		}
		i_logger.logSuccess(i_objId, JdbcMethod.RSST_wasNull, l_result);
			return l_result;
	}
	//{AUTOGEN:RSST_wasNull:END}
	///{AUTOGEN:FRAME_END}
////////////////////////////////////////////////////////////////////////////

	@Override
	public boolean isWrapperFor(Class<?> iface) throws SQLException {
		// TT_LOW Some day will have to find if this is what is expected...
		if (iface.isAssignableFrom(i_wrapped.getClass())) {
			return true;
		} else {
			return i_wrapped.isWrapperFor(iface);
		}
	}

	@Override
	public <T> T unwrap(Class<T> arg0) throws SQLException {
		// TT_LOW Some day will have to find if this is what is expected...
		if (arg0.isAssignableFrom(i_wrapped.getClass())) {
			return (T)i_wrapped;
		} else {
			return i_wrapped.unwrap(arg0);
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "CpResultSet:" + i_objId;
	}

}
