/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.cpn2.log;

import java.io.IOException;
import java.util.*;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ddbc.cpn2.JdbcMethod;
import ddbc.commons.digest.DigestData;
import ddbc.commons.util.Logs;

public class CpLogger {
	private static Logger LOGGER = LoggerFactory.getLogger(CpLogger.class);
	
	public static class Context {
		private String i_connectionId;
		private ArrayDeque<DataCallStart> i_stackStarts = new ArrayDeque<DataCallStart>(2);
		public Context(String connectionId) {
			i_connectionId = connectionId;
		}
		public String getConnectionId() {
			return i_connectionId;
		}
		public void pushStart(String callerId, JdbcMethod method, Object... parameters) {
			i_stackStarts.push(new DataCallStart(callerId, method, parameters));
		}
		public DataCallStart popStart(String callerId, JdbcMethod method) {
			DataCallStart l_result = i_stackStarts.pop();
			assert l_result.getCallerId() == callerId;
			assert l_result.getMethod() == method;
			return l_result;
		}
	}

	////////////////////////////////////////////
	// Queue elements. To decouple the log processing
	//
	private static class QueuePayload {
		LogEvent event;
		CpLogger logger;
		QueuePayload(LogEvent event, CpLogger logger){
			this.event = event;
			this.logger = logger;
		}
	}
	private static Queue<QueuePayload> s_pendingEvents = new LinkedList<QueuePayload>();
	private static void queueEvent(LogEvent event, CpLogger logger) {
		synchronized(s_pendingEvents) {
			s_pendingEvents.add(new QueuePayload(event,logger));
			s_pendingEvents.notify();
		}
	}

	private static class QueueThread extends Thread {
		private boolean i_pendingFlush = false;
		@Override
		public void run() {
			while(true) {
				try {
					QueuePayload payload;
					synchronized(s_pendingEvents){
						while (s_pendingEvents.isEmpty()) {
							// Flush before sleeping (if applies)
							if (i_pendingFlush) {
								// TT_LOW Blocking operation in critical section
								for (CpLogger logger : s_standingLoggers) {
									logger.flush();
								}
								i_pendingFlush = false;
							}
							s_pendingEvents.wait();
						}
						payload = s_pendingEvents.remove();
					}
					payload.logger.process(payload.event);
				} catch(InterruptedException e) {
					// TT_LOW Decide whether some handling or recover is needed.
					return;
				} catch(Exception e) {
					// Something happended in the listener. Log and continue.
					LOGGER.error("Processing log event", e);
				}
			}
		}
	}
	private static QueueThread s_queueThread = new QueueThread();
	static {
		s_queueThread.start();
	}
	
	//////////////////////////////////////////////
	public static void registerCustomTag(String tag, Class ... parameterTypes) {
		// TODO
	}
	//////////////////////////////////////////////

	private static ArrayList<CpLogger> s_standingLoggers = new ArrayList<CpLogger>();
	
	public static void flushAll() throws IOException {
		synchronized(s_pendingEvents) {
			s_queueThread.i_pendingFlush = true;
			s_pendingEvents.notify();
		}
	}
	
	private static int eventCode(LogEventType evType, JdbcMethod method) {
		return evType.ordinal() * 1000 + method.ordinal();
	}
	
	
	// TT_LOW Allow multiple listeners
	private LogListener i_listener;
	private LoggerProfile i_profile;
	
	public CpLogger() {
		super();
		
		s_standingLoggers.add(this);
	}

	/**
	 * @return the listener
	 */
	public LogListener getListener() {
		return i_listener;
	}

	/**
	 * @return the profile
	 */
	public LoggerProfile getProfile() {
		if (i_profile == null) {
			i_profile = LoggerProfile.getInstance(LoggerProfile.PROFILE_NONE);
		}
		return i_profile;
	}

	/**
	 * @param listener the listener to set
	 */
	public void setListener(LogListener listener) {
		i_listener = listener;
	}

	/**
	 * @param profile the profile to set
	 */
	public void setProfile(LoggerProfile profile) {
		i_profile = profile;
	}

	public void flush() throws IOException {
		if (i_listener != null) {
			i_listener.flush();
		}
	}

	public void logStart(Context ctx, String srcId, JdbcMethod method, Object ... parameters) {
		// Do not log at the start of the method. Just store the data in case it is needed.
		// transform values for their logging value (make readable or ommit/hash the potentially private).
		for (int i = 0; i < parameters.length; i++) {
			parameters[i] = transformLogValue(parameters[i]);
		}
		ctx.pushStart(srcId, method, parameters);
	}

	public void logFailure(Context ctx, String srcId, JdbcMethod method, Throwable t) {
		// In case of failure will trace even if the method is not loggable.
		DataCallStart l_dataStart = ctx.popStart(srcId, method);
		LogEvent event = LogEvent.createMethodFailed(ctx, l_dataStart, t);
		queueEvent(event, this);
	}
	
	public void logSuccess(Context ctx, String srcId, JdbcMethod method, Object result) {
		DataCallStart l_dataStart = ctx.popStart(srcId, method);
		if (getProfile().acceptsMethod(method)) {
			// Introduce stacktrace into event
			StackTraceElement[] l_stackTrace = null;
			if (getProfile().getMethodAttributes(method).contains(LoggerProfile.ATT_STACK)) {
				// TT_LOW Do a smarter stacktrace filtering.
				l_stackTrace = Logs.getStackTrace(3, 5);
			}
			LogEvent event = LogEvent.createMethodEnded(ctx, l_dataStart, result, l_stackTrace);
			queueEvent(event, this);
		}
	}
	public void logCustom(Context ctx, String srcId, String tag, Object ... parameters) {
		if (getProfile().acceptsCustomTag(tag)) {
			// Introduce stacktrace into event
			StackTraceElement[] l_stackTrace = null;
			if (getProfile().getCustomTagAttributes(tag).contains(LoggerProfile.ATT_STACK)) {
				l_stackTrace = Logs.getStackTrace(2, 5);
			}
			LogEvent event = LogEvent.createCustom(ctx.getConnectionId(),
					srcId, tag, System.currentTimeMillis(), parameters, l_stackTrace);
			queueEvent(event, this);
		}
	}

	private void process(LogEvent event) {
		i_listener.process(event);
	}
	
	private Object transformLogValue(Object o) {
		if (o instanceof byte[]) {
			return DigestData.createDigestData((byte[])o);
		} else if (o instanceof CharSequence) {
			CharSequence s = (CharSequence)o;
			if (s.length() > 1024) {
				// Long enough for any statement? (don't want the sql to get garbled)
				// TT_LOW Process as inputstream
				return DigestData.createDigestData(s);
			} else {
				return s;
			}
		} else {
			return o;
		}
	}
}
