/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.cpn2.log;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ddbc.cpn2.JdbcMethod;

import ddbc.commons.util.Configs;
import ddbc.commons.util.Strings;

public class LoggerProfile {
	public static final String PROFILE_NONE = "none";
	public static final String PROFILE_GENREPORT = "genReport";
	public static final String PROFILE_FULL = "full";
	
	// TT_LOW Consider parametrized attributes (such as maximum duration).
	// TT_LOW Consider an attribute for explicitly telling overriding (so warnings can be avoided).
	// TT_LOW Consider attributes for telling attribute handling (ommit,hash).
	public static final String ATT_DISABLED = "DISABLED";
	public static final String ATT_STACK = "STACK";
	
	private static Logger LOGGER = LoggerFactory.getLogger(LoggerProfile.class);
	
	private static HashMap<String, LoggerProfile> s_instancesByName = new HashMap<String, LoggerProfile>();
	
	public static LoggerProfile getInstance(String profileName) {
		LoggerProfile l_result = s_instancesByName.get(profileName);
		if (l_result == null) {
			LOGGER.info("Loading logger profile: " + profileName);
			String l_profileLocation = "rsrc:" + profileName + ".profile";
			l_result = loadProfileFromLocation(l_profileLocation);
			s_instancesByName.put(profileName, l_result);
		}
		return l_result;
	}
	
	public static LoggerProfile loadProfileFromLocation(String profileLocation) {
		JdbcMethod[] l_acceptedMethods = JdbcMethod.values();
		LoggerProfile l_result = new LoggerProfile();
		l_result.i_acceptedMethods = new BitSet(l_acceptedMethods.length);
		l_result.i_methodAttributes = new HashMap<String, String>();
		l_result.i_customAttributes = new HashMap<String, String>();
		l_result.i_customRegexAndAttributes = new ArrayList<String>();
		
		try {
			InputStream l_input = Configs.getInputStreamFromLocation(profileLocation, LoggerProfile.class);
			BufferedReader l_breader = new BufferedReader(new InputStreamReader(l_input));
			String l_line = l_breader.readLine();
			// Group1: regex of accepted tag, Group 2 (Optional): Attributes.
			Pattern l_patternLine = Pattern.compile("([^# \t][^ \t]*)(?:\\s+(.*))?");
			// First file section: Accepted methods.
			while (l_line != null && !l_line.trim().equals("%%%")) {
				Matcher l_matcher = l_patternLine.matcher(l_line);
				if (l_matcher.matches()) {
					String l_strRegexMethod = l_matcher.group(1);
					String l_strAttributes = Strings.defaultIfEmptyOrWhiteSpace(
							l_matcher.group(2), "").trim();
					// TT_LOW Improve attribute matching (risk of false positive)
					boolean l_disabled = l_strAttributes.contains(ATT_DISABLED);
					// Search for accepted methods and add them
					Pattern l_patternMethod = Pattern.compile(l_strRegexMethod);
					// TODO Check non-matching patterns
					for (JdbcMethod l_method : l_acceptedMethods) {
						// TODO Check and warn for method collision
						String l_methodName = l_method.name();
						if (l_patternMethod.matcher(l_methodName).matches()) {
							l_result.i_acceptedMethods.set(l_method.ordinal(), !l_disabled);
							l_result.i_methodAttributes.put(l_methodName, l_strAttributes);
						}
					}
				}
				l_line = l_breader.readLine();
			}
			if (l_line.trim().equals("%%%")) {
				l_line = l_breader.readLine();
			}
			// Second file section: Accepted custom tags
			while (l_line != null) {
				Matcher l_matcher = l_patternLine.matcher(l_line);
				if (l_matcher.matches()) {
					// TODO Check and warn for tag collision
					String l_regexTag = l_matcher.group(1);
					String l_strAttributes = Strings.defaultIfEmptyOrWhiteSpace(
							l_matcher.group(2), "").trim();
					l_result.i_customRegexAndAttributes.add(l_regexTag);
					l_result.i_customRegexAndAttributes.add(l_strAttributes);
				}
				l_line = l_breader.readLine();
			}

			l_input.close();
		} catch (Throwable t) {
			LOGGER.error("Error loading profile. Using empty/incomplete setting" + profileLocation, t);
		}
		return l_result;
	}
	
	protected BitSet i_acceptedMethods;
	protected Map<String,String> i_methodAttributes;
	protected Map<String,String> i_customAttributes;
	protected ArrayList<String> i_customRegexAndAttributes;

	protected LoggerProfile() {
		super();
	}
	
	public boolean acceptsMethod(JdbcMethod method) {
		return i_acceptedMethods.get(method.ordinal());
	}
	
	public String getMethodAttributes(JdbcMethod method) {
		return i_methodAttributes.get(method.name());
	}

	public boolean acceptsCustomTag(String tag) {
		String l_attributes = getCustomTagAttributes(tag);
		// TT_LOW Improve attribute matching
		return ! ATT_DISABLED.equals(l_attributes);
	}

	public String getCustomTagAttributes(String tag) {
		String l_attributes = i_customAttributes.get(tag);
		if (l_attributes == null) {
			// Traverse the regex list in search of the attributes
			// By default
			l_attributes = ATT_DISABLED;
			Iterator<String> itRegex = i_customRegexAndAttributes.iterator();
			while(itRegex.hasNext()) {
				String l_candidateRegex = itRegex.next();
				String l_candidateAttributes = itRegex.next();
				if (Pattern.matches(l_candidateRegex, tag)) {
					l_attributes = l_candidateAttributes;
					break;
				}
			}
			i_customAttributes.put(tag, l_attributes);
		}
		return l_attributes;
	}
	
}
