/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.cpn2.log;

import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Date;

import org.apache.commons.xmlio.out.XMLWriter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ddbc.cpn2.CpUtils;

import ddbc.commons.util.FlyWeight;

public class XmlWriterListener implements LogListener {
	public static String TMS_FORMAT = "yyyyMMdd.HHmmss.SSS";
	
	/**
	 * @param frames
	 * @throws IOException
	 */
	public static void writeFrames(XMLWriter writer, StackTraceElement[] frames) throws IOException {
		if (frames != null) {
			for (StackTraceElement l_frame : frames) {
				writer.writeEmptyElement(XMLWriter.createEmptyTag("f",
						new String[][] {
								{ "class", l_frame.getClassName() },
								{ "m", l_frame.getMethodName() },
								{ "f", l_frame.getFileName() },
								{
										"l",
										Integer.toString(l_frame
												.getLineNumber()) },

						}), XMLWriter.NEWLINE);
			}
		}
	}


	private static final Logger LOGGER = LoggerFactory.getLogger(XmlWriterListener.class);
	private XMLWriter i_writer;
	
	public XmlWriterListener(Writer a_writer) {
		super();
		i_writer = new XMLWriter(a_writer);
	}
	
	@Override
	public void flush() throws IOException {
		i_writer.flush();
		
	}

	@Override
	public void process(LogEvent event) {
		try {
			String l_mainTagName = "";
			switch (event.getType()) {
			case METHOD_SUCCESS:
				l_mainTagName = "call";
				break;
			case METHOD_FAIL:
				l_mainTagName = "fail";
				break;
			case CUSTOM:
				l_mainTagName = "custom";
				break;
			case METHOD_START:
			default:
				// Unexpected
			}
			ArrayList<String[]> l_attList = new ArrayList<String[]>();
			// Write start tag
			l_attList.add(new String[] { "conn", event.getConnectionId() });
			l_attList.add(new String[] { "src", event.getSrcId() });
			if (event.getMethod() != null) {
				// TT_LOW Use code instead ??
				l_attList.add(new String[] { "m", event.getMethod().name() });
			}
			if (event.getCustomTag() != null) {
				l_attList.add(new String[] { "tag", event.getCustomTag() });
			}
			if (event.getTimeStart() != 0) {
				l_attList.add(new String[] {
						"start",
						FlyWeight.getDateFormat(TMS_FORMAT).format(
								new Date(event.getTimeStart())) });
				l_attList.add(new String[] { "d",
						Long.toString(event.getTimeEllapsed()) });
			}
			i_writer.writeStartTag(XMLWriter.createStartTag(l_mainTagName,
					l_attList.toArray(new String[l_attList.size()][])),
					XMLWriter.NEWLINE);
			// Write stacktrace
			StackTraceElement[] l_frames = event.getStackTrace();
			writeFrames(i_writer, l_frames);
			// Write parameters
			Object[] l_parameters = event.getParams();
			for (Object l_param : l_parameters) {
				i_writer.writeElementWithCData("<p>", CpUtils.toString(l_param),
						"</p>");
			}
			// Write result
			Object l_result = event.getResult();
			if (l_result != null) {
				if (l_result instanceof Throwable) {
					Throwable t = (Throwable) l_result;
					// TT_LOW write more information? (the exception should be logged somewhere else more suitable.
					i_writer.writeElementWithCData("<t>", t.toString(), "</t>");
				} else {
					i_writer.writeElementWithCData("<r>",
							CpUtils.toString(l_result),
							"</r>");
				}
			}
			i_writer.writeEndTag("</" + l_mainTagName + ">", XMLWriter.NEWLINE);
		} catch (Exception e) {
			// Allow further processing
			LOGGER.error("writing ddbc log", e);
		}
	}

}
