/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.cpn2.st;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ddbc.cpn2.CpUtils;
import ddbc.cpn2.JdbcMethod;
import ddbc.cpn2.impl.CpConnection;
import ddbc.cpn2.impl.CpPreparedStatement;
import ddbc.cpn2.impl.CpStatement;
import ddbc.commons.util.FCollections;
import ddbc.commons.util.Strings;

/**
 * Basic statement handler for non-modification operation, and as a base for extended handlers.
 * @author F.Javier Aranda (fjarandag@gmail.com)
 */
public class BaseHandler implements StatementHandler {
	protected static Logger s_logger = LoggerFactory.getLogger(BaseHandler.class);
	public static final BaseHandler INSTANCE_UNEXPECTED = new BaseHandler(
			FCollections.newHashMap("unexpected", "true"), "Unregistered statement handled:{}");
	public static final BaseHandler INSTANCE_INERT = new BaseHandler(
			FCollections.newHashMap("unexpected", "false"), "");

	public static BaseHandler getInstance(Map<String,String> attributes, String cfgText) {
		String l_strUnexpected = attributes.get("unexpected");
		boolean l_unexpected = Strings.parseBoolean(l_strUnexpected);
		return l_unexpected ?
				BaseHandler.INSTANCE_UNEXPECTED :
				BaseHandler.INSTANCE_INERT;
	}

	protected Map<String,String> i_attributes = StUtils.ATTRIBUTES_TOSTATE_INITIAL;
	protected boolean i_trace;
	protected String i_traceMessage; 
	private BaseHandler i_nextHandler;

	protected BaseHandler(Map<String,String> attributes, String traceMessage) {
		super();
		
		i_attributes = attributes;
		String l_strUnexpected = attributes.get("unexpected");
		i_trace = Strings.parseBoolean(l_strUnexpected);
		i_traceMessage = traceMessage;
	}

	public void setNext(BaseHandler nextHandler) {
		this.i_nextHandler = nextHandler;
	}
	
	public void applyStart(CpPreparedStatement ps, JdbcMethod method, String sql, Object ... moreParams) throws SQLException {
		String l_translatedSql = translateSql(ps, sql);
		if (i_nextHandler != null) {
			i_nextHandler.applyStart(ps, method, l_translatedSql, moreParams);
		} else {
			CpConnection cpConn = ps.cpConnection();
			try {
				Object[] l_invokeParams = FCollections.concatArray(Object.class, l_translatedSql, moreParams);
				PreparedStatement innerPs = (PreparedStatement)method.getMethod().invoke(cpConn.cpWrapped(), l_invokeParams);
				ps.cpSetWrapped(innerPs);
			} catch(Throwable t) {
				CpUtils.handleCatchSqle(cpConn.cpLogger(), cpConn.cpObjectId(), method, t);
			}
		}
	}
	
	public void applyParam(CpPreparedStatement ps, JdbcMethod method, int paramIdx, Object ... moreParams) throws SQLException {
		if (i_nextHandler != null) {
			i_nextHandler.applyParam(ps, method, paramIdx, moreParams);
		} else {
			try {
				Object[] l_invokeParams = FCollections.concatArray(Object.class, paramIdx, moreParams);
				method.getMethod().invoke(ps.cpWrapped(), l_invokeParams);
			} catch(Throwable t) {
				CpUtils.handleCatchSqle(ps.cpConnection().cpLogger(), ps.cpObjectId(), method, t);
			}
		}
	}
	
	public Object executePs(CpPreparedStatement ps, JdbcMethod method, Object ... moreParams) throws SQLException {
		if (i_nextHandler != null) {
			return i_nextHandler.executePs(ps, method, moreParams);
		} else {
			Object l_result = null;
			try {
				l_result = method.getMethod().invoke(ps.cpWrapped(), moreParams);
				if (l_result instanceof ResultSet) {
					l_result = wrapResultSet(ps, (ResultSet)l_result);
				}
			} catch(Throwable t) {
				CpUtils.handleCatchSqle(ps.cpConnection().cpLogger(), ps.cpObjectId(), method, t);
			}
			assert l_result != null;
			return l_result;
		}
	}
	
	public Object executeStmt(CpStatement stmt, JdbcMethod method, String sql, Object ... moreParams) throws SQLException {
		Object l_result = null;
		String l_translatedSql = translateSql(stmt, sql);
		if (i_nextHandler != null) {
			l_result = i_nextHandler.executeStmt(stmt, method, l_translatedSql, moreParams);
		} else {
			try {
				Object[] l_invokeParams = FCollections.concatArray(Object.class, l_translatedSql, moreParams);
				l_result = method.getMethod().invoke(stmt.cpWrapped(), l_invokeParams);
			} catch(Throwable t) {
				CpUtils.handleCatchSqle(stmt.cpConnection().cpLogger(), stmt.cpObjectId(), method, t);
			}
		}
		assert l_result != null;
		if (l_result instanceof ResultSet) {
			l_result = wrapResultSet(stmt, (ResultSet)l_result);
		}
		return l_result;
	}
	
	public ResultSet getResultSet(CpStatement stmt) throws SQLException {
		ResultSet l_rs = null;
		if (i_nextHandler != null) {
			l_rs = i_nextHandler.getResultSet(stmt);
		} else {
			l_rs = stmt.cpWrapped().getResultSet();
		}
		assert l_rs != null;
		return wrapResultSet(stmt, l_rs);
	}
	
	public ResultSet getGeneratedKeys(CpStatement stmt) throws SQLException {
		ResultSet l_rs = null;
		if (i_nextHandler != null) {
			l_rs = i_nextHandler.getGeneratedKeys(stmt);
		} else {
			l_rs = stmt.cpWrapped().getGeneratedKeys();
		}
		return l_rs;
	}

	protected String translateSql(CpStatement stmt, String sql) {
		// No translation so far
		// NextHandler translation concerns to next handler not to this -> Don't delegate.
		return sql;
	}
	
	protected ResultSet wrapResultSet(CpStatement stmt, ResultSet rs) {
		// No translation so far
		// NextHandler translation concerns to next handler not to this -> Don't delegate.
		return rs;
	}
	
}
