package ddbc.impl;

import java.sql.*;
import java.util.Map;
import java.util.Properties;

/**
 * Base Proxy/Decorator for the Connection interface.
 * Just works as a filter for the wrapped connection (wrapping generated objects if applies).
 * It is expected to be extended to provide some functionality.
 * @author F.Javier Aranda (fjarandag@gmail.com)
 */
public class DConnection implements Connection {
	/** The wrapped Connection, on which to delegate. */
	protected Connection i_wrapped;
	/** Factory used to create wrapper objects.
	 * By default has a factory for the base objects, which may not do
	 * anything interesting. */
	protected DWrapperFactory i_factory = DWrapperFactory.instance();

	// Default constructor, Just in case a subclass doesn't fit the informed constructor
	protected DConnection() {
		super();
	}

	/**
	 * @param wrapped
	 */
	public DConnection(Connection wrapped) {
		super();
		i_wrapped = wrapped;
	}
	
	
	
	///////////////////////////////////////////////////////
	// BOILERPLATE
	//


	///////////////////////////////////
	// FIELD ACCESORS
	//
	/**
	 * Returns the wrapped Connection object
	 */
	public Connection dGetWrappedConn() {
		return i_wrapped;
	}

	/**
	 * Returns the wrapper factory
	 * @return JpWrapperFactory this instance uses to create wrapped objects for statements and lobs.
	 */
	public DWrapperFactory dGetWrapperFactory() {
		return i_factory;
	}

	/**
	 * Sets the Wrapper Factory object
	 * @param a_factory the factory to set
	 */
	public void dSetWrapperFactory(DWrapperFactory a_factory) {
		i_factory = a_factory;
	}

	
	///////////////////////////////////
	// IMPLEMENT/DELEGATE (Statement)
	//
	/* (non-Javadoc)
	 * @see java.sql.Connection#clearWarnings()
	 */
	public void clearWarnings() throws SQLException {
		i_wrapped.clearWarnings();
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#close()
	 */
	public void close() throws SQLException {
		i_wrapped.close();
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#commit()
	 */
	public void commit() throws SQLException {
		i_wrapped.commit();
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#createArrayOf(java.lang.String, java.lang.Object[])
	 */
	public Array createArrayOf(String a_arg0, Object[] a_arg1)
			throws SQLException {
		Array l_array = i_wrapped.createArrayOf(a_arg0, a_arg1);
		return i_factory.wrapArray(l_array, this);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#createBlob()
	 */
	public Blob createBlob() throws SQLException {
		Blob wrapped = i_wrapped.createBlob();
		return i_factory.wrapBlob(wrapped, this);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#createClob()
	 */
	public Clob createClob() throws SQLException {
		Clob wrapped = i_wrapped.createClob();
		return i_factory.wrapClob(wrapped, this);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#createNClob()
	 */
	public NClob createNClob() throws SQLException {
		NClob wrapped = i_wrapped.createNClob();
		return i_factory.wrapNClob(wrapped, this);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#createSQLXML()
	 */
	public SQLXML createSQLXML() throws SQLException {
		// TODO Determinar: ignorar|impedir|adaptar
		SQLXML l_xml = i_wrapped.createSQLXML();
		return i_factory.wrapSqlXml(l_xml, this);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#createStatement()
	 */
	public Statement createStatement() throws SQLException {
		Statement wrapped = i_wrapped.createStatement();
		return i_factory.wrapStatement(wrapped, null);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#createStatement(int, int, int)
	 */
	public Statement createStatement(int a_arg0, int a_arg1, int a_arg2)
			throws SQLException {
		Statement wrapped = i_wrapped.createStatement(a_arg0, a_arg1, a_arg2);
		return i_factory.wrapStatement(wrapped, null);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#createStatement(int, int)
	 */
	public Statement createStatement(int a_arg0, int a_arg1)
			throws SQLException {
		Statement wrapped = i_wrapped.createStatement(a_arg0, a_arg1);
		return i_factory.wrapStatement(wrapped, null);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#createStruct(java.lang.String, java.lang.Object[])
	 */
	public Struct createStruct(String a_arg0, Object[] a_arg1)
			throws SQLException {
		//
		return i_wrapped.createStruct(a_arg0, a_arg1);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#getAutoCommit()
	 */
	public boolean getAutoCommit() throws SQLException {
		return i_wrapped.getAutoCommit();
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#getCatalog()
	 */
	public String getCatalog() throws SQLException {
		return i_wrapped.getCatalog();
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#getClientInfo()
	 */
	public Properties getClientInfo() throws SQLException {
		return i_wrapped.getClientInfo();
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#getClientInfo(java.lang.String)
	 */
	public String getClientInfo(String a_arg0) throws SQLException {
		return i_wrapped.getClientInfo(a_arg0);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#getHoldability()
	 */
	public int getHoldability() throws SQLException {
		return i_wrapped.getHoldability();
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#getMetaData()
	 */
	public DatabaseMetaData getMetaData() throws SQLException {
		return i_wrapped.getMetaData();
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#getTransactionIsolation()
	 */
	public int getTransactionIsolation() throws SQLException {
		return i_wrapped.getTransactionIsolation();
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#getTypeMap()
	 */
	public Map<String, Class<?>> getTypeMap() throws SQLException {
		return i_wrapped.getTypeMap();
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#getWarnings()
	 */
	public SQLWarning getWarnings() throws SQLException {
		return i_wrapped.getWarnings();
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#isClosed()
	 */
	public boolean isClosed() throws SQLException {
		return i_wrapped.isClosed();
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#isReadOnly()
	 */
	public boolean isReadOnly() throws SQLException {
		return i_wrapped.isReadOnly();
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#isValid(int)
	 */
	public boolean isValid(int a_arg0) throws SQLException {
		return i_wrapped.isValid(a_arg0);
	}
	/* (non-Javadoc)
	 * @see java.sql.Wrapper#isWrapperFor(java.lang.Class)
	 */
	public boolean isWrapperFor(Class<?> a_arg0) throws SQLException {
		return i_wrapped.isWrapperFor(a_arg0);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#nativeSQL(java.lang.String)
	 */
	public String nativeSQL(String a_arg0) throws SQLException {
		return i_wrapped.nativeSQL(a_arg0);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#prepareCall(java.lang.String, int, int, int)
	 */
	public CallableStatement prepareCall(String a_arg0, int a_arg1, int a_arg2,
			int a_arg3) throws SQLException {
		CallableStatement wrapped = i_wrapped.prepareCall(a_arg0, a_arg1, a_arg2, a_arg3);
		return i_factory.wrapCallableStatement(wrapped, null);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#prepareCall(java.lang.String, int, int)
	 */
	public CallableStatement prepareCall(String a_arg0, int a_arg1, int a_arg2)
			throws SQLException {
		CallableStatement wrapped = i_wrapped.prepareCall(a_arg0, a_arg1, a_arg2);
		return i_factory.wrapCallableStatement(wrapped, null);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#prepareCall(java.lang.String)
	 */
	public CallableStatement prepareCall(String a_arg0) throws SQLException {
		CallableStatement wrapped = i_wrapped.prepareCall(a_arg0);
		return i_factory.wrapCallableStatement(wrapped, null);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#prepareStatement(java.lang.String, int, int, int)
	 */
	public PreparedStatement prepareStatement(String a_arg0, int a_arg1,
			int a_arg2, int a_arg3) throws SQLException {
		PreparedStatement wrapped = i_wrapped.prepareStatement(a_arg0, a_arg1, a_arg2, a_arg3);
		return i_factory.wrapPreparedStatement(wrapped, this);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#prepareStatement(java.lang.String, int, int)
	 */
	public PreparedStatement prepareStatement(String a_arg0, int a_arg1,
			int a_arg2) throws SQLException {
		PreparedStatement wrapped = i_wrapped.prepareStatement(a_arg0, a_arg1, a_arg2);
		return i_factory.wrapPreparedStatement(wrapped, this);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#prepareStatement(java.lang.String, int)
	 */
	public PreparedStatement prepareStatement(String a_arg0, int a_arg1)
			throws SQLException {
		PreparedStatement wrapped = i_wrapped.prepareStatement(a_arg0, a_arg1);
		return i_factory.wrapPreparedStatement(wrapped, this);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#prepareStatement(java.lang.String, int[])
	 */
	public PreparedStatement prepareStatement(String a_arg0, int[] a_arg1)
			throws SQLException {
		PreparedStatement wrapped = i_wrapped.prepareStatement(a_arg0, a_arg1);
		return i_factory.wrapPreparedStatement(wrapped, this);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#prepareStatement(java.lang.String, java.lang.String[])
	 */
	public PreparedStatement prepareStatement(String a_arg0, String[] a_arg1)
			throws SQLException {
		PreparedStatement wrapped = i_wrapped.prepareStatement(a_arg0, a_arg1);
		return i_factory.wrapPreparedStatement(wrapped, this);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#prepareStatement(java.lang.String)
	 */
	public PreparedStatement prepareStatement(String a_arg0)
			throws SQLException {
		PreparedStatement wrapped = i_wrapped.prepareStatement(a_arg0);
		return i_factory.wrapPreparedStatement(wrapped, this);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#releaseSavepoint(java.sql.Savepoint)
	 */
	public void releaseSavepoint(Savepoint a_arg0) throws SQLException {
		i_wrapped.releaseSavepoint(a_arg0);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#rollback()
	 */
	public void rollback() throws SQLException {
		i_wrapped.rollback();
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#rollback(java.sql.Savepoint)
	 */
	public void rollback(Savepoint a_arg0) throws SQLException {
		i_wrapped.rollback(a_arg0);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#setAutoCommit(boolean)
	 */
	public void setAutoCommit(boolean a_arg0) throws SQLException {
		i_wrapped.setAutoCommit(a_arg0);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#setCatalog(java.lang.String)
	 */
	public void setCatalog(String a_arg0) throws SQLException {
		i_wrapped.setCatalog(a_arg0);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#setClientInfo(java.util.Properties)
	 */
	public void setClientInfo(Properties a_arg0) throws SQLClientInfoException {
		i_wrapped.setClientInfo(a_arg0);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#setClientInfo(java.lang.String, java.lang.String)
	 */
	public void setClientInfo(String a_arg0, String a_arg1)
			throws SQLClientInfoException {
		i_wrapped.setClientInfo(a_arg0, a_arg1);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#setHoldability(int)
	 */
	public void setHoldability(int a_arg0) throws SQLException {
		i_wrapped.setHoldability(a_arg0);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#setReadOnly(boolean)
	 */
	public void setReadOnly(boolean a_arg0) throws SQLException {
		i_wrapped.setReadOnly(a_arg0);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#setSavepoint()
	 */
	public Savepoint setSavepoint() throws SQLException {
		return i_wrapped.setSavepoint();
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#setSavepoint(java.lang.String)
	 */
	public Savepoint setSavepoint(String a_arg0) throws SQLException {
		return i_wrapped.setSavepoint(a_arg0);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#setTransactionIsolation(int)
	 */
	public void setTransactionIsolation(int a_arg0) throws SQLException {
		i_wrapped.setTransactionIsolation(a_arg0);
	}
	/* (non-Javadoc)
	 * @see java.sql.Connection#setTypeMap(java.util.Map)
	 */
	public void setTypeMap(Map<String, Class<?>> a_arg0) throws SQLException {
		i_wrapped.setTypeMap(a_arg0);
	}
	/* (non-Javadoc)
	 * @see java.sql.Wrapper#unwrap(java.lang.Class)
	 */
	public <T> T unwrap(Class<T> a_arg0) throws SQLException {
		return i_wrapped.unwrap(a_arg0);
	}
	

}
