/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.importers;

import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.sql.Driver;
import java.sql.DriverManager;
import java.util.*;

import org.apache.commons.xmlio.in.SimpleImportHandler;
import org.apache.commons.xmlio.in.SimplePath;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.helpers.AttributesImpl;

import ddbc.DDriverConnectionFactory;
import ddbc.cpn2.CpFactory;
import ddbc.cpn2.log.CpLogger;
import ddbc.cpn2.log.LoggerProfile;
import ddbc.cpn2.log.XmlWriterListener;
import ddbc.cpn2.st.HandlerMapper;
import ddbc.impl.DSimpleConnectionFactory;
import ddbc.impl.DWrapperFactory;
import ddbc.commons.util.Configs;
import ddbc.commons.util.Strings;

public class DriverImporter implements SimpleImportHandler {
	// TODO move the constants to their point of use for clarity.
	public static final String PATH_CONFIG = "/config";
	public static final String ATT_CONFIG_OVERRIDE = "override-drivers";
	
	public static final String PATH_URLMAPPING = "/config/urlMapping";

	public static final String PATH_WRAPPEDURL = "/config/urlMapping/wrappedUrl";
	public static final String ATT_WRAPPEDURL_URL = "url";
	public static final String ATT_WRAPPEDURL_DELEGATEDRIVER = "delegateDriver";

	public static final String PATH_HANDLER = "/config/urlMapping/handler";

	public static final String PATH_LOGGING = "/config/urlMapping/handler/logging";
	/** Logging profile: none or reportGen. */
	public static final String ATT_LOGGING_PROFILE = "profile";
/*
	public static enum LoggingProfiles {
		NONE("none"),
		REPORTGEN("reportGen");
		static LoggingProfiles getByName(String name) {
			if ("reportGen".equals(name)) return REPORTGEN; else return NONE;
		}
		private String name;
		LoggingProfiles(String name) {this.name = name;}
		public String getName() {return this.name; }
	}
*/
	public static final String PATH_LOGLISTENER = "/config/urlMapping/handler/logging/logListener";
	public static final String ATT_LOGLISTENER_FILE = "file";

	public static final String PATH_DAI = "/config/urlMapping/handler/dai";
	public static final String ATT_DAI_EMULATEDVER = "url";

	public static final String ATT_COMMON_ID = "id";
	public static final String ATT_COMMON_REF = "ref";
	public static final String ATT_COMMON_TYPE = "type";

	private static Logger LOGGER = LoggerFactory.getLogger(DriverImporter.class);
	
	private static HashMap<String,Driver> s_registeredDrivers = new HashMap<String, Driver>();
	private static Driver getDriverForName(String className) {
		Driver l_result = s_registeredDrivers.get(className);
		if (l_result == null) {
			try {
				Class<?> l_driverClass = Class.forName(className);
				Enumeration<Driver>drivers = DriverManager.getDrivers();
				while (drivers.hasMoreElements()) {
					Driver l_driver = drivers.nextElement();
					if (l_driverClass.equals(l_driver.getClass())) {
						l_result = l_driver;
						s_registeredDrivers.put(className, l_driver);
					}
				}
			} catch(Exception e) {
				LOGGER.error("Loading driver " + className, e);
				// Don't propagate. Hope it isn't important.
			}
		}
		return l_result;
	}

	private Map<String,String> i_importProperties = new HashMap<String, String>();

	private boolean i_overridesUrls = false;
	private Map<String, DDriverConnectionFactory> i_connFactoriesByUrl =
			new HashMap<String, DDriverConnectionFactory>();
//	private List<Pattern> s_urlPatterns = new ArrayList<Pattern>();
//	private List<Transformer> s_connMetaFactories = new ArrayList<Transformer>();
	
	// Temporal elements in the import process:
	private String tmp_mappingUrl;
	private Driver tmp_delegateDriver;
	private List<DWrapperFactory> tmp_factories = new ArrayList<DWrapperFactory>();
	private CpFactory tmp_factory;
//	private CpLogger tmp_logger;
	
	public void startElement(SimplePath path, String name,
			AttributesImpl attributes, String leadingCDdata) {
		try {
			if (path.matches(PATH_CONFIG)) {
				// No initialization
			} else if (path.matches(PATH_URLMAPPING)) {
				// Start a new mapping
				tmp_mappingUrl = null;
				tmp_factories.clear();

				// Pending finalization on element's end
			} else if (path.matches(PATH_WRAPPEDURL)) {
				tmp_mappingUrl = attributes.getValue(ATT_WRAPPEDURL_URL);
				String strDriver = attributes
						.getValue(ATT_WRAPPEDURL_DELEGATEDRIVER);
				if (!Strings.isEmpty(strDriver)) {
					tmp_delegateDriver = getDriverForName(strDriver);
				}
			} else if (path.matches(PATH_HANDLER)) {
				// TODO Delegate load in function of the handler type.
				// At the moment only cpn2 is needed.
				tmp_factory = new CpFactory();
				// Pending finalization on element's end
			} else if (path.matches(PATH_LOGGING)) {
				String strProfile = attributes.getValue(ATT_LOGGING_PROFILE);
				CpLogger l_logger = new CpLogger();
				LoggerProfile l_profile = LoggerProfile.getInstance(strProfile);
				l_logger.setProfile(l_profile);
				tmp_factory.setLogger(l_logger);
			} else if (path.matches(PATH_LOGLISTENER)) {
				String strListenerFile = attributes.getValue(ATT_LOGLISTENER_FILE);
				strListenerFile = Strings.replaceStrVariables(strListenerFile, i_importProperties);
				String strListenerType = attributes.getValue(ATT_COMMON_TYPE);
				try {
					LOGGER.info("Loading log listener of type {} into file {}",
							strListenerType, strListenerFile);
					FileOutputStream l_fos = new FileOutputStream(
							strListenerFile, true);
					OutputStreamWriter l_osw = new OutputStreamWriter(l_fos,
							"UTF-8");
					XmlWriterListener l_listener = new XmlWriterListener(l_osw);
					tmp_factory.getLogger().setListener(l_listener);
				} catch (Exception e) {
					LOGGER.error("Could not load log listener", e);
				}
			} else if (path.matches(PATH_DAI)) {
				String l_daiSrc = attributes.getValue("src");
				l_daiSrc = Strings.replaceStrVariables(l_daiSrc, i_importProperties);
				try {
					LOGGER.info("Loading traslation definition from "
							+ l_daiSrc);
					MapperImporter l_mapperImporter = new MapperImporter();
					Configs.parseXmlIoFromLocation(l_daiSrc, l_mapperImporter);
					HandlerMapper l_mapper = l_mapperImporter.getBuilt();
					tmp_factory.setMapper(l_mapper);
				} catch (Exception e) {
					LOGGER.error("Could not load sql translator", e);
				}

				// TODO Delegates for increased features.
			}
		} catch (Exception e) {
			// Allow further processing
			LOGGER.error("loading driver element " + path + attributes, e);
		}
	}

	public void endElement(SimplePath path, String name) {
		try {
			if (path.matches(PATH_CONFIG)) {

			} else if (path.matches(PATH_URLMAPPING)) {
				DDriverConnectionFactory factory = buildFactoryChain();
				i_connFactoriesByUrl.put(tmp_mappingUrl, factory);
			} else if (path.matches(PATH_WRAPPEDURL)) {

			} else if (path.matches(PATH_HANDLER)) {
				if (tmp_factory.getLogger() == null) {
					tmp_factory.setLogger(new CpLogger());
				}
				if (tmp_factory.getMapper() == null) {
					tmp_factory.setMapper(HandlerMapper.dumbInstance());
				}
				tmp_factories.add(tmp_factory);
				tmp_factory = null;
			} else if (path.matches(PATH_LOGGING)) {
				// CpLogger already set
			} else if (path.matches(PATH_LOGLISTENER)) {
				// LogListener already set
			} else if (path.matches(PATH_DAI)) {
				// HandlerMapper already set
			}
		} catch (Exception e) {
			// Allow further processing
			LOGGER.error("loading driver element " + path, e);
		}
	}

	public void cData(SimplePath a_path, String a_cdata) {}

	public void startDocument() {}

	public void endDocument() {}

	private DDriverConnectionFactory buildFactoryChain() {
		// TT_LOW Consider a chain without an standard backend
		DDriverConnectionFactory tmpFactory = DSimpleConnectionFactory.createBackEnd(tmp_delegateDriver, null);
		// Wrap from last to first to make the invocation chain be in order.
		for (int i = tmp_factories.size() - 1; i >= 0; i--) {
			tmpFactory = tmp_factories.get(i).wrapConnectionFactory(tmpFactory);
		}
		return tmpFactory;
	}
	
	/**
	 * @return the s_connFactoriesByUrl
	 */
	public Map<String, DDriverConnectionFactory> getConnFactoriesByUrl() {
		return i_connFactoriesByUrl;
	}

	/**
	 * @return the overridesUrls
	 */
	public boolean getOverridesUrls() {
		return i_overridesUrls;
	}

	public Map<String, String> getImportProperties() {
		return i_importProperties;
	}

	public void setImportProperties(Map<String, String> a_importProperties) {
		i_importProperties.clear();
		if (a_importProperties != null) {
			i_importProperties.putAll(a_importProperties);
		}
	}

}
