/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.importers;

import java.util.Map;

import org.apache.commons.xmlio.in.SimplePath;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.helpers.AttributesImpl;

import ddbc.commons.util.FactoryUtils;
import ddbc.commons.util.FactoryUtils.FactoryImporter;

/**
 * Implementation of import handler for element types which are configured only with attributes and a simple element text.
 * In order to serve for different types, the element class (and possibly the factory method) must be specified.
 * A class name is required.
 * In case a method is specified, it is assumed that exists a static factory method with that name and the required signature (Map attributes, String text).
 * If no method is specified, it is automaticaly tried to use one of the following:
 * - A static factory method with the name "getInstance" and the complete signature.
 * - A constructor with the complete signature.
 * - A default constructor.
 * @author F.Javier Aranda (fjarandag@gmail.com)
 *
 */
public class ImporterSimple<T> implements NestedImportHandler<T> {
	private static Logger s_logger = LoggerFactory.getLogger(ImporterSimple.class);
	// determined on contruction/init
	private FactoryImporter i_factory = null;
	// initialized on #start invocation
	private String i_basePath;
	private T i_built;

	/**
	 * Constructs without field initialization.
	 * Left protected, but use in inherited classes at your own risk.
	 */
	protected ImporterSimple() {
		super();
	}
	
	/**
	 * Creates a simple importer for an element type. The importer configuration
	 *   is a list of attributes and the contained text (just like the imported elements).
	 *   <b>class</b> attribute contains the imported element qualified class name.
	 *   <b>method</b> attribute (optional) contains the factory method name (if applies)
	 * @param attributes Importer/type definition attributes, converted to Map<String,String>
	 * @param configStr Importer/type definition text. Currently without use.
	 */
	public ImporterSimple(Map<String,String> attributes, String configStr) throws NoSuchMethodException {
		super();
		
		init(attributes, configStr);
	}

	/**
	 * Initialization method used in the constructor ({@link #JpamCustomSimpleImportHandler(Map, String)}.
	 * Left protected, but redefine at your own risk.
	 * @param attributes Attributes as supplied to the constructor.
	 * @param configStr Element text as supplied to the constructor.
	 */
	protected void init(Map<String, String> attributes, String configStr) throws NoSuchMethodException {
		// Search for the factory method or constructor.
		String l_strClass = attributes.get("class");
		String l_strMethod = attributes.get("method");
		i_factory = FactoryUtils.createFactoryData(l_strClass, l_strMethod, "getInstance");
	}
	
	public void start(ImportContext context, String basePath) {
		i_basePath = basePath;
		// Reset, as the importer might be reused.
		i_built = null;
	}

	public T getBuilt() {
		return i_built;
	}

	public void startElement(SimplePath path, String name,
			AttributesImpl attributes, String leadingCDdata) {
		if (path.matches(i_basePath)) {
			try {
				i_built = i_factory.create(attributes, leadingCDdata);
			} catch (Exception e) {
				// If we propagate an exception, the whole loading may fail.
				// Better to trace, return null, and let invoker to decide.
				i_built = null;
				s_logger.error("Could not create instance", e);
			}
		} else {
			// No elements expected inside a simple element.
			// Importer should be de-attached after processing element.
			s_logger.warn("Found path {} when expecting {}", path.toString(), i_basePath);
		}
	}

	public void cData(SimplePath a_path, String a_cdata) {
		// Nothing to do
	}
	public void endElement(SimplePath a_path, String a_name) {
		// Nothing to do
	}

	public void startDocument() {
		// Unexpected
	}
	public void endDocument() {
		// Unexpected
	}
}
