/*
 * ======================================================================== DDBC : Free for Java(tm) platform
 * ========================================================================
 * 
 * (C) Copyright 2010, by Fco.Javier Aranda.
 * 
 * Project Info: http://www.gorrion.info/ddbc/
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * 
 * [Java is a trademark or registered trademark of Oracle Corporation, Inc. in the United States and other countries.]
 */
package ddbc.importers;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.xmlio.in.SimplePath;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.helpers.AttributesImpl;

import ddbc.cpn2.st.*;
import ddbc.commons.util.FCollections;
import ddbc.commons.util.FactoryUtils;
import ddbc.commons.util.XmlUtils;
import ddbc.commons.util.FactoryUtils.FactoryImporter;

public class MapperImporter implements NestedImportHandler {
	private static Logger LOGGER = LoggerFactory.getLogger(MapperImporter.class);
	
	private String i_basePath;
	private HandlerMapper i_built;
	private ImportContext i_context;
	private NestedImportHandler<?> i_currentDelegate = null;
	private MappingElement i_currentMappingElement = null;
	private StatementHandler i_currentMappingHandler;
	private String i_currentMappingId;
	private StatementMatcher i_currentMappingMatcher;
	private Map<String,String> i_mapAttributes;

	protected static final Map<String, NestedImportHandler<?>>
			BUILTIN_TYPES_IMPORTERS = createBuiltinTypesImporters();

	public MapperImporter() {
		super();
		start(new ImportContext(), "/adapt-spec");
	}
	/* (non-Javadoc)
	 * @see fjarandag.jdbcproxy.adapt.mapped.JpamDelegateSimpleImportHandler#start(java.lang.Object, java.lang.String)
	 */
	public void start(ImportContext context, String basePath) {
		i_context = context;
		i_basePath = basePath;
		i_built = new HandlerMapper();
	}

	/* (non-Javadoc)
	 * @see org.apache.commons.xmlio.in.SimpleImportHandler#startElement(org.apache.commons.xmlio.in.SimplePath, java.lang.String, org.xml.sax.helpers.AttributesImpl, java.lang.String)
	 */
	public void startElement(SimplePath path, String name,
			AttributesImpl attributes, String leadingCDdata) {
		try {
			if (path.matches(i_basePath + "/state")) {
				String l_state = attributes.getValue("id");
				i_context.declaredStates().add(l_state);
			} else if (path.matches(i_basePath + "/def")) {
				i_mapAttributes = XmlUtils.attributesToMap(attributes);
				String l_builder = i_mapAttributes.get("builder");
				String l_type = i_mapAttributes.get("type");
				if ("customSimple".equals(l_builder)) {
					NestedImportHandler<?> l_importer;
					try {
						l_importer = new ImporterSimple(i_mapAttributes,
								leadingCDdata);
						i_context.typesByName().put(l_type, l_importer);
					} catch (NoSuchMethodException e) {
						LOGGER.error("Bad customSimple " + l_type, e);
					}
				} else if ("customImporter".equals(l_builder)) {
					// An specific importer is implemented for parsing
					//   the instances elements.
					String l_strClass = i_mapAttributes.get("class");
					String l_strMethod = i_mapAttributes.get("method");
					try {
						FactoryImporter l_factory = FactoryUtils
								.createFactoryData(l_strClass, l_strMethod,
										"getImporter");
						NestedImportHandler<?> l_importer = l_factory.create(
								attributes, leadingCDdata);
						i_context.typesByName().put(l_type, l_importer);
					} catch (Exception e) {
						LOGGER.error("Bad customImporter " + l_type, e);
					}
					i_currentDelegate.start(i_context, path.toString());
				} else if ("customMeta".equals(l_builder)) {
					// A meta-importer is an importer for the instance importer.
					// We need it when the type definition has a complex configuration.
					// I hope we never need it.
					String l_metaClass = i_mapAttributes.get("metaClass");
					String l_metaMethod = i_mapAttributes.get("metaMethod");
					try {
						FactoryImporter l_factory = FactoryUtils
								.createFactoryData(l_metaClass, l_metaMethod,
										"getMetaImporter");
						i_currentDelegate = l_factory.create(attributes,
								leadingCDdata);
						i_currentDelegate.start(i_context, path.toString());
					} catch (Exception e) {
						LOGGER.error("Bad customMeta " + l_type, e);
					}
				} else {
					// TODO Notify bad definition
				}
			} else if (path.matches(i_basePath + "/mapping")) {
				i_currentMappingElement = new MappingElement(XmlUtils
						.attributesToMap(attributes), leadingCDdata);
				// TODO Set the id and other attributes. Mapping may be initialized as attributes+string.
				i_currentMappingId = attributes.getValue("id");
				// Remaining process at the end of the element...
			} else if (path.matches(i_basePath + "/mapping/matcher")) {
				i_mapAttributes = XmlUtils.attributesToMap(attributes);
				// matcher might been already defined ("ref")
				i_currentMappingMatcher = i_context
						.referencedComponent(i_mapAttributes);
				if (i_currentMappingMatcher == null) {
					// Delegate on importer ("type")
					String l_type = i_mapAttributes.get("type");
					i_currentDelegate = i_context.typesByName().get(l_type);
					assert i_currentDelegate != null : "Not found type "
							+ l_type;
					i_currentDelegate.start(i_context, path.toString());
				}
				// Remaining process at the end of the element...
			} else if (path.matches(i_basePath + "/mapping/handler")
					|| path.matches(i_basePath + "/otherwise/handler")) {
				i_mapAttributes = XmlUtils.attributesToMap(attributes);
				// handler might been already defined ("ref")
				i_currentMappingHandler = i_context
						.referencedComponent(i_mapAttributes);
				if (i_currentMappingHandler == null) {
					// Delegate on importer ("type")
					String l_type = i_mapAttributes.get("type");
					i_currentDelegate = i_context.typesByName().get(l_type);
					assert i_currentDelegate != null : "Not found type "
							+ l_type;
					i_currentDelegate.start(i_context, path.toString());
				}
				// Remaining process at the end of the element...
			}
			// Delegate if applies
			if (i_currentDelegate != null) {
				i_currentDelegate.startElement(path, name, attributes,
						leadingCDdata);
			}
		} catch (Exception e) {
			// Allow further processing
			LOGGER.error("loading mapping element " + path + attributes, e);
		}
	}

	/* (non-Javadoc)
	 * @see org.apache.commons.xmlio.in.SimpleImportHandler#endElement(org.apache.commons.xmlio.in.SimplePath, java.lang.String)
	 */
	public void endElement(SimplePath path, String name) {
		try {
			// Delegate if applies (before the delegate might be removed).
			if (i_currentDelegate != null) {
				i_currentDelegate.endElement(path, name);
			}
			if (path.matches(i_basePath + "/def")) {
				// Only in case a meta-importer was operating, it will be de-attached.
				if (i_currentDelegate != null) {
					NestedImportHandler<?> l_importer = (NestedImportHandler<?>) i_currentDelegate
							.getBuilt();
					i_context.typesByName().put(i_mapAttributes.get("type"),
							l_importer);
					i_currentDelegate = null;
				}
			} else if (path.matches(i_basePath + "/mapping")) {
				// TODO Register the mapping id (for debug information or whatever)
				i_currentMappingElement.setMatcher(i_currentMappingMatcher);
				i_currentMappingElement.setHandler(i_currentMappingHandler);
				i_built.addMapping(i_currentMappingElement);
				i_currentMappingMatcher = null;
				i_currentMappingHandler = null;
				i_currentMappingElement = null;
			} else if (path.matches(i_basePath + "/mapping/matcher")) {
				// If the component was not given by "ref", it has been built by the delegate.
				if (i_currentDelegate != null) { // Being built by the delegate
					i_currentMappingMatcher = (StatementMatcher) i_currentDelegate
							.getBuilt();
					i_currentDelegate = null;
					i_context.registerComponent(i_mapAttributes,
							i_currentMappingMatcher);
				}
			} else if (path.matches(i_basePath + "/mapping/handler")
					|| path.matches(i_basePath + "/otherwise/handler")) {
				// If the component was not given by "ref", it has been built by the delegate.
				if (i_currentDelegate != null) { // Being built by the delegate
					i_currentMappingHandler = (StatementHandler) i_currentDelegate
							.getBuilt();
					i_currentDelegate = null;
					i_context.registerComponent(i_mapAttributes,
							i_currentMappingHandler);
				}
			} else if (path.matches(i_basePath + "/otherwise")) {
				i_built.setDefaultHandler(i_currentMappingHandler);
				i_currentMappingHandler = null;
			}
		} catch (Exception e) {
			// Allow further processing
			LOGGER.error("loading mapping element " + path, e);
		}
	}

	/* (non-Javadoc)
	 * @see org.apache.commons.xmlio.in.SimpleImportHandler#cData(org.apache.commons.xmlio.in.SimplePath, java.lang.String)
	 */
	public void cData(SimplePath path, String cdata) {
		// Delegate if applies
		if (i_currentDelegate != null) {
			i_currentDelegate.cData(path, cdata);
		}
	}

	/* (non-Javadoc)
	 * @see org.apache.commons.xmlio.in.SimpleImportHandler#startDocument()
	 */
	public void startDocument() {
	}
	
	/* (non-Javadoc)
	 * @see org.apache.commons.xmlio.in.SimpleImportHandler#endDocument()
	 */
	public void endDocument() {
	}

	/* (non-Javadoc)
	 * @see fjarandag.jdbcproxy.adapt.mapped.JpamDelegateSimpleImportHandler#getBuilt()
	 */
	public HandlerMapper getBuilt() {
		return i_built;
	}

	private static <T> ImporterSimple<T> newSimpleHandler(String a_type, Class<T> a_class) throws NoSuchMethodException {
		return new ImporterSimple<T>(
				FCollections.newHashMap("type", a_type, "class", a_class.getName()),
				null);
	}

	private static Map<String, NestedImportHandler<?>> createBuiltinTypesImporters() {
		HashMap<String, NestedImportHandler<?>> l_map = 
			new HashMap<String, NestedImportHandler<?>>();
	
		try {
			l_map.put("equals", newSimpleHandler("equals", MatcherEquals.class));
			l_map.put("rxMatch", newSimpleHandler("rxMatch", MatcherRegex.class));
			l_map.put("literal", newSimpleHandler("literal", HandlerLiteral.class));
			l_map.put("fail", newSimpleHandler("fail", HandlerFail.class));
			l_map.put("passThrough", newSimpleHandler("passThrough", BaseHandler.class)); // TODO
			l_map.put("rxHandler", new ImporterHandlerRegex());
	
			// TODO More import handlers for scripting, peacock (Complex N1), etc.
			
			return Collections.unmodifiableMap(l_map);
		} catch (NoSuchMethodException e) {
			// Initialization failed, propagate as Runtime. Because this is invoked at
			//   class initialization, it will cause a failure that the class user will have
			//   to handle.
			LOGGER.error("In initialization ", e);
			throw new RuntimeException(e);
		}
	}

}
